searchData={"items":[{"type":"module","title":"TypeCheck","doc":"Fast and flexible runtime type-checking.\n\nThe main way to use TypeCheck is by adding `use TypeCheck` in your modules.\nThis will allow you to use the macros of `TypeCheck.Macros` in your module,\nwhich are versions of the normal type-specification module attributes\nwith an extra exclamation mark at the end:  `@type!`, `@spec!`, `@typep!` and `@opaque!`.\n\n\nIt will also bring all functions in `TypeCheck.Builtin` in scope,\nwhich is usually what you want as this allows you to\nuse all types and special syntax that are built-in to Elixir\nin the TypeCheck specifications.\n\n\nUsing these, you're able to add function-specifications to your functions\nwhich will wrap them with runtime type-checks.\nYou'll also be able to create your own type-specifications that can be used\nin other type- and function-specifications in the same or other modules later on:\n\n    defmodule User do\n      use TypeCheck\n      defstruct [:name, :age]\n      @type! age :: non_neg_integer()\n      @type! t :: %User{name: binary(), age: age()}\n\n      @spec! new(binary(), age()) :: t()\n      def new(name, age) do\n        %User{name: name, age: age}\n      end\n\n      @spec! old_enough?(t(), age()) :: boolean()\n      def old_enough?(user, limit) do\n        user.age >= limit\n      end\n    end\n\nFinally, you can test whether your functions correctly adhere to their specs,\nby adding a `spectest` in your testing suite. See `TypeCheck.ExUnit.spectest/2` for details.","ref":"TypeCheck.html"},{"type":"module","title":"Types and their syntax - TypeCheck","doc":"TypeCheck allows types written using (essentially) the same syntax as [Elixir's builtin typespecs](https://hexdocs.pm/elixir/typespecs.html#types-and-their-syntax).\nThis means the following:\n\n- literal values like `:ok`, `10.0` or `\"my string\"` desugar to a call to `TypeCheck.Builtin.literal/1`, which is a type that matches only exactly that value.\n- Basic types like `integer()`, `float()`, `atom()` etc. are directly supported (and exist as functions in `TypeCheck.Builtin`).\n- tuples of types like `{atom(), integer()}` are supported (and desugar to `TypeCheck.Builtin.fixed_tuple/1`)\n- maps where keys are literal values and the values are types like `%{a: integer(), b: integer(), 42 => float()}` desugar to calls to `TypeCheck.Builtin.fixed_map/1`.\n  - The same happens with structs like `%User{name: binary(), age: non_neg_integer()}`\n- sum types like `integer() | string() | atom()` are supported, and desugar to calls to `TypeCheck.Builtin.one_of/1`.\n- Ranges like `lower..higher` are supported, matching integers within the given range. This desugars into a call to `TypeCheck.Builtin.range/1`.","ref":"TypeCheck.html#module-types-and-their-syntax"},{"type":"module","title":"Currently unsupported features - TypeCheck","doc":"The following typespec syntax can _not_ currently be used in TypeCheck. This will hopefully change in future versions of the library.\n\n- Literal maps with `required(...)` and `optional(...)` keys. (TypeCheck does already support literal maps with a fixed set of keys, as well as maps with any number of key-value-pairs of fixed types. It is the special syntax that might mix these approaches that is not supported yet.)","ref":"TypeCheck.html#module-currently-unsupported-features"},{"type":"module","title":"Extensions - TypeCheck","doc":"TypeCheck adds the following extensions on Elixir's builtin typespec syntax:\n\n\n- fixed-size lists containing types like `[1, 2, integer()]` are supported, and desugar to `TypeCheck.Builtin.fixed_list/1`.\n  This example matches only lists of 3 elements where the first element is the literal `1`, the second the literal `2` and the last element any integer.\n  Elixir's builtin typespecs do not support fixed-size lists.\n- named types like `x :: integer()` are supported; these are useful in combination with \"type guards\" (see the section below).\n- \"type guards\" using the syntax `some_type when arbitrary_code` are supported, to add extra arbitrary checks to a value for it to match the type. (See the section about type guards below.)\n- `lazy(some_type)`, which defers type-expansion until during runtime. This is required to be able to expand recursive types. C.f. `TypeCheck.Builtin.lazy/1`","ref":"TypeCheck.html#module-extensions"},{"type":"module","title":"Named Types Type Guards - TypeCheck","doc":"To add extra custom checks to a type, you can use a so-called 'type guard'.\nThis is arbitrary code that is executed during a type-check once the type itself already matches.\n\nYou can use \"named types\" to refer to (parts of) the value that matched the type, and refer to these from a type-guard:\n\n```\ntype sorted_pair :: {lower :: number(), higher :: number()} when lower <= higher\n```\n\n    iex> TypeCheck.conforms!({10, 20}, sorted_pair)\n    {10, 20}\n    iex> TypeCheck.conforms!({20, 10}, sorted_pair)\n    ** (TypeCheck.TypeError) `{20, 10}` does not match the definition of the named type `TypeCheckTest.TypeGuardExample.sorted_pair`\n        which is: `TypeCheckTest.TypeGuardExample.sorted_pair\n        ::\n        (sorted_pair :: {lower :: number(), higher :: number()} when lower <= higher)`. Reason:\n          `{20, 10}` does not check against `(sorted_pair :: {lower :: number(), higher :: number()} when lower <= higher)`. Reason:\n            type guard:\n              `lower <= higher` evaluated to false or nil.\n              bound values: %{higher: 10, lower: 20, sorted_pair: {20, 10}}\n\nNamed types are available in your guard even from the (both local and remote) types that you are using in your time, as long as those types are not defined as _opaque_ types.","ref":"TypeCheck.html#module-named-types-type-guards"},{"type":"module","title":"Manual type-checking - TypeCheck","doc":"If you want to check values against a type _outside_ of the checks the `@spec!` macro\nwraps a function with,\nyou can use the `conforms/2`/`conforms?/2`/`conforms!/2` macros in this module directly in your code.\n\nThese are evaluated _at compile time_ which means the resulting checks will be optimized by the compiler.\nUnfortunately it also means that the types passed to them have to be known at compile time.\n\nIf you have a type that is constructed dynamically at runtime, you can resort to\n`dynamic_conforms/2` and variants.\nBecause these variants have to evaluate the type-checking code at runtime,\nthese checks are not optimized by the compiler.","ref":"TypeCheck.html#module-manual-type-checking"},{"type":"module","title":"Introspection - TypeCheck","doc":"To allow checking what types and specs exist,\nthe introspection function `__type_check__/1` will be added to a module when `use TypeCheck` is used.\n\n- `YourModule.__type_check__(:types)` returns a keyword list of all `{type_name, arity}`\n  pairs for all types defined in the module using `@type!`/`@typep!` or `@opaque!`.\n- `YourModule.__type_check__(:specs)` returns a keyword list of all `{function_name, arity}`\n  pairs for all functions in the module wrapped with a spec using `@spec!`.\n\nNote that these lists might also contain private types / private function names.","ref":"TypeCheck.html#module-introspection"},{"type":"macro","title":"TypeCheck.conforms/3","doc":"Makes sure `value` typechecks the type description `type`.\n\nIf it typechecks, we return `{:ok, value}`\nOtherwise, we return `{:error, %TypeCheck.TypeError{}}` which contains information\nabout why the value failed the check.\n\n`conforms` is a macro and expands the type check at compile-time,\nallowing it to be optimized by the compiler.\n\nC.f. `TypeCheck.Type.build/1` for more information on what type-expressions\nare allowed as `type` parameter.\n\nNote: _usually_ you'll want to `import TypeCheck.Builtin` in the context where you use `conforms`,\nwhich will bring Elixir's builtin types into scope.\n(Calling `use TypeCheck` will already do this; see the module documentation of `TypeCheck` for more information))","ref":"TypeCheck.html#conforms/3"},{"type":"macro","title":"TypeCheck.conforms!/3","doc":"Similar to `conforms/2`, but returns `value` if the value typechecked and raises TypeCheck.TypeError if it did not.\n\nThe same features and restrictions apply to this function as to `conforms/2`.","ref":"TypeCheck.html#conforms!/3"},{"type":"macro","title":"TypeCheck.conforms?/3","doc":"Similar to `conforms/2`, but returns `true` if the value typechecked and `false` if it did not.\n\nThe same features and restrictions apply to this function as to `conforms/2`.","ref":"TypeCheck.html#conforms?/3"},{"type":"function","title":"TypeCheck.dynamic_conforms/3","doc":"Makes sure `value` typechecks the type `type`. Evaluated _at runtime_.\n\nBecause `dynamic_conforms/2` is evaluated at runtime:\n\n1. The typecheck cannot be optimized by the compiler, which makes it slower.\n\n2. You must pass an already-expanded type as `type`.\n   This can be done by using one of your custom types directly (e.g. `YourModule.typename()`),\n   or by calling `TypeCheck.Type.build`.\n\nUse `dynamic_conforms` only when you cannot use the normal `conforms/2`,\nfor instance when you're only able to construct the type to check against at runtime.\n\n    iex> forty_two = TypeCheck.Type.build(42)\n    iex> TypeCheck.dynamic_conforms(42, forty_two)\n    {:ok, 42}\n    iex> {:error, type_error} = TypeCheck.dynamic_conforms(20, forty_two)\n    iex> type_error.message =~ \"`20` is not the same value as `42`.\"\n    true","ref":"TypeCheck.html#dynamic_conforms/3"},{"type":"function","title":"TypeCheck.dynamic_conforms!/3","doc":"Similar to `dynamic_conforms/2`, but returns `value` if the value typechecked and raises TypeCheck.TypeError if it did not.\n\nThe same features and restrictions apply to this function as to `dynamic_conforms/2`.\n\n    iex> forty_two = TypeCheck.Type.build(42)\n    iex> TypeCheck.dynamic_conforms!(42, forty_two)\n    42\n    iex> TypeCheck.dynamic_conforms!(20, forty_two)\n    ** (TypeCheck.TypeError) At lib/type_check.ex:362:\n        `20` is not the same value as `42`.","ref":"TypeCheck.html#dynamic_conforms!/3"},{"type":"function","title":"TypeCheck.dynamic_conforms?/3","doc":"Similar to `dynamic_conforms/2`, but returns `true` if the value typechecked and `false` if it did not.\n\nThe same features and restrictions apply to this function as to `dynamic_conforms/2`.\n\n\n    iex> forty_two = TypeCheck.Type.build(42)\n    iex> TypeCheck.dynamic_conforms?(42, forty_two)\n    true\n    iex> TypeCheck.dynamic_conforms?(20, forty_two)\n    false","ref":"TypeCheck.html#dynamic_conforms?/3"},{"type":"type","title":"TypeCheck.value/0","doc":"","ref":"TypeCheck.html#t:value/0"},{"type":"module","title":"TypeCheck.Defstruct","doc":"Implements the `defstruct!` macro.\n\nTo use this macro:\n- Ensure you `use TypeCheck` in your module\n- Also `use TypeCheck.Defstruct`\n\nAnd now call `defstruct!/1` when you want to define a struct.","ref":"TypeCheck.Defstruct.html"},{"type":"macro","title":"TypeCheck.Defstruct.defstruct!/1","doc":"Defines a struct and a TypeCheck type at the same time.\n\n\n# Example:\n\n    defmodule User do\n      use TypeCheck\n      use TypeCheck.Defstruct\n\n      defstruct!(\n        name: \"Guest\" :: String.t(),\n        age: _ :: non_neg_integer()\n      )\n    end\n\nThis is syntactic sugar for:\n\n    defmodule User do\n      use TypeCheck\n      use TypeCheck.Defstruct\n\n      @type! t() :: %User{\n        name: String.t(),\n        age: non_neg_integer()\n      }\n      @enforce_keys [:age]\n      defstruct [:age, name: \"Guest\"]\n    end","ref":"TypeCheck.Defstruct.html#defstruct!/1"},{"type":"macro","title":"Optional and required keys - TypeCheck.Defstruct.defstruct!/1","doc":"A key is considered optional if it uses the syntax\n\n    name: default_value :: type\n\nA key is considered required if it uses one of the following syntaxes:\n\n    :name :: type\n\n    name: _ :: type\n\nIn this case, it will be added to the `@enforce_keys` list. (c.f. `Kernel.defstruct`).","ref":"TypeCheck.Defstruct.html#defstruct!/1-optional-and-required-keys"},{"type":"module","title":"TypeCheck.ExUnit","doc":"Provides macros for 'spectests': spec-automated property-testing.\n\nThe core macro exposed by this module is `spectest/2`, which\nwill will check for all function-specs in the given module,\nwhether those functions correctly follow the spec.\n\nTo use this functionality, add `use TypeCheck.ExUnit` to your testing module.\n\n\nCurrently, spectesting uses `StreamData` under the hood.\nThis means that to use the `spectest` functionality,\nyou need to add the `:stream_data` dependency to your application\n(it is an optional dependency of `TypeCheck`.)\n\nIn the future, support for other property-generating libraries might be added.","ref":"TypeCheck.ExUnit.html"},{"type":"module","title":"What is a spectest? - TypeCheck.ExUnit","doc":"A 'function-specification test' is a property-based test in which\nwe check whether the function adheres to its _invariants_\n(also known as the function's _contract_ or _preconditions and postconditions_).\n\nWe generate a large amount of possible function inputs,\nand for each of these, check whether the function:\n\n- Does not raise an exception.\n- Returns a result that type-checks against the spec's return-type.\n  (To be precise, if an incorrect result is returned, the function\n  is wrapped in will end up raising an exception for this.)\n\nWhile `@spec!`s themselves ensure that callers do not mis-use your function,\na `spectest` ensures¹ that the function itself is working correctly.\n\nSpectests are given its own test-category in ExUnit, for easier recognition\n(Just like 'doctests' and 'properties' are different from normal tests, so are 'spectests'.)\n\n¹: Because of the nature of property-based testing, we can never know for 100% sure\nthat a function is correct. However, with every new randomly-generated\ntest-case, the level of confidence grows a little. So while we\ncan never by _fully_ sure, we are able to get asymptotically close to it.","ref":"TypeCheck.ExUnit.html#module-what-is-a-spectest"},{"type":"macro","title":"TypeCheck.ExUnit.__using__/1","doc":"Sets up a testing module for spectesting.\n\nNot normally invoked directly, but rather by calling `use TypeCheck.ExUnit`.\n\nCurrently does not accept any options, but this might change in the future.","ref":"TypeCheck.ExUnit.html#__using__/1"},{"type":"macro","title":"TypeCheck.ExUnit.spectest/2","doc":"Tests the functions in `module` against their `@spec!`s.\n\n`spectest` will look at all functions which have a TypeCheck spec in `module`,\nand will for each of them run a 'spectest'.\nSee the module documentation for more information on spectests in general.","ref":"TypeCheck.ExUnit.html#spectest/2"},{"type":"macro","title":"Examples - TypeCheck.ExUnit.spectest/2","doc":"```\ndefmodule MyModuleTest do\n  use ExUnit.Case, async: true\n  use TypeCheck.ExUnit\n\n  # Test all functions that have `@spec!`s in `MyModule`\n  spectest MyModule\n\n  # Test all functions that have `@spec!`s in `MyOtherModule`,\n  # except `MyOtherModule.bar/2` and `MyOtherModule.baz/0`\n  spectest MyOtherModule, except: [{:bar, 2}, {:baz, 0}]\n\n  # Test only `OneMoreModule.foo/2` and `MyOtherModule.qux/0`\n  spectest OneMoreModule, only: [{:foo, 2}, {:qux, 0}]\nend\n```","ref":"TypeCheck.ExUnit.html#spectest/2-examples"},{"type":"macro","title":"Options - TypeCheck.ExUnit.spectest/2","doc":"- `:except`\n- `:only`\n- `:initial_seed`\n- `:generator`\n\n#### Except and Only\n\nBy default, all functions in the module (that have an associated `@spec!`) will be tested.\n\nIf any of them need to be skipped, you can add them as a list of `{name, arity}`-pairs under `except:`.\n\nIf instead of excluding a few functions, you want to _only_ test a small subset of functions, you can add them as `{name, arity}`-pairs under `only:`.\n\n#### Initial seed\n\nThe `:initial_seed`-option can be used to seed the property-generation.\nIt expects an integer value.\nThis option is passed on to the generator automatically (without requiring the usage of generator-specific options).\nBy default, the seed of the ExUnit configuration is used (which by default differs every test run).\n\n#### Generator\n\nThe `generator:` option expects either the name of a property-testing library, or a `{name, options}`-tuple.\n(If only the name is specified, this is a shorthand for `{Name, []}`).\n\nFor now, only `StreamData` is supported, and this is its default value. If you want to pass extra options to the library, the notation `{StreamData, list_of_options}` can be passed.\nFor the list of options supported by StreamData, see `StreamData.check_all/3`.","ref":"TypeCheck.ExUnit.html#spectest/2-options"},{"type":"module","title":"TypeCheck.External","doc":"Working with regular Elixir and Erlang `@spec` and `@type` definitions.","ref":"TypeCheck.External.html"},{"type":"module","title":"Experimental - TypeCheck.External","doc":"This module is experimental.\nThe API of the various functions in this module are not yet stable;\nthey might change slightly in future minor versions of the library.\n\nIf you encounter a problem, please, [open an issue](https://github.com/Qqwy/elixir-type_check/issues).","ref":"TypeCheck.External.html#module-experimental"},{"type":"function","title":"TypeCheck.External.apply/4","doc":"Non-raising version of `TypeCheck.External.apply!/4`.\nEnsures values conform to the given type.\n\nReturns `{:error, reason}` on failure, `{:ok, function_call_result}` otherwise.\n\n    iex> alias TypeCheck.Builtin, as: B\n    iex> type = B.function([B.number], B.number)\n    iex> TypeCheck.External.apply(type, Kernel, :abs, [-13])\n    {:ok, 13}\n    iex> {:error, err} = TypeCheck.External.apply(type, Kernel, :abs, [false])\n    iex> err.message\n    \"At lib/type_check/external.ex:176:\\n    `false` is not a number.\"","ref":"TypeCheck.External.html#apply/4"},{"type":"function","title":"TypeCheck.External.apply!/4","doc":"The same as `Kernel.apply/3` but ensures that values conform the given type.\n\nThe first argument must be a function type of the function to be called.\n\nIn case of type error, raises an exception.","ref":"TypeCheck.External.html#apply!/4"},{"type":"function","title":"Examples - TypeCheck.External.apply!/4","doc":"iex> alias TypeCheck.Builtin, as: B\n    iex> type = B.function([B.number], B.number)\n    iex> TypeCheck.External.apply!(type, Kernel, :abs, [-13])\n    13\n    iex> TypeCheck.External.apply!(type, Kernel, :abs, [\"hello\"])\n    ** (TypeCheck.TypeError) At lib/type_check/external.ex:176:\n        `\"hello\"` is not a number.","ref":"TypeCheck.External.html#apply!/4-examples"},{"type":"macro","title":"TypeCheck.External.enforce_spec!/1","doc":"Ensure at runtime that arguments at result of function call conform the function spec.\n\nThe function spec is extracted at compile time from the regular Elixir (or Erlang) `@spec`.","ref":"TypeCheck.External.html#enforce_spec!/1"},{"type":"macro","title":"Examples - TypeCheck.External.enforce_spec!/1","doc":"iex> import TypeCheck.External\n    iex> enforce_spec!(Kernel.abs(-13))\n    13\n    iex> enforce_spec!(Kernel.abs(\"hi\"))\n    ** (TypeCheck.TypeError) At lib/type_check/external.ex:176:\n        `\"hi\"` is not a number.","ref":"TypeCheck.External.html#enforce_spec!/1-examples"},{"type":"function","title":"TypeCheck.External.fetch_spec/3","doc":"Extract TypeCheck type from the regular Elixir (or Erlang) `@spec` of the given function.","ref":"TypeCheck.External.html#fetch_spec/3"},{"type":"function","title":"Examples - TypeCheck.External.fetch_spec/3","doc":"iex> import TypeCheck.External\n    iex> {:ok, type} = fetch_spec(Kernel, :abs, 1)\n    iex> type\n    #TypeCheck.Type< (number() -> number()) >\n    iex> {:ok, type} = fetch_spec(Atom, :to_string, 1)\n    iex> type\n    #TypeCheck.Type< (atom() -> binary()) >\n    iex> fetch_spec(Kernel, :non_existent, 1)\n    {:error, \"cannot find spec for function\"}","ref":"TypeCheck.External.html#fetch_spec/3-examples"},{"type":"function","title":"TypeCheck.External.fetch_type/3","doc":"Extract TypeCheck type from the regular Elixir (or Erlang) `@type` with the given name.\n\nTo fetch a generic type, you must pass a list of types to be placed instead of\ngeneric variables. If you don't know these types, just punch in `any()`\nas many times as you need. For example, to fetch the type of `Range.t(left, right)`,\nyou need to pass `[number(), number()]` or `[any(), any()]`.","ref":"TypeCheck.External.html#fetch_type/3"},{"type":"function","title":"Examples - TypeCheck.External.fetch_type/3","doc":"Fetching a regular type:\n\n    iex> import TypeCheck.External\n    iex> {:ok, type} = fetch_type(String, :t)\n    iex> type\n    #TypeCheck.Type \n\nFetching a generic type:\n\n    iex> import TypeCheck.External\n    iex> import TypeCheck.Builtin\n    iex> {:ok, type} = fetch_type(:elixir, :keyword, [number()])\n    iex> type\n    #TypeCheck.Type \n    iex> {:ok, type} = fetch_type(Range, :t, [integer(), integer()])\n    iex> type\n    #TypeCheck.Type< %Range{first: integer(), last: integer(), step: pos_integer() | neg_integer()} >\n\nFetching non-existent type causes an error:\n\n    iex> import TypeCheck.External\n    iex> fetch_type(String, :non_existent)\n    {:error, \"cannot find type with the given name\"}","ref":"TypeCheck.External.html#fetch_type/3-examples"},{"type":"module","title":"TypeCheck.Macros","doc":"Contains the `@spec!`, `@type!`, `@typep!`, `@opaque!` macros to define runtime-checked function- and type-specifications.","ref":"TypeCheck.Macros.html"},{"type":"module","title":"Usage - TypeCheck.Macros","doc":"This module is included by calling `use TypeCheck`.\nThis will set up the module to use the special macros.\n\nUsually you'll want to use the module attribute-style of the macros, like\n`@spec!` and `@type!`.\nUsing these forms has two advantages over using the direct calls:\n\n1. Syntax highlighting will highlight the types correctly\nand the Elixir formatter will not mess with the way you write your type.\n2. It is clear to people who have not heard of `TypeCheck` before that `@type!` and `@spec!`\nwill work similarly to resp. `@type` and `@spec`.","ref":"TypeCheck.Macros.html#module-usage"},{"type":"module","title":"Avoiding naming conflicts with TypeCheck.Builtin - TypeCheck.Macros","doc":"If you want to define a type with the same name as one in TypeCheck.Builtin,\n_(which is not particularly recommended)_,\nyou should hide those particular functions from TypeCheck.Builtin by adding\n`import TypeCheck.Builtin, except: [...]`\nbelow `use TypeCheck` manually.","ref":"TypeCheck.Macros.html#module-avoiding-naming-conflicts-with-typecheck-builtin"},{"type":"module","title":"Calling the explicit implementations - TypeCheck.Macros","doc":"In case you are working in an environment where the `@/1` is already overridden\nby another library, you can still use this library,\nby simply adding `import TypeCheck.Macros, except: [@: 1]` to your module\nand calling the direct versions of the macros instead.","ref":"TypeCheck.Macros.html#module-calling-the-explicit-implementations"},{"type":"module","title":"TypeCheck and metaprogramming - TypeCheck.Macros","doc":"In certain cases you might want to use TypeCheck to dynamically generate\ntypes or functions, such as to add `@spec!`-s to functions\nthat themselves are dynamically generated.\n\nTypeCheck's macros support 'unquote fragments',\njust like many builtin 'definition' constructs like `def`, but also `@type` do.\n(c.f. `Elixir.Kernel.SpecialForms.quote/2` for more details about unquote fragments.)\n\nAn example:\n\n```\ndefmodule MetaExample do\n  use TypeCheck\n  people = ~w[joe robert mike]a\n  for name <- people do\n    @type! unquote(name)() :: %{name: unquote(name), coolness_level: :high}\n  end\nend\n```\n\n```\niex> MetaExample.joe\n#TypeCheck.Type \n\niex> MetaExample.mike\n#TypeCheck.Type \n\n```\n\n#### Macros\n\nInside macros, we use unquote fragments in the same way.\nThere is however one more thing to keep in mind:\nYou'll need to add a call to `import Kernel, except: [@: 1]` in your macro (before the quote)\nto make sure you can call `@type!`, `@spec!` etc.\nThis is a subtle consequence of Elixir's macro-hygiene rules.\n[See this issue on Elixir's GitHub repository for more info](https://github.com/elixir-lang/elixir/issues/10497#issuecomment-729479434)\n\n(Alternatively, directly calls to `type!`, `spec!` etc. are possible without overriding the import.)\n\nAn example:\n\n```\ndefmodule GreeterMacro do\n  defmacro generate_greeter(greeting) do\n    import Kernel, except: [@: 1] # Ensures TypeSpec's overridden `@` is used in the quote\n    quote do\n      @spec! unquote(greeting)(binary) :: binary\n      def unquote(greeting)(name) do\n        \"#{greeting}, #{name}!\"\n      end\n    end\n  end\nend\n\ndefmodule GreeterExample do\n  use TypeCheck\n  require GreeterMacro\n\n  GreeterMacro.generate_greeter(:hi)\n  GreeterMacro.generate_greeter(:hello)\nend\n```\n\n```\niex> GreeterExample.hi(\"John\")\n\"hi, John!\"\n\niex> GreeterExample.hello(\"Frank\")\n\"hello, Frank!\"\n\niex> GreeterExample.hi(42)\n** (TypeCheck.TypeError) At test/type_check/macros_test.exs:32:\n    The call to `hi/1` failed,\n    because parameter no. 1 does not adhere to the spec `binary()`.\n    Rather, its value is: `42`.\n    Details:\n      The call `hi(42)`\n      does not adhere to spec `hi(binary()) :: binary()`. Reason:\n        parameter no. 1:\n          `42` is not a binary.\n\n```\n\n#### About `use TypeCheck`\n\nThe `use TypeCheck` statement adds an `@before_compile`-hook to the final module,\nwhich is used to wrap functions with the specified runtime type-checks.\n\nThis means that some care needs to be taken to ensure that a call to `use TypeCheck` exists\nin the final module, if you're generating specs dynamically from inside macros.\n\n#### Hiding the autogenerated typespec\n\nBy default, `TypeCheck` will automatically generate `@type`, `@opaque` and `@spec`-attributes,\nwhich will be shown in the documentation, as well as used by tools such as Dialyzer.\n\nIn rare situations, `TypeCheck` might try to generate typespecs which are invalid.\n(In such case, please [open a bug report!](https://github.com/Qqwy/elixir-type_check/issues))\nOr sometimes, you might want to alter the type which is exported.\n\nIn such situations, you can disable the autogeneration of these attributes,\nby calling `@autogen_typespec false` just before the next `@type!`/`@opaque!`/`@spec!`:\n\n```\ndefmodule AutogenTypespecsExample do\n  use TypeCheck\n\n  # The typespec of `foo` is auto-generated:\n  # A line `@type foo() :: integer()` will be visible in the documentation/Dialyzer.\n  @type! foo() :: integer()\n\n  # The typespec of `bar` is _not_ auto-generated.\n  # As such, we could write a completely different `@type` (or leave it out all-together).\n  @autogen_typespec false\n  @type! bar() :: integer()\nend\n```\n\n```\niex>t AutogenTypespecsExample # Will show type of `foo` but not `bar`\n\"@type foo() :: integer()\"\n```","ref":"TypeCheck.Macros.html#module-typecheck-and-metaprogramming"},{"type":"macro","title":"TypeCheck.Macros.@/1","doc":"","ref":"TypeCheck.Macros.html#@/1"},{"type":"macro","title":"TypeCheck.Macros.opaque!/1","doc":"Define a opaque type specification.\n\n\nUsually invoked as `@opaque!`\n\nThis behaves similarly to Elixir's builtin `@opaque` attribute,\nand will create a type whose name is public\nbut whose structure is private.\n\n\nCalling this macro will:\n\n- Fill the `@opaque`-attribute with a Typespec-friendly\n  representation of the TypeCheck type.\n- Add a (or append to an already existing) `@typedoc` detailing that the type is\n  managed by TypeCheck, and containing the name of the TypeCheck type.\n  (not the definition, since it is an opaque type).\n- Define a (hidden) public function with the same name (and arity) as the type\n  that returns the TypeCheck.Type as a datastructure when called.\n  This makes the type usable in calls to:\n  - definitions of other type-specifications (in the same or different modules).\n  - definitions of function-specifications (in the same or different modules).\n  - `TypeCheck.conforms/2` and variants,\n  - `TypeCheck.Type.build/1`\n\n`opaque!/1` accepts the same typedef expression as `type!/1`.","ref":"TypeCheck.Macros.html#opaque!/1"},{"type":"macro","title":"TypeCheck.Macros.spec!/1","doc":"Define a function specification.\n\n\nUsually invoked as `@spec!`\n\nA function specification will wrap the function\nwith checks that each of its parameters are of the types it expects.\nas well as checking that the return type is as expected.","ref":"TypeCheck.Macros.html#spec!/1"},{"type":"macro","title":"Usage - TypeCheck.Macros.spec!/1","doc":"The syntax is essentially the same as for built-in `@spec` attributes:\n\n```\n@spec! function_name(type1, type2) :: return_type\n```\n\nIt is also allowed to introduce named types:\n\n```\n@spec! days_since_epoch(year :: integer, month :: integer, day :: integer) :: integer\n```\n\nNote that `TypeCheck` does _not_ allow the `when` keyword to be used\nto restrict the types of recurring type variables (which Elixir's\nbuiltin Typespecs allow). This is because:\n\n- Usually it is more clear to give a recurring type\n  an explicit name.\n- The `when` keyword is used instead for TypeCheck's type guards'.\n  (See `TypeCheck.Builtin.guarded_by/2` for more information.)","ref":"TypeCheck.Macros.html#spec!/1-usage"},{"type":"macro","title":"TypeCheck.Macros.type!/1","doc":"Define a public type specification.\n\nUsually invoked as `@type!`\n\nThis behaves similarly to Elixir's builtin `@type` attribute,\nand will create a type whose name and definition are public.\n\nCalling this macro will:\n\n- Fill the `@type`-attribute with a Typespec-friendly\n  representation of the TypeCheck type.\n- Add a (or append to an already existing) `@typedoc` detailing that the type is\n  managed by TypeCheck, and containing the full definition of the TypeCheck type.\n- Define a (hidden) public function with the same name (and arity) as the type\n  that returns the TypeCheck.Type as a datastructure when called.\n  This makes the type usable in calls to:\n  - definitions of other type-specifications (in the same or different modules).\n  - definitions of function-specifications (in the same or different modules).\n  - `TypeCheck.conforms/2` and variants,\n  - `TypeCheck.Type.build/1`","ref":"TypeCheck.Macros.html#type!/1"},{"type":"macro","title":"Usage - TypeCheck.Macros.type!/1","doc":"The syntax is essentially the same as for the built-in `@type` attribute:\n\n```elixir\n@type! type_name :: type_description\n```\n\nIt is possible to create parameterized types as well:\n\n```\n@type! dict(key, value) :: [{key, value}]\n```","ref":"TypeCheck.Macros.html#type!/1-usage"},{"type":"macro","title":"Named types - TypeCheck.Macros.type!/1","doc":"You can also introduce named types:\n\n```\n@type! color :: {red :: integer, green :: integer, blue :: integer}\n```\nNot only is this nice to document that the same type\nis being used for different purposes,\nit can also be used with a 'type guard' to add custom checks\nto your type specifications:\n\n```\n@type! sorted_pair(a, b) :: {first :: a, second :: b} when first <= second\n```","ref":"TypeCheck.Macros.html#type!/1-named-types"},{"type":"macro","title":"TypeCheck.Macros.typep!/1","doc":"Define a private type specification.\n\n\nUsually invoked as `@typep!`\n\nThis behaves similarly to Elixir's builtin `@typep` attribute,\nand will create a type whose name and structure is private\n(therefore only usable in the current module).\n\n- Fill the `@typep`-attribute with a Typespec-friendly\n  representation of the TypeCheck type.\n- Define a private function with the same name (and arity) as the type\n  that returns the TypeCheck.Type as a datastructure when called.\n  This makes the type usable in calls (in the same module) to:\n    - definitions of other type-specifications\n    - definitions of function-specifications\n    - `TypeCheck.conforms/2` and variants,\n    - `TypeCheck.Type.build/1`\n\n`typep!/1` accepts the same typedef expression as `type!/1`.","ref":"TypeCheck.Macros.html#typep!/1"},{"type":"module","title":"TypeCheck.Options","doc":"Defines the options that TypeCheck supports on calls to `use TypeCheck`.\n\nSupported options:\n\n- `:overrides`: A list of overrides for remote types. (default: `[]`)\n- `:default_overrides`: A boolean. If false, will not include any of the overrides of the types of Elixir's standard library (c.f. `TypeCheck.DefaultOverrides.default_overrides/0`). (default: `true`)\n- `:enable_runtime_checks`: When true, functions that contain a `@spec!` will be wrapped with a runtime check which will check the input to and result returned from the function. (Default: `true`).\n- `:debug`: When true, will (at compile-time) print the generated TypeCheck-checking code. (Default: `false`)\n\nThese options are usually specified as passed to `use TypeCheck`,\nalthough they may also be passed in direct calls to `TypeCheck.conforms/3` (and its variants).\n\nThese options are module-specific and are read/used at compile-time.","ref":"TypeCheck.Options.html"},{"type":"module","title":"Setting different options for different dependencies - TypeCheck.Options","doc":"Different default options can be set for each of the different dependencies,\nby supplying the desired options in the configuration of that dependency under the `:type_check` key:\n\n```\nconfig :your_dependency_name, :type_check, [\n  enable_runtime_checks: false\n  # etc.\n]\n```\n\nNote that these values might (by design) be overridden by module-specific options\nthat might be specified at the location where the dependency calls `use TypeCheck`.","ref":"TypeCheck.Options.html#module-setting-different-options-for-different-dependencies"},{"type":"module","title":"The supported options in detail - TypeCheck.Options","doc":"","ref":"TypeCheck.Options.html#module-the-supported-options-in-detail"},{"type":"module","title":"Overrides: - TypeCheck.Options","doc":"The `:overrides` field contains a list of remote types to be overridden by a replacement.\nThis is useful to be able to specify TypeCheck-types for types that you do not have control over\n(because they are for instance defined in a library that is not itself using TypeCheck).\n\nFor obvious reasons, using TypeCheck directly should be preferred over overriding types.\n\n\nEach of the elements in the `:overrides` list should be written as `{original_type, replacement_type}`.\nBoth of these can take the shape of either`&Module.type/arity` or the longer form `{Module, :type, arity}`.\n\nAn example:\n\n```\nuse TypeCheck, overrides: [\n  {&Ecto.Schema.t/0, &MyProject.TypeCheckOverrides.Ecto.Schema.t/0}\n]\n```","ref":"TypeCheck.Options.html#module-overrides"},{"type":"module","title":"Enabling/Disabling runtime checks - TypeCheck.Options","doc":"By default, runtime checks are enabled.\n\nIn the case where the runtime checks turn out to be too slow (for instance, because of working with very large or deeply nested collections) in a particular module,\nthey can be turned off completely.\n\nIt is recommended to:\n\n- Only turn them off after benchmarking has shown that this will make a significant difference.\n- Only turn them off in e.g. the production environment, keeping them on in the development and test environments.\n\nAn example:\n\n```elixir\nuse TypeCheck, enable_runtime_checks: Mix.env() != :prod\n```","ref":"TypeCheck.Options.html#module-enabling-disabling-runtime-checks"},{"type":"module","title":"Debugging - TypeCheck.Options","doc":"Passing the option `debug: true` will at compile-time print the generated code\nfor all added `@spec`s, as well as `TypeCheck.conforms/3`/`TypeCheck.conforms?/3`/`TypeCheck.conforms!/3` calls.","ref":"TypeCheck.Options.html#module-debugging"},{"type":"function","title":"TypeCheck.Options.check_overrides!/1","doc":"","ref":"TypeCheck.Options.html#check_overrides!/1"},{"type":"function","title":"TypeCheck.Options.new/0","doc":"","ref":"TypeCheck.Options.html#new/0"},{"type":"function","title":"TypeCheck.Options.new/1","doc":"","ref":"TypeCheck.Options.html#new/1"},{"type":"type","title":"TypeCheck.Options.remote_type/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nremote_type() :: mfa() | function\n```","ref":"TypeCheck.Options.html#t:remote_type/0"},{"type":"type","title":"TypeCheck.Options.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Options{\n  overrides: type_overrides(),\n  default_overrides: boolean(),\n  enable_runtime_checks: boolean(),\n  debug: boolean()\n}\n```","ref":"TypeCheck.Options.html#t:t/0"},{"type":"type","title":"TypeCheck.Options.type_override/0","doc":"An extra check is performed to ensure that the original type\nand the replacement type have the same arity.\n\n\n_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ntype_override :: {original :: remote_type(), replacement :: remote_type()}\n```","ref":"TypeCheck.Options.html#t:type_override/0"},{"type":"type","title":"TypeCheck.Options.type_overrides/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ntype_overrides :: list(type_override())\n```","ref":"TypeCheck.Options.html#t:type_overrides/0"},{"type":"module","title":"TypeCheck.Spec","doc":"","ref":"TypeCheck.Spec.html"},{"type":"function","title":"TypeCheck.Spec.defined?/3","doc":"True if a spec was added to `{module, function, arity}`.\n\nc.f. `lookup/3`.\n\n    iex(1)> defmodule Example3 do\n    iex(2)>   use TypeCheck\n    iex(3)>   @spec! greeter(name :: binary()) :: binary()\n    iex(4)>   def greeter(name), do: \"Hello, #{name}!\"\n    iex(5)> end\n    iex(6)>\n    iex(7)> TypeCheck.Spec.defined?(Example3, :greeter, 1)\n    true\n    iex(8)> TypeCheck.Spec.defined?(Example3, :nonexistent, 0)\n    false","ref":"TypeCheck.Spec.html#defined?/3"},{"type":"function","title":"TypeCheck.Spec.lookup/3","doc":"Looks up the spec for a particular `{module, function, arity}`.\n\nOn success, returns `{:ok, spec}`.\nOn failure, returns `{:error, :not_found}`.\n\nThis is quite an advanced low-level function,\nwhich you usually won't need to interact with directly.\n\n\nc.f. `lookup!/3`.\n\n    iex(1)> defmodule Example do\n    ...(2)>   use TypeCheck\n    ...(3)>   @spec! greeter(name :: binary()) :: binary()\n    ...(4)>   def greeter(name), do: \"Hello, #{name}!\"\n    ...(5)> end\n    ...(6)>\n    ...(7)> {:ok, spec} = TypeCheck.Spec.lookup(Example, :greeter, 1)\n    ...(8)> spec\n    #TypeCheck.Spec \n\n    iex> TypeCheck.Spec.lookup(Example, :nonexistent, 0)\n    {:error, :not_found}","ref":"TypeCheck.Spec.html#lookup/3"},{"type":"function","title":"TypeCheck.Spec.lookup!/3","doc":"Looks up the spec for a particular `{module, function, arity}`.\n\nOn success, returns `spec`.\nRaises when the spec cannot be found.\n\nc.f. `lookup/3`.\n\n    iex(1)> defmodule Example2 do\n    ...(2)>   use TypeCheck\n    ...(3)>   @spec! greeter(name :: binary()) :: binary()\n    ...(4)>   def greeter(name), do: \"Hello, #{name}!\"\n    ...(5)> end\n    ...(6)>\n    ...(7)> TypeCheck.Spec.lookup!(Example2, :greeter, 1)\n    #TypeCheck.Spec \n\n    iex> TypeCheck.Spec.lookup!(Example2, :nonexistent, 0)\n    ** (ArgumentError) No spec found for `Example2.nonexistent/0`","ref":"TypeCheck.Spec.html#lookup!/3"},{"type":"type","title":"TypeCheck.Spec.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :param_error,\n %{index: non_neg_integer(), problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple())},\n list(any())}\n| {t(), :return_error,\n   %{arguments: list(term()), problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple())},\n   list(any())}\n```","ref":"TypeCheck.Spec.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Spec.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %TypeCheck.Spec{\n  name: TypeCheck.DefaultOverrides.String.t(),\n  param_types: list(TypeCheck.Type.t()),\n  return_type: TypeCheck.Type.t(),\n  location: [] | list({:file, TypeCheck.DefaultOverrides.String.t()} | {:line, non_neg_integer()})\n}\n```","ref":"TypeCheck.Spec.html#t:t/0"},{"type":"module","title":"TypeCheck.Type","doc":"TODO","ref":"TypeCheck.Type.html"},{"type":"macro","title":"TypeCheck.Type.build/2","doc":"Constructs a concrete type from the given `type_ast`.\n\nThis means that you can pass type-syntax to this macro,\nwhich will be transformed into explicit calls to the functions in `TypeCheck.Builtin`.\n\n    iex> res = TypeCheck.Type.build(:ok | :error)\n    iex> res\n    #TypeCheck.Type< :ok | :error >\n    iex> # This is the same as:\n    iex> import TypeCheck.Builtin, only: [one_of: 2, literal: 1]\n    iex> explicit = one_of(literal(:ok), literal(:error))\n    iex> res == explicit\n    true\n\n    iex> res = TypeCheck.Type.build({a :: number(), b :: number()} when a <= b)\n    iex> res\n    #TypeCheck.Type< ({a :: number(), b :: number()} when a <= b) >\n    iex> # This is the same as:\n    iex> import TypeCheck.Builtin, only: [fixed_tuple: 1, number: 0, guarded_by: 2, named_type: 2]\n    iex> explicit = guarded_by(fixed_tuple([named_type(:a, number()), named_type(:b, number())]), quote do a <= b end)\n    iex> explicit\n    #TypeCheck.Type< ({a :: number(), b :: number()} when a <= b) >\n\nOf course, you can refer to your own local and remote types as well.","ref":"TypeCheck.Type.html#build/2"},{"type":"function","title":"TypeCheck.Type.type?/1","doc":"","ref":"TypeCheck.Type.html#type?/1"},{"type":"type","title":"TypeCheck.Type.expandable_type/0","doc":"Indicates that we expect a 'type AST' that will be expanded\nto a proper type. This means that it might contain essentially the full syntax that Elixir Typespecs\nallow, which will be rewritten to calls to the functions in `TypeCheck.Builtin`.\n\nSee `TypeCheck.Builtin` for the precise syntax you are allowed to use.","ref":"TypeCheck.Type.html#t:expandable_type/0"},{"type":"type","title":"TypeCheck.Type.t/0","doc":"Something is a TypeCheck.Type if it implements the TypeCheck.Protocols.ToCheck protocol.\n\nIt is also expected to implement the TypeCheck.Protocols.Inspect protocol (although that has an `Any` fallback).\n\nIn practice, this type means 'any of the' structs in the `TypeCheck.Builtin.*` modules.\n\n\n_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: wrap_with_gen(\n  x :: any() when TypeCheck.Type.type?(x),\n  &TypeCheck.Type.StreamData.arbitrary_type_gen/0\n)\n```","ref":"TypeCheck.Type.html#t:t/0"},{"type":"exception","title":"TypeCheck.TypeError","doc":"Exception to be returned or raised when a value is not of the expected type.\n\nThis exception has two fields:\n\n- `:raw`, which will contain the problem tuple of the type check failure.\n- `:message`, which will contain a the humanly-readable representation of the raw problem_tuple\n\n`:message` is constructed from `:raw` using the TypeCheck.TypeError.DefaultFormatter.\n(TODO at some point this might be configured to use your custom formatter instead)","ref":"TypeCheck.TypeError.html"},{"type":"type","title":"TypeCheck.TypeError.check_name/0","doc":"The name of the particular check. Might be `:no_match` for simple types,\nbut for more complex types that have multiple checks, it disambugates between them.\n\nFor instance, for `TypeCheck.Builtin.List` we have `:not_a_list`, `:different_length`, and `:element_error`.","ref":"TypeCheck.TypeError.html#t:check_name/0"},{"type":"type","title":"TypeCheck.TypeError.extra_information/0","doc":"An extra map with any information related to the check that failed.\n\nFor instance, if the check was a compound check, will contain the field `problem:` with the child problem_tuple\nas well as `:index` or `:key` etc. to indicate _where_ in the compound structure the check failed.","ref":"TypeCheck.TypeError.html#t:extra_information/0"},{"type":"type","title":"TypeCheck.TypeError.location/0","doc":"","ref":"TypeCheck.TypeError.html#t:location/0"},{"type":"type","title":"TypeCheck.TypeError.problem_tuple/0","doc":"A problem_tuple contains all information about a failed type check.\n\nc.f. TypeCheck.TypeError.Formatter.problem_tuple for a more precise definition","ref":"TypeCheck.TypeError.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.TypeError.problematic_value/0","doc":"The value that was passed in which failed the check.\n\nIt is included for the easy creation of `value did not match y`-style messages.","ref":"TypeCheck.TypeError.html#t:problematic_value/0"},{"type":"type","title":"TypeCheck.TypeError.t/0","doc":"","ref":"TypeCheck.TypeError.html#t:t/0"},{"type":"type","title":"TypeCheck.TypeError.type_checked_against/0","doc":"Any built-in TypeCheck struct (c.f. `TypeCheck.Builtin.*`), whose check(s) failed.","ref":"TypeCheck.TypeError.html#t:type_checked_against/0"},{"type":"module","title":"TypeCheck.TypeError.DefaultFormatter","doc":"","ref":"TypeCheck.TypeError.DefaultFormatter.html"},{"type":"function","title":"TypeCheck.TypeError.DefaultFormatter.do_format/1","doc":"Transforms a `problem_tuple` into a humanly-readable explanation string.\n\nC.f. `TypeCheck.TypeError.Formatter` for more information about problem tuples.","ref":"TypeCheck.TypeError.DefaultFormatter.html#do_format/1"},{"type":"function","title":"TypeCheck.TypeError.DefaultFormatter.format/2","doc":"","ref":"TypeCheck.TypeError.DefaultFormatter.html#format/2"},{"type":"behaviour","title":"TypeCheck.TypeError.Formatter","doc":"Behaviour to format your own type errors","ref":"TypeCheck.TypeError.Formatter.html"},{"type":"callback","title":"TypeCheck.TypeError.Formatter.format/2","doc":"A formatter is expected to turn a `problem_tuple` into a string\nthat can be used as `:message` of the TypeCheck.TypeError exception.","ref":"TypeCheck.TypeError.Formatter.html#c:format/2"},{"type":"type","title":"TypeCheck.TypeError.Formatter.problem_tuple/0","doc":"A `problem tuple` contains four fields:\n\n1. the module of the type for which a check did not pass\n2. an atom describing the exact error;\n   for many types there are multiple checks\n3. a map with fields containing extra information about the error.\n   in the cases of a compound type, this often contains information\n   about the deeper problem that happened as well.\n4. the datastructure that did not pass the check\n\n\nSee the module documentation of all `TypeCheck.Builtin.*` modules\nfor more information about the checks that they perform and the problem tuples they might return.\n\n\n_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: TypeCheck.Builtin.Any.problem_tuple()\n| TypeCheck.Builtin.Atom.problem_tuple()\n| TypeCheck.Builtin.Binary.problem_tuple()\n| TypeCheck.Builtin.Bitstring.problem_tuple()\n| TypeCheck.Builtin.Boolean.problem_tuple()\n| TypeCheck.Builtin.FixedList.problem_tuple()\n| TypeCheck.Builtin.FixedMap.problem_tuple()\n| TypeCheck.Builtin.CompoundFixedMap.problem_tuple()\n| TypeCheck.Builtin.FixedTuple.problem_tuple()\n| TypeCheck.Builtin.Float.problem_tuple()\n| TypeCheck.Builtin.Integer.problem_tuple()\n| TypeCheck.Builtin.List.problem_tuple()\n| TypeCheck.Builtin.MaybeImproperList.problem_tuple()\n| TypeCheck.Builtin.Literal.problem_tuple()\n| TypeCheck.Builtin.Map.problem_tuple()\n| TypeCheck.Builtin.NamedType.problem_tuple()\n| TypeCheck.Builtin.NegInteger.problem_tuple()\n| TypeCheck.Builtin.NonNegInteger.problem_tuple()\n| TypeCheck.Builtin.None.problem_tuple()\n| TypeCheck.Builtin.Number.problem_tuple()\n| TypeCheck.Builtin.OneOf.problem_tuple()\n| TypeCheck.Builtin.PosInteger.problem_tuple()\n| TypeCheck.Builtin.PID.problem_tuple()\n| TypeCheck.Builtin.Port.problem_tuple()\n| TypeCheck.Builtin.Reference.problem_tuple()\n| TypeCheck.Builtin.Range.problem_tuple()\n| TypeCheck.Builtin.Tuple.problem_tuple()\n| TypeCheck.Spec.problem_tuple()\n```","ref":"TypeCheck.TypeError.Formatter.html#t:problem_tuple/0"},{"type":"module","title":"TypeCheck.Type.StreamData","doc":"Transforms types into StreamData generators.\n\nWith the exception of `wrap_with_gen/2`,\nmethods in this module are\nonly compiled when the optional dependency\n`:stream_data` is added to your project's dependencies.","ref":"TypeCheck.Type.StreamData.html"},{"type":"function","title":"TypeCheck.Type.StreamData.arbitrary_primitive_type_gen/0","doc":"","ref":"TypeCheck.Type.StreamData.html#arbitrary_primitive_type_gen/0"},{"type":"function","title":"TypeCheck.Type.StreamData.arbitrary_type_gen/0","doc":"","ref":"TypeCheck.Type.StreamData.html#arbitrary_type_gen/0"},{"type":"function","title":"TypeCheck.Type.StreamData.fixed_list_gen/0","doc":"","ref":"TypeCheck.Type.StreamData.html#fixed_list_gen/0"},{"type":"function","title":"TypeCheck.Type.StreamData.to_gen/1","doc":"When given a type, it is transformed to a StreamData generator\nthat can be used in a property test.\n\n    iex> import TypeCheck.Type.StreamData\n    iex> generator = TypeCheck.Type.build({:ok | :error, integer()}) |> to_gen()\n    iex> StreamData.seeded(generator, 42) |> Enum.take(10)\n    [\n    {:ok, -1},\n    {:ok, 2},\n    {:ok, -2},\n    {:ok, -4},\n    {:ok, 1},\n    {:ok, 1},\n    {:ok, 2},\n    {:ok, 4},\n    {:ok, -7},\n    {:ok, 5}\n    ]","ref":"TypeCheck.Type.StreamData.html#to_gen/1"},{"type":"function","title":"TypeCheck.Type.StreamData.wrap_with_gen/2","doc":"Customizes a type with a _custom_ generator.\n\n`generator_function` can be a arity-zero function\n(in which case it should simply return a StreamData generator)\n\nor a arity-one function, in which case it is passed\nthe value that would be generated by default and it can be altered\nby e.g. using `StreamData.map/2` or `StreamData.bind/2`.\n\nNote that these functions _must_ be of the form `&Module.function/arity`\nbecause this is the only form of function capture that can be stored at compile-time.","ref":"TypeCheck.Type.StreamData.html#wrap_with_gen/2"},{"type":"function","title":"Example: - TypeCheck.Type.StreamData.wrap_with_gen/2","doc":"iex> defmodule IntString do\n    ...>   use TypeCheck\n    ...>   import TypeCheck.Type.StreamData\n    ...>   @type! t() :: ((val :: binary()) when Integer.parse(val) != :error)\n    ...>                 |> wrap_with_gen(&IntString.gen/0)\n    ...>\n    ...>   def gen() do\n    ...>     StreamData.integer()\n    ...>     |> StreamData.map(&to_string/1)\n    ...>   end\n    ...> end\n    ...>\n    ...> IntString.t() |> TypeCheck.Type.StreamData.to_gen() |> StreamData.seeded(42) |> Enum.take(10)\n    [\"0\", \"2\", \"1\", \"-3\", \"-5\", \"-4\", \"-3\", \"-4\", \"3\", \"-6\"]","ref":"TypeCheck.Type.StreamData.html#wrap_with_gen/2-example"},{"type":"module","title":"TypeCheck.Builtin","doc":"Contains TypeCheck specifications for all 'built-in' Elixir types.\n\nThese are all the types described on the ['Basic Types', 'Literals' and 'Builtin Types' sections of the Elixir 'Typespecs' documentation page.](https://hexdocs.pm/elixir/typespecs.html#basic-types)\n\nSee `TypeCheck.DefaultOverrides` for the 'Remote Types' supported by TypeCheck.\n\nUsually you'd want to import this module when you're using TypeCheck.\nThis is done automatically when calling `use TypeCheck`.\n\nIf necessary, feel free to hide (using `import ... except: `)\nthe things you don't need.","ref":"TypeCheck.Builtin.html"},{"type":"module","title":"Ommissions - TypeCheck.Builtin","doc":"TypeCheck strives to implement all of the syntax and builtin types\nwhich Elixir itself also supports.\nMost of them are supported today.\nThe rest will hopefully be supported in the near future.\n\nFor an up-to-date comparison of what types TypeCheck\ndoes and does not support w.r.t. Elixir's builtin typespecs,\nsee [Comparison to Plain Typespecs](comparing-typecheck-and-elixir-typespecs.html).","ref":"TypeCheck.Builtin.html#module-ommissions"},{"type":"function","title":"TypeCheck.Builtin.any/0","doc":"Any Elixir value.\n\nWill always succeed.\n\nc.f. `TypeCheck.Builtin.Any`\n\n\n    iex> TypeCheck.conforms!(10, any())\n    10\n    iex> TypeCheck.conforms!(\"foobar\", any())\n    \"foobar\"","ref":"TypeCheck.Builtin.html#any/0"},{"type":"function","title":"TypeCheck.Builtin.arity/0","doc":"Shorthand for `range(0..255)`\n\n    iex> TypeCheck.conforms!(1, arity())\n    1\n    iex> TypeCheck.conforms!(1000, arity())\n    ** (TypeCheck.TypeError) `1000` does not check against `0..255`. Reason:\n          `1000` falls outside the range 0..255.","ref":"TypeCheck.Builtin.html#arity/0"},{"type":"function","title":"TypeCheck.Builtin.as_boolean/1","doc":"The same as `type`,\nbut indicates that the result will be used\nas a boolean.\n\n    iex> TypeCheck.conforms!(:ok, as_boolean(atom()))\n    :ok\n    iex> TypeCheck.conforms!(10, as_boolean(atom()))\n    ** (TypeCheck.TypeError) `10` is not an atom.","ref":"TypeCheck.Builtin.html#as_boolean/1"},{"type":"function","title":"TypeCheck.Builtin.atom/0","doc":"Any Elixir atom.\n\nc.f. `TypeCheck.Builtin.Atom`\n\n    iex> TypeCheck.conforms!(:ok, atom())\n    :ok\n    iex> TypeCheck.conforms!(:foo, atom())\n    :foo\n    iex> TypeCheck.conforms!(10, atom())\n    ** (TypeCheck.TypeError) `10` is not an atom.","ref":"TypeCheck.Builtin.html#atom/0"},{"type":"function","title":"TypeCheck.Builtin.binary/0","doc":"Any binary.\n\nA binary is a bitstring with a bitsize divisible by eight.\n\nc.f. `TypeCheck.Builtin.Binary`","ref":"TypeCheck.Builtin.html#binary/0"},{"type":"function","title":"TypeCheck.Builtin.bitstring/0","doc":"Any bitstring\n\nc.f. `TypeCheck.Builtin.Bitstring`","ref":"TypeCheck.Builtin.html#bitstring/0"},{"type":"function","title":"TypeCheck.Builtin.boolean/0","doc":"Any boolean\n\n(either `true` or `false`.)\n\nc.f. `TypeCheck.Builtin.Boolean`","ref":"TypeCheck.Builtin.html#boolean/0"},{"type":"function","title":"TypeCheck.Builtin.byte/0","doc":"A byte; shorthand for `range(0..255)`\n\nc.f. `range/1`\n\n    iex> TypeCheck.conforms!(1, byte())\n    1\n    iex> TypeCheck.conforms!(255, byte())\n    255\n    iex> TypeCheck.conforms!(256, byte())\n    ** (TypeCheck.TypeError) `256` does not check against `0..255`. Reason:\n          `256` falls outside the range 0..255.","ref":"TypeCheck.Builtin.html#byte/0"},{"type":"function","title":"TypeCheck.Builtin.char/0","doc":"A char; shorthand for `range(0..0x10FFFF)`\n\nc.f. `range/1`\n\n    iex> TypeCheck.conforms!(?a, char())\n    97\n    iex> TypeCheck.conforms!(-1, char())\n    ** (TypeCheck.TypeError) `-1` does not check against `0..1114111`. Reason:\n          `-1` falls outside the range 0..1114111.","ref":"TypeCheck.Builtin.html#char/0"},{"type":"function","title":"TypeCheck.Builtin.charlist/0","doc":"A list filled with characters; exactly `list(char())`\n\nc.f. `list/1` and `char/0`\n\n    iex> TypeCheck.conforms!(~c'hello world', charlist())\n    ~c'hello world'\n    iex> TypeCheck.conforms!(\"hello world\", charlist())\n    ** (TypeCheck.TypeError) `\"hello world\"` does not check against `list(0..1114111)`. Reason:\n          `\"hello world\"` is not a list.","ref":"TypeCheck.Builtin.html#charlist/0"},{"type":"function","title":"TypeCheck.Builtin.fancy_map/3","doc":"Allows constructing map types containing a combination of fixed, required and optional keys (and their associated type-values)\n\nDesugaring of most ways of map syntaxes.\n\nNote that because of reasons of efficiency and implementation difficulty,\nnot all possibilities are supported by TypeCheck currently.\n\nSupported are:\n- maps with only fixed keys (`%{a: 1, b: 2, \"foo\" => number()}`)\n- maps with a single required keypair (`%{required(key_type) => value_type}`)\n- maps with a single optional keypair (`%{optional(key_type) => value_type}`)\n- maps with only fixed keys and one optional keypair (`%{:a => 1, :b => 2, \"foo\" => number(), optional(integer()) => boolean()}`)\n\nHelp with extending this support is very welcome.\nc.f. https://github.com/Qqwy/elixir-type_check/issues/7","ref":"TypeCheck.Builtin.html#fancy_map/3"},{"type":"function","title":"TypeCheck.Builtin.fixed_list/1","doc":"A list of fixed size where `element_types` dictates the types\nof each of the respective elements.\n\nDesugaring of literal lists like `[:a, 10, \"foo\"]`.\n\nCannot directly be represented in Elixir's builtin Typespecs,\nand is thus represented as `[any()]` instead.","ref":"TypeCheck.Builtin.html#fixed_list/1"},{"type":"function","title":"TypeCheck.Builtin.fixed_map/1","doc":"A map with exactly the key-value-pairs indicated by `keywords`\n\nwhere all keys are required to be literal values,\nand the values are a type specification.\n\nDesugaring of literal maps like `%{a_key: value_type, \"other_key\" => value_type2}`.\n\nRepresented in Elixir's builtin Typespecs as\n```\n%{required(:a_key) => value_type1, required(\"other key\") => value_type2}\n```.\n(for e.g. a call to `fixed_map([a_key: value_type1, {\"other key\", value_type2}])`)","ref":"TypeCheck.Builtin.html#fixed_map/1"},{"type":"function","title":"TypeCheck.Builtin.fixed_tuple/1","doc":"A tuple whose elements are of the types given by `list_of_element_types`.\n\nDesugaring of writing tuples directly in your types:\n`{a, b, c}` desugars to `fixed_tuple([a, b, c])`.\n\nRepresented in Elixir's builtin Typespecs as a plain tuple,\nwhere each of the elements are the respective element of `list_of_types`.\n\nC.f. `TypeCheck.Builtin.Tuple`","ref":"TypeCheck.Builtin.html#fixed_tuple/1"},{"type":"function","title":"TypeCheck.Builtin.float/0","doc":"Any float.\n\nC.f. `TypeCheck.Builtin.Float`","ref":"TypeCheck.Builtin.html#float/0"},{"type":"function","title":"TypeCheck.Builtin.fun/0","doc":"Alias for `function/0`.\n\n    iex> TypeCheck.conforms!(&div/2, fun())\n    &:erlang.div/2","ref":"TypeCheck.Builtin.html#fun/0"},{"type":"function","title":"TypeCheck.Builtin.function/0","doc":"Any function (of any arity), regardless of input or output types\n\nc.f. `TypeCheck.Builtin.Function`\n\n    iex> TypeCheck.conforms!(&div/2, function())\n    &:erlang.div/2\n    iex> TypeCheck.conforms!(&Application.get_env/3, function())\n    &Application.get_env/3\n    iex> TypeCheck.conforms!(42, function())\n    ** (TypeCheck.TypeError) `42` is not a function.","ref":"TypeCheck.Builtin.html#function/0"},{"type":"function","title":"TypeCheck.Builtin.function/1","doc":"A function (of any arity) returning `return_type`.\n\nDesugaring of `(... -> return_type)`\n\nSee `function/2` for more info.\n\nc.f. `TypeCheck.Builtin.Function`","ref":"TypeCheck.Builtin.html#function/1"},{"type":"function","title":"TypeCheck.Builtin.function/2","doc":"A function taking `param_types` as parameters, returning `return_type`.\n\nDesugaring of `(param_type -> return_type)`,\n`(param_type, param_type2 -> return_type)`,\n`(param_type, param_type2, param_type3 -> return_type)` etc.\n\nType-checking a function value against a function-type works a bit differently from most other types.\nThe reason for this is that we can only ascertain whether the function-value works correctly when the function-value is called.\n\nSpecifically:\n- When a call to `TypeCheck.conforms/3` (and variants) or a function wrapped with a `@spec` is called, we can immediately check whether a particular parameter:\n  - is a function\n  - accepts the expected arity\n- Then, the parameter-which-is-a-function is wrapped in a 'wrapper function' which, when called:\n  - typechecks whether the passed parameters are of the expected types (_This checks whether *your* function uses the parameter-function correctly_.)\n  - calls the original function with the parameters.\n  - typechecks whether the result is of the expected type. (_This checks whether the *parameter-function* works correctly_.)\n  - returns the result.\n\nIn other words, the 'wrapper function' which is added for a type `(param_type, param_type2 -> result_type)` works similarly\nto a named function with the spec `@spec myfunction(param_type, param_type2) :: result_type`.\n\n    iex> # The following passes the first check...\n    iex> fun = TypeCheck.conforms!(&div/2, (integer(), integer() -> boolean()))\n    iex> # ... but once the function returns, the wrapper will raise\n    iex> fun.(20, 5)\n    ** (TypeCheck.TypeError) The call to `#Function<...>/2` failed,\n        because the returned result does not adhere to the spec `boolean()`.\n        Rather, its value is: `4`.\n        Details:\n          The result of calling `#Function<...>.(20, 5)`\n          does not adhere to spec `(integer(), integer() -> boolean())`. Reason:\n            Returned result:\n              `4` is not a boolean.\n\nc.f. `TypeCheck.Builtin.Function`","ref":"TypeCheck.Builtin.html#function/2"},{"type":"function","title":"TypeCheck.Builtin.guarded_by/3","doc":"Adds a 'type guard' to the type, which is an extra check\nwritten using arbitrary Elixir code.\n\nDesugaring of `some_type when guard_code`.\n\nThe type guard is a check written using any Elixir code,\nwhich may refer to names set in the type using `named_type/2`.\n\nIf this type guard fails (by returning a non-truthy value),\nthe type will not check.\n\nFor user-friendly error-handling, don't let your type guards\nthrow exceptions.\n\nC.f. `TypeCheck.Builtin.Guarded`\n\nCannot be represented in Elixir's builtin Typespecs,\nand is thus represented as `type` (without the guard) instead.","ref":"TypeCheck.Builtin.html#guarded_by/3"},{"type":"function","title":"TypeCheck.Builtin.identifier/0","doc":"Syntactic sugar for `pid() | port() | reference()`\n\n    iex> TypeCheck.conforms?(self(), identifier())\n    true","ref":"TypeCheck.Builtin.html#identifier/0"},{"type":"function","title":"TypeCheck.Builtin.impl/1","doc":"Checks whether the given value implements the particular protocol.\n\n\nFor this type-check to work, [Protocol Consolidation](https://hexdocs.pm/elixir/Protocol.html#module-consolidation) needs to be active.","ref":"TypeCheck.Builtin.html#impl/1"},{"type":"function","title":"Data generation - TypeCheck.Builtin.impl/1","doc":"TypeCheck tries to generate values of any type implementing the protocol.\nThese generators can generate any built-in type for which the protocol is implemented (with the exception of functions, and datetimes).\n\nIt can also generate your custom structs, as long as:\n\n- They contain a TypeCheck type called `t`.\n  In this case, any values adhering to `t` will be generated.\n- They don't have a `t` TypeCheck type, but contain a `new/0` function.\n  In this case, a single value is generated each time: the result of calling `YourStructModule.new/0`.\n\nA deliberate choice was made not to automatically generate values for any module by using `struct/0`,\nbecause this would not respect the `@enforce_keys` option that might be given to structs.","ref":"TypeCheck.Builtin.html#impl/1-data-generation"},{"type":"function","title":"TypeCheck.Builtin.integer/0","doc":"Any integer.\n\nC.f. `TypeCheck.Builtin.Integer`\n\n    iex> TypeCheck.conforms!(42, integer())\n    42\n\n    iex> TypeCheck.conforms!(42.0, integer())\n    ** (TypeCheck.TypeError) `42.0` is not an integer.\n\n    iex> TypeCheck.conforms!(\"hello\", integer())\n    ** (TypeCheck.TypeError) `\"hello\"` is not an integer.","ref":"TypeCheck.Builtin.html#integer/0"},{"type":"function","title":"TypeCheck.Builtin.iodata/0","doc":"Syntactic sugar for `binary() | iolist()`","ref":"TypeCheck.Builtin.html#iodata/0"},{"type":"function","title":"TypeCheck.Builtin.iolist/0","doc":"A potentially-improper list containing binaries,\nsingle characters, or nested iolists.\n\nSyntactic sugar for `maybe_improper_list(byte() | binary() | iolist(), binary() | []) `","ref":"TypeCheck.Builtin.html#iolist/0"},{"type":"function","title":"TypeCheck.Builtin.keyword/0","doc":"A list of pairs with atoms as 'keys' and anything allowed as as 'values'.\n\nShorthand for `list({atom(), any()})`\n\n    iex> x = [a: 1, b: 2]\n    iex> TypeCheck.conforms!(x, keyword())\n    [a: 1, b: 2]\n\n    iex> y = [a: 1, b: 2] ++ [3, 4]\n    iex> TypeCheck.conforms!(y, keyword())\n    ** (TypeCheck.TypeError) `[{:a, 1}, {:b, 2}, 3, 4]` does not check against `list({atom(), any()})`. Reason:\n          at index 2:\n            `3` does not check against `{atom(), any()}`. Reason:\n              `3` is not a tuple.","ref":"TypeCheck.Builtin.html#keyword/0"},{"type":"function","title":"TypeCheck.Builtin.keyword/1","doc":"A list of pairs with atoms as 'keys' and t's as 'values'.\n\nShorthand for `list({atom(), t})`","ref":"TypeCheck.Builtin.html#keyword/1"},{"type":"macro","title":"TypeCheck.Builtin.lazy/1","doc":"Defers type-expansion until the last possible moment.\n\nThis is used to be able to expand recursive types.\n\nFor instance, if you have the following:\n\n```\ndefmodule MyBrokenlist do\n  type empty :: nil\n  type cons(a) :: {a, mylist(a)}\n  type mylist(a) :: empty() | cons(a)\n\n  spec new_list() :: mylist(any())\n  def new_list() do\n    nil\n  end\n\n  spec cons_val(mylist(any()), any()) :: mylist(any)\n  def cons_val(list, val) do\n    {val, list}\n  end\nend\n```\n\nthen when `TypeCheck` is expanding the `spec`s at compile-time\nto build the type-checking code, `mylist(a)` will call `cons(a)`\nwhich will call `mylist(a)` which will call `cons(a)` etc. until infinity.\nThis makes compilation hang indefinitely.\n\nTo be able to handle types like this, use `lazy`:\n\n\n```\ndefmodule MyFixedList do\n  type empty :: nil\n  type cons(a) :: {a, lazy(mylist(a))}\n  type mylist(a) :: empty() | cons(a)\n\n  spec new_list() :: mylist(any())\n  def new_list() do\n    nil\n  end\n\n  spec cons_val(mylist(any()), any()) :: mylist(any)\n  def cons_val(list, val) do\n    {val, list}\n  end\nend\n```\n\nThis will work as intended.\n\nSince `lazy/1` defers type-expansion (and check-code-generation) until\nruntime, the compiler is not able to optimize the type-checking code.\n\nThus, you should only use it when necessary, since it will be slower\nthan when using the inner type directly.","ref":"TypeCheck.Builtin.html#lazy/1"},{"type":"macro","title":"In builtin typespecs - TypeCheck.Builtin.lazy/1","doc":"`lazy/1` does not exist in Elixir's builtin typespecs\n(since builtin typespecs does not expand types it does not need to handle\nrecursive types in a special way).\nTherefore, `lazy(some_type)` is represented\nas `some_type` directly in ELixir's builtin typespecs.","ref":"TypeCheck.Builtin.html#lazy/1-in-builtin-typespecs"},{"type":"function","title":"TypeCheck.Builtin.list/0","doc":"A (proper) list with any type of elements;\n\nshorthand for `list(any())`\n\nC.f. `list/1` and `any/0`","ref":"TypeCheck.Builtin.html#list/0"},{"type":"function","title":"TypeCheck.Builtin.list/1","doc":"A (proper) list containing only elements of type `a`.\n\nC.f. `TypeCheck.Builtin.List`\n\n\n    iex> TypeCheck.conforms!([1,2,3], list(integer()))\n    [1,2,3]\n\n    iex> TypeCheck.conforms!(:foo, list(integer()))\n    ** (TypeCheck.TypeError) `:foo` does not check against `list(integer())`. Reason:\n          `:foo` is not a list.\n\n    iex> TypeCheck.conforms!([1, 2, 3.3], list(integer()))\n    ** (TypeCheck.TypeError) `[1, 2, 3.3]` does not check against `list(integer())`. Reason:\n          at index 2:\n            `3.3` is not an integer.","ref":"TypeCheck.Builtin.html#list/1"},{"type":"function","title":"TypeCheck.Builtin.literal/1","doc":"A literal value.\n\nDesugaring of using any literal primitive value\n(like a particular integer, float, atom, binary or bitstring)\ndirectly a type.\n\nFor instance, `10` desugars to `literal(10)`.\n\nRepresented in Elixir's builtin Typespecs as\n- for integers, atoms and booleans: the primitive value itself.\n- for binaries, a more general `binary()` is used\n  as Elixir's builtin typespecs do not support literal UTF-8 binaries as literal values.\n- For other kinds of values which Elixir's builtin typespecs do not support as literals,\n  we similarly represent it as a more general type.\n\nC.f. `TypeCheck.Builtin.Literal`","ref":"TypeCheck.Builtin.html#literal/1"},{"type":"function","title":"TypeCheck.Builtin.map/0","doc":"Any Elixir map with any types as keys and any types as values.\n\nC.f. `TypeCheck.Builtin.Map`","ref":"TypeCheck.Builtin.html#map/0"},{"type":"function","title":"TypeCheck.Builtin.map/2","doc":"Any map containing zero or more keys of `key_type` and values of `value_type`.\n\nRepresented in Elixir's builtin Typespecs as `%{optional(key_type) => value_type}`,\nand indeed a desugaring of this.\n\nNote that multiple optional keypairs are not (yet) supported.\n\nC.f. `TypeCheck.Builtin.Map`","ref":"TypeCheck.Builtin.html#map/2"},{"type":"function","title":"TypeCheck.Builtin.maybe_improper_list/0","doc":"Syntactic sugar for `maybe_improper_list(any(), any())`","ref":"TypeCheck.Builtin.html#maybe_improper_list/0"},{"type":"function","title":"TypeCheck.Builtin.maybe_improper_list/2","doc":"WIP","ref":"TypeCheck.Builtin.html#maybe_improper_list/2"},{"type":"function","title":"TypeCheck.Builtin.mfa/0","doc":"A module-function-arity tuple\n\n- Module is a `module/0`\n- function is an `atom/0`\n- Arity is an `arity/0`\n\nC.f. `fixed_tuple/1`","ref":"TypeCheck.Builtin.html#mfa/0"},{"type":"function","title":"TypeCheck.Builtin.module/0","doc":"Any Elixir atom,\nbut indicates that the atom\nis expected to be used as a module.\n\n    iex> TypeCheck.conforms!(String, module())\n    String\n    iex> TypeCheck.conforms!(:array, module())\n    :array\n    iex> TypeCheck.conforms!(\"hello\", module())\n    ** (TypeCheck.TypeError) `\"hello\"` is not an atom.\n\nc.f. `atom/0`","ref":"TypeCheck.Builtin.html#module/0"},{"type":"function","title":"TypeCheck.Builtin.named_type/4","doc":"A type with a local name, which can be referred to from a 'type guard'.\n\nThis name can be used in 'type guards'.\nSee the module documentation and `guarded_by/2` for more information.\n\nDesugaring of `name :: type` (when `::` is used _inside_ a type.).\n\nCannot directly be represented in Elixir's builtin Typespecs,\nand is thus represented as `type` (without the name) instead.","ref":"TypeCheck.Builtin.html#named_type/4"},{"type":"function","title":"TypeCheck.Builtin.neg_integer/0","doc":"Any integer smaller than zero.\n\nC.f. `TypeCheck.Builtin.NegInteger`","ref":"TypeCheck.Builtin.html#neg_integer/0"},{"type":"function","title":"TypeCheck.Builtin.no_return/0","doc":"See `none/0`.","ref":"TypeCheck.Builtin.html#no_return/0"},{"type":"function","title":"TypeCheck.Builtin.non_neg_integer/0","doc":"Any integer zero or larger.\n\nC.f. `TypeCheck.Builtin.NonNegInteger`","ref":"TypeCheck.Builtin.html#non_neg_integer/0"},{"type":"function","title":"TypeCheck.Builtin.none/0","doc":"Matches no value at all.\n\n`none()` is not very useful on its own,\nbut it is a useful default in certain circumstances,\nas well as to indicate that you expect some place to not return at all.\n(instead for instance throwing an exception or looping forever.)\n\nC.f. `TypeCheck.Builtin.None`.","ref":"TypeCheck.Builtin.html#none/0"},{"type":"function","title":"TypeCheck.Builtin.nonempty_binary/0","doc":"A binary which contains at least one byte.\n\nShorthand for `sized_bitstring(8, 8)`.","ref":"TypeCheck.Builtin.html#nonempty_binary/0"},{"type":"function","title":"TypeCheck.Builtin.nonempty_bitstring/0","doc":"A bitstring which contains at least one bit.\n\nShorthand for `sized_bitstring(1, 1)`.","ref":"TypeCheck.Builtin.html#nonempty_bitstring/0"},{"type":"function","title":"TypeCheck.Builtin.nonempty_charlist/0","doc":"Shorthand for nonempty_list(char()).","ref":"TypeCheck.Builtin.html#nonempty_charlist/0"},{"type":"function","title":"TypeCheck.Builtin.nonempty_improper_list/2","doc":"Any list with at least one element, which has to be terminated by something else than `[]`.\n\nTo be precise, the list needs to be terminated with `terminator_type`.","ref":"TypeCheck.Builtin.html#nonempty_improper_list/2"},{"type":"function","title":"TypeCheck.Builtin.nonempty_list/0","doc":"Shorthand for nonempty_list(any()).","ref":"TypeCheck.Builtin.html#nonempty_list/0"},{"type":"function","title":"TypeCheck.Builtin.nonempty_list/1","doc":"A nonempty_list is any list with at least one element.","ref":"TypeCheck.Builtin.html#nonempty_list/1"},{"type":"function","title":"TypeCheck.Builtin.nonempty_maybe_improper_list/2","doc":"Any list with at least one element, which might be terminated by something else than `[]`.\n\nTo be precise, the list needs to be terminated with either `[]` or `terminator_type`","ref":"TypeCheck.Builtin.html#nonempty_maybe_improper_list/2"},{"type":"function","title":"TypeCheck.Builtin.number/0","doc":"Any number (either a float or an integer)\n\nMatches the same as `integer | float` but is more efficient.\n\nC.f. `TypeCheck.Builtin.Number`","ref":"TypeCheck.Builtin.html#number/0"},{"type":"function","title":"TypeCheck.Builtin.one_of/1","doc":"Version of `one_of` that allows passing many possibilities\nat once.\n\nA union of multiple types (also known as a 'sum type')\n\nDesugaring of types separated by `|` like `a | b` or `a | b | c | d`.\n(and represented that way in Elixir's builtin Typespecs).\n\nc.f. `one_of/2`.","ref":"TypeCheck.Builtin.html#one_of/1"},{"type":"function","title":"TypeCheck.Builtin.one_of/2","doc":"A union of multiple types (also known as a 'sum type')\n\nDesugaring of types separated by `|` like `a | b` or `a | b | c | d`.\n(and represented that way in Elixir's builtin Typespecs).","ref":"TypeCheck.Builtin.html#one_of/2"},{"type":"function","title":"TypeCheck.Builtin.pid/0","doc":"Matches any process-identifier.\n\nNote that no checks are made to see whether the process is alive or not.\n\nAlso, the current property-generator will generate arbitrary PIDs, most of which\nwill not point to alive processes.","ref":"TypeCheck.Builtin.html#pid/0"},{"type":"function","title":"TypeCheck.Builtin.port/0","doc":"Matches any port.\n\nc.f. `TypeCheck.Builtin.Port`\n\n    iex> TypeCheck.conforms?(Kernel.make_ref(), reference())\n    true\n    iex> some_port = Port.open({:spawn, \"cat\"}, [:binary])\n    ...> TypeCheck.conforms?(some_port, port())\n    true","ref":"TypeCheck.Builtin.html#port/0"},{"type":"function","title":"TypeCheck.Builtin.pos_integer/0","doc":"Any integer larger than zero.\n\nC.f. `TypeCheck.Builtin.PosInteger`","ref":"TypeCheck.Builtin.html#pos_integer/0"},{"type":"function","title":"TypeCheck.Builtin.range/1","doc":"Any integer in the half-open range `range`.\n\nDesugaring of `a..b`.\n(And represented that way in Elixir's builtin Typespecs.)\n\nC.f. `TypeCheck.Builtin.Range`","ref":"TypeCheck.Builtin.html#range/1"},{"type":"function","title":"TypeCheck.Builtin.range/2","doc":"Any integer between `lower` (includsive) and `higher` (exclusive).\n\nDesugaring of `lower..higher`.\n(And represented that way in Elixir's builtin Typespecs.)\n\nC.f. `range/1`","ref":"TypeCheck.Builtin.html#range/2"},{"type":"function","title":"TypeCheck.Builtin.reference/0","doc":"Matches any reference.\n\nc.f. `TypeCheck.Builtin.Reference`\n\n    iex> TypeCheck.conforms?(Kernel.make_ref(), reference())\n    true\n    iex> some_ref = IEx.Helpers.ref(0, 749884137, 111673345, 43386)\n    ...> TypeCheck.conforms!(some_ref, reference())\n    #Reference<0.749884137.111673345.43386>","ref":"TypeCheck.Builtin.html#reference/0"},{"type":"function","title":"TypeCheck.Builtin.sized_bitstring/1","doc":"A bitstring of fixed size.\n\nDesugaring of bitstring types like `<< _ :: size>>`.\n\nc.f. `TypeCheck.Builtin.SizedBitstring`.","ref":"TypeCheck.Builtin.html#sized_bitstring/1"},{"type":"function","title":"TypeCheck.Builtin.sized_bitstring/2","doc":"A bitstring with a fixed `prefix_size` (which might be `0`), followed by zero or repetitions of `unit_size`.\n\nDesugaring of bitstring types like `<< _ :: _ * unit_size>>` and `<< _ :: prefix_size, _ :: _ * unit_size>>`.\n\n    iex> TypeCheck.conforms!(\"hi\", <<_ :: 16>>)\n    \"hi\"\n\n    iex> TypeCheck.conforms!(\"bye\", <<_ :: 16>>)\n    ** (TypeCheck.TypeError) `\"bye\"` has a different bit_size (24) than expected (16).\n\n    iex> TypeCheck.conforms!(<<1 :: size(2)>>, <<_ :: 2>>)\n    <<1 :: size(2)>>\n\n    iex> TypeCheck.conforms!(<<1 :: size(3)>>, <<_ :: 2>>)\n    ** (TypeCheck.TypeError) `<<1::size(3)>>` has a different bit_size (3) than expected (2).\n\n    iex> [\"ab\", \"abcd\", \"abcdef\"] |> Enum.map(&TypeCheck.conforms!(&1, <<_ :: _ * 16>>))\n    [\"ab\", \"abcd\", \"abcdef\"]\n\n    iex> TypeCheck.conforms!(\"abc\",  <<_ :: _ * 16>>)\n    ** (TypeCheck.TypeError) `\"abc\"` has a different bit_size (24) than expected (_ * 16).\n\n    iex> [\"a\", \"abc\", \"abcde\"] |> Enum.map(&TypeCheck.conforms!(&1, <<_ :: 8, _ :: _ * 16>>))\n    [\"a\", \"abc\", \"abcde\"]\n\n    iex> TypeCheck.conforms!(\"ab\",  <<_ :: 8, _ :: _ * 16>>)\n    ** (TypeCheck.TypeError) `\"ab\"` has a different bit_size (16) than expected (8 + _ * 16).\n\n\nc.f. `TypeCheck.Builtin.SizedBitstring`.","ref":"TypeCheck.Builtin.html#sized_bitstring/2"},{"type":"function","title":"TypeCheck.Builtin.struct/0","doc":"Any kind of struct.\n\nSyntactic sugar for %{:__struct__ => atom(), optional(atom()) => any()}","ref":"TypeCheck.Builtin.html#struct/0"},{"type":"function","title":"TypeCheck.Builtin.term/0","doc":"alias for `any/0`","ref":"TypeCheck.Builtin.html#term/0"},{"type":"function","title":"TypeCheck.Builtin.timeout/0","doc":"Builtin type. Syntactic sugar for `:infinity | non_neg_integer()`","ref":"TypeCheck.Builtin.html#timeout/0"},{"type":"function","title":"TypeCheck.Builtin.tuple/0","doc":"A tuple of any size (with any elements).\n\nC.f. `TypeCheck.Builtin.Tuple`","ref":"TypeCheck.Builtin.html#tuple/0"},{"type":"function","title":"TypeCheck.Builtin.tuple/1","doc":"A tuple whose elements have any types,\nbut which has exactly `size` elements.\n\nRepresented in Elixir's builtin Typespecs as a plain tuple,\nwith `size` elements, where each of the element types\nis `any()`.\n\nFor instance, `tuple(3)` is represented as `{any(), any(), any()}`.","ref":"TypeCheck.Builtin.html#tuple/1"},{"type":"module","title":"TypeCheck.Builtin.Any","doc":"","ref":"TypeCheck.Builtin.Any.html"},{"type":"type","title":"TypeCheck.Builtin.Any.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: none()\n```","ref":"TypeCheck.Builtin.Any.html#t:problem_tuple/0"},{"type":"opaque","title":"TypeCheck.Builtin.Any.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n`t` _(opaque type)_","ref":"TypeCheck.Builtin.Any.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Atom","doc":"Checks whether the value is any atom.\n\nReturns a problem tuple with the reason `:no_match` otherwise.","ref":"TypeCheck.Builtin.Atom.html"},{"type":"type","title":"TypeCheck.Builtin.Atom.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.Atom.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.Atom.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.Atom{}\n```","ref":"TypeCheck.Builtin.Atom.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Binary","doc":"","ref":"TypeCheck.Builtin.Binary.html"},{"type":"type","title":"TypeCheck.Builtin.Binary.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.Binary.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.Binary.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.Binary{}\n```","ref":"TypeCheck.Builtin.Binary.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Bitstring","doc":"","ref":"TypeCheck.Builtin.Bitstring.html"},{"type":"type","title":"TypeCheck.Builtin.Bitstring.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.Bitstring.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.Bitstring.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.Bitstring{}\n```","ref":"TypeCheck.Builtin.Bitstring.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Boolean","doc":"","ref":"TypeCheck.Builtin.Boolean.html"},{"type":"type","title":"TypeCheck.Builtin.Boolean.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.Boolean.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.Boolean.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.Boolean{}\n```","ref":"TypeCheck.Builtin.Boolean.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.CompoundFixedMap","doc":"Special type for map-typespecs that contain a combination of fixed keys as well as an `optional(...)` or `required(...)` part.\n\nIts checks compile down to a combination of `TypeCheck.Builtin.FixedMap` and `TypeCheck.Builtin.Map`\nfor the fixed resp. non-fixed parts.","ref":"TypeCheck.Builtin.CompoundFixedMap.html"},{"type":"type","title":"TypeCheck.Builtin.CompoundFixedMap.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :not_a_map, %{}, any()}\n| {t(), :missing_keys, %{keys: list(atom())}, map()}\n| {t(), :superfluous_keys, %{keys: list(atom())}, map()}\n| {t(), :value_error, %{problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple()), key: any()},\n   map()}\n| {t(), :key_error, %{problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple()), key: any()},\n   map()}\n```","ref":"TypeCheck.Builtin.CompoundFixedMap.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.CompoundFixedMap.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.CompoundFixedMap{\n  fixed: TypeCheck.Builtin.FixedMap.t(),\n  flexible: TypeCheck.Builtin.Map.t() | TypeCheck.Builtin.OptionalFixedMap.t()\n}\n```","ref":"TypeCheck.Builtin.CompoundFixedMap.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.FixedList","doc":"Checks whether the value is a list with the expected elements\n\nOn failure returns a problem tuple with:\n  - `:not_a_list` if the value is not a list\n  - `:different_length` if the value is a list but not of equal size.\n  - `:element_error` if one of the elements does not match. The extra information contains in this case `:problem` and `:index` to indicate what and where the problem occurred.","ref":"TypeCheck.Builtin.FixedList.html"},{"type":"type","title":"TypeCheck.Builtin.FixedList.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :not_a_list, %{}, any()}\n| {t(), :different_length, %{expected_length: non_neg_integer()}, list()}\n| {t(), :element_error,\n   %{problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple()), index: non_neg_integer()},\n   list()}\n```","ref":"TypeCheck.Builtin.FixedList.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.FixedList.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.FixedList{element_types: list(TypeCheck.Type.t())}\n```","ref":"TypeCheck.Builtin.FixedList.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.FixedMap","doc":"Checks whether the value is a map with a required set of keys\n\nOn failure returns a problem tuple with:\n- `:not_a_map` if the value is not a map\n- `:missing_keys` if the value does not have all of the expected keys. The extra information contains in this case `:keys` with a list of keys that are missing.\n- `:superfluous_keys` if the value have any keys other than the expected keys. The extra information contains in this case `:keys` with a list of keys that are superfluous.\n- `:value_error` if one of the elements does not match. The extra information contains in this case `:problem` and `:key` to indicate what and where the problem occurred.","ref":"TypeCheck.Builtin.FixedMap.html"},{"type":"type","title":"TypeCheck.Builtin.FixedMap.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :not_a_map, %{}, any()}\n| {t(), :missing_keys, %{keys: list(atom())}, map()}\n| {t(), :superfluous_keys, %{keys: list(atom())}, map()}\n| {t(), :value_error, %{problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple()), key: any()},\n   map()}\n```","ref":"TypeCheck.Builtin.FixedMap.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.FixedMap.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.FixedMap{keypairs: list({term(), TypeCheck.Type.t()})}\n```","ref":"TypeCheck.Builtin.FixedMap.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.FixedTuple","doc":"","ref":"TypeCheck.Builtin.FixedTuple.html"},{"type":"type","title":"TypeCheck.Builtin.FixedTuple.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :not_a_tuple, %{}, any()}\n| {t(), :different_size, %{expected_size: integer()}, tuple()}\n| {t(), :element_error,\n   %{problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple()), index: integer()}, tuple()}\n```","ref":"TypeCheck.Builtin.FixedTuple.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.FixedTuple.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.FixedTuple{element_types: list(TypeCheck.Type.t())}\n```","ref":"TypeCheck.Builtin.FixedTuple.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Float","doc":"","ref":"TypeCheck.Builtin.Float.html"},{"type":"type","title":"TypeCheck.Builtin.Float.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.Float.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.Float.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.Float{}\n```","ref":"TypeCheck.Builtin.Float.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Function","doc":"","ref":"TypeCheck.Builtin.Function.html"},{"type":"function","title":"TypeCheck.Builtin.Function.contravariant_wrapper/2","doc":"","ref":"TypeCheck.Builtin.Function.html#contravariant_wrapper/2"},{"type":"function","title":"TypeCheck.Builtin.Function.param_check_code/3","doc":"","ref":"TypeCheck.Builtin.Function.html#param_check_code/3"},{"type":"type","title":"TypeCheck.Builtin.Function.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.Function.html#t:problem_tuple/0"},{"type":"opaque","title":"TypeCheck.Builtin.Function.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n`t` _(opaque type)_","ref":"TypeCheck.Builtin.Function.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Guarded","doc":"","ref":"TypeCheck.Builtin.Guarded.html"},{"type":"function","title":"TypeCheck.Builtin.Guarded.ast_gen/1","doc":"","ref":"TypeCheck.Builtin.Guarded.html#ast_gen/1"},{"type":"type","title":"TypeCheck.Builtin.Guarded.ast/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nast() :: wrap_with_gen(term(), &TypeCheck.Builtin.Guarded.ast_gen/1)\n```","ref":"TypeCheck.Builtin.Guarded.html#t:ast/0"},{"type":"type","title":"TypeCheck.Builtin.Guarded.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %TypeCheck.Builtin.Guarded{\n  type: TypeCheck.Type.t(),\n  guard: ast(),\n  original_module: module() | nil\n}\n```","ref":"TypeCheck.Builtin.Guarded.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.ImplementsProtocol","doc":"Checks whether there is a protocol implementation for this value.\n\nReturns a problem tuple with the reason `:no_match` otherwise.","ref":"TypeCheck.Builtin.ImplementsProtocol.html"},{"type":"type","title":"TypeCheck.Builtin.ImplementsProtocol.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.ImplementsProtocol.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.ImplementsProtocol.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.ImplementsProtocol{protocol: module()}\n```","ref":"TypeCheck.Builtin.ImplementsProtocol.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Integer","doc":"","ref":"TypeCheck.Builtin.Integer.html"},{"type":"type","title":"TypeCheck.Builtin.Integer.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.Integer.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.Integer.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.Integer{}\n```","ref":"TypeCheck.Builtin.Integer.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Lazy","doc":"","ref":"TypeCheck.Builtin.Lazy.html"},{"type":"function","title":"TypeCheck.Builtin.Lazy.lazily_expand_type/1","doc":"","ref":"TypeCheck.Builtin.Lazy.html#lazily_expand_type/1"},{"type":"type","title":"TypeCheck.Builtin.Lazy.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: TypeCheck.TypeError.Formatter.problem_tuple()\n```","ref":"TypeCheck.Builtin.Lazy.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.Lazy.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.Lazy{module: module(), function: atom(), arguments: list(term())}\n```","ref":"TypeCheck.Builtin.Lazy.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.List","doc":"","ref":"TypeCheck.Builtin.List.html"},{"type":"type","title":"TypeCheck.Builtin.List.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :not_a_list, %{}, any()}\n| {t(), :element_error,\n   %{problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple()), index: non_neg_integer()},\n   any()}\n```","ref":"TypeCheck.Builtin.List.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.List.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: t(TypeCheck.Type.t())\n```","ref":"TypeCheck.Builtin.List.html#t:t/0"},{"type":"type","title":"TypeCheck.Builtin.List.t/1","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt(element_type) :: %TypeCheck.Builtin.List{element_type: element_type}\n```","ref":"TypeCheck.Builtin.List.html#t:t/1"},{"type":"module","title":"TypeCheck.Builtin.Literal","doc":"","ref":"TypeCheck.Builtin.Literal.html"},{"type":"type","title":"TypeCheck.Builtin.Literal.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :not_same_value, %{}, value :: term()}\n```","ref":"TypeCheck.Builtin.Literal.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.Literal.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.Literal{value: term()}\n```","ref":"TypeCheck.Builtin.Literal.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Map","doc":"","ref":"TypeCheck.Builtin.Map.html"},{"type":"type","title":"TypeCheck.Builtin.Map.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :not_a_map, %{}, any()}\n| {t(), :key_error, %{problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple()), key: any()},\n   any()}\n| {t(), :value_error, %{problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple()), key: any()},\n   any()}\n```","ref":"TypeCheck.Builtin.Map.html#t:problem_tuple/0"},{"type":"opaque","title":"TypeCheck.Builtin.Map.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n`t` _(opaque type)_","ref":"TypeCheck.Builtin.Map.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.MaybeImproperList","doc":"","ref":"TypeCheck.Builtin.MaybeImproperList.html"},{"type":"function","title":"TypeCheck.Builtin.MaybeImproperList.empty?/1","doc":"","ref":"TypeCheck.Builtin.MaybeImproperList.html#empty?/1"},{"type":"type","title":"TypeCheck.Builtin.MaybeImproperList.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :not_a_list, %{}, any()}\n| {t(), :element_error,\n   %{problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple()), index: non_neg_integer()},\n   any()}\n| {t(), :terminator_error, %{problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple())}, any()}\n```","ref":"TypeCheck.Builtin.MaybeImproperList.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.MaybeImproperList.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: t(TypeCheck.Type.t(), TypeCheck.Type.t())\n```","ref":"TypeCheck.Builtin.MaybeImproperList.html#t:t/0"},{"type":"type","title":"TypeCheck.Builtin.MaybeImproperList.t/2","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt(element_type, terminator_type) :: %TypeCheck.Builtin.MaybeImproperList{element_type: element_type, terminator_type: terminator_type}\n```","ref":"TypeCheck.Builtin.MaybeImproperList.html#t:t/2"},{"type":"module","title":"TypeCheck.Builtin.NamedType","doc":"","ref":"TypeCheck.Builtin.NamedType.html"},{"type":"function","title":"TypeCheck.Builtin.NamedType.stringify_name/2","doc":"","ref":"TypeCheck.Builtin.NamedType.html#stringify_name/2"},{"type":"type","title":"TypeCheck.Builtin.NamedType.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :named_type, %{problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple())}, any()}\n```","ref":"TypeCheck.Builtin.NamedType.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.NamedType.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.NamedType{\n  name: atom(),\n  type: TypeCheck.Type.t(),\n  local: boolean(),\n  type_kind: :type | :typep | :opaque,\n  called_as: nil | {atom(), list(any())}\n}\n```","ref":"TypeCheck.Builtin.NamedType.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.NegInteger","doc":"","ref":"TypeCheck.Builtin.NegInteger.html"},{"type":"type","title":"TypeCheck.Builtin.NegInteger.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.NegInteger.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.NegInteger.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.NegInteger{}\n```","ref":"TypeCheck.Builtin.NegInteger.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.NonNegInteger","doc":"","ref":"TypeCheck.Builtin.NonNegInteger.html"},{"type":"type","title":"TypeCheck.Builtin.NonNegInteger.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.NonNegInteger.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.NonNegInteger.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.NonNegInteger{}\n```","ref":"TypeCheck.Builtin.NonNegInteger.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.None","doc":"The 'none' type has no inhabitants.\nIn other words, no value will typecheck against this type.\n\nThis means that we always return a problem tuple with `:no_match` as reason in its check.\n\nIt also means that the StreamData-generator will not generate any values;\ninstead, it will filter away values that would have been produced by `none()`,\nmeaning that if you attempt to use `none()` directly in a generator, you might get a `StreamData.FilterTooNarrowError`.\nHowever, it's still possible to combine it with other types like `:ok | :error | none()` and e.g. use the resulting generator of that.","ref":"TypeCheck.Builtin.None.html"},{"type":"type","title":"TypeCheck.Builtin.None.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, val :: any()}\n```","ref":"TypeCheck.Builtin.None.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.None.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.None{}\n```","ref":"TypeCheck.Builtin.None.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Number","doc":"","ref":"TypeCheck.Builtin.Number.html"},{"type":"type","title":"TypeCheck.Builtin.Number.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, val :: any()}\n```","ref":"TypeCheck.Builtin.Number.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.Number.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.Number{}\n```","ref":"TypeCheck.Builtin.Number.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.OneOf","doc":"","ref":"TypeCheck.Builtin.OneOf.html"},{"type":"type","title":"TypeCheck.Builtin.OneOf.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :all_failed, %{problems: list(lazy(TypeCheck.TypeError.Formatter.problem_tuple()))}, term()}\n```","ref":"TypeCheck.Builtin.OneOf.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.OneOf.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %TypeCheck.Builtin.OneOf{choices: list(TypeCheck.Type.t())}\n```","ref":"TypeCheck.Builtin.OneOf.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.OptionalFixedMap","doc":"Checks whether the value is a map with a optional set of keys\n\nOn failure returns a problem tuple with:\n- `:not_a_map` if the value is not a map\n- `:superfluous_keys` if the value have any keys other than the expected keys. The extra information contains in this case `:keys` with a list of keys that are superfluous.\n- `:value_error` if one of the elements does not match. The extra information contains in this case `:problem` and `:key` to indicate what and where the problem occurred.","ref":"TypeCheck.Builtin.OptionalFixedMap.html"},{"type":"type","title":"TypeCheck.Builtin.OptionalFixedMap.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :not_a_map, %{}, any()}\n| {t(), :superfluous_keys, %{keys: list(atom())}, map()}\n| {t(), :value_error, %{problem: lazy(TypeCheck.TypeError.Formatter.problem_tuple()), key: any()},\n   map()}\n```","ref":"TypeCheck.Builtin.OptionalFixedMap.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.OptionalFixedMap.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.OptionalFixedMap{keypairs: list({term(), TypeCheck.Type.t()})}\n```","ref":"TypeCheck.Builtin.OptionalFixedMap.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.PID","doc":"","ref":"TypeCheck.Builtin.PID.html"},{"type":"type","title":"TypeCheck.Builtin.PID.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.PID.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.PID.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.PID{}\n```","ref":"TypeCheck.Builtin.PID.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Port","doc":"Type to check whether the given input is any port.\n\nCheck `Port` for more information on ports.\n\nNOTE: The property testing generator will generate ports of the `cat` binary\nwhich is a sensible default as it will send back any binaries sent to it exactly.\nHowever, note that generated ports are not automatically closed.","ref":"TypeCheck.Builtin.Port.html"},{"type":"type","title":"TypeCheck.Builtin.Port.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.Port.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.Port.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.Port{}\n```","ref":"TypeCheck.Builtin.Port.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.PosInteger","doc":"","ref":"TypeCheck.Builtin.PosInteger.html"},{"type":"type","title":"TypeCheck.Builtin.PosInteger.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.PosInteger.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.PosInteger.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.PosInteger{}\n```","ref":"TypeCheck.Builtin.PosInteger.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Range","doc":"","ref":"TypeCheck.Builtin.Range.html"},{"type":"type","title":"TypeCheck.Builtin.Range.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :not_an_integer, %{}, any()} | {t(), :not_in_range, %{}, integer()}\n```","ref":"TypeCheck.Builtin.Range.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.Range.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.Range{range: %Range{first: integer(), last: integer(), step: 1}}\n```","ref":"TypeCheck.Builtin.Range.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Reference","doc":"Type to check whether the given input is a reference.\n\nElixir/Erlang uses references for two use-cases:\n1. As unique identifiers.\n2. To refer to resources created and returned by NIFs (to be passed to other NIFs of the same NIF module).\n\nThe property testing generator will generate arbitrary references using `Kernel.make_ref()`.\nTo property-test the second kind of data, you should create your own kind of generator\nthat calls the appropriate NIF.","ref":"TypeCheck.Builtin.Reference.html"},{"type":"type","title":"TypeCheck.Builtin.Reference.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.Reference.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.Reference.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.Reference{}\n```","ref":"TypeCheck.Builtin.Reference.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.SizedBitstring","doc":"","ref":"TypeCheck.Builtin.SizedBitstring.html"},{"type":"type","title":"TypeCheck.Builtin.SizedBitstring.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()} | {t(), :wrong_size, %{}, any()}\n```","ref":"TypeCheck.Builtin.SizedBitstring.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.SizedBitstring.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.SizedBitstring{\n  prefix_size: non_neg_integer(),\n  unit_size: nil | %{__struct__: Range, first: 1, last: 256, step: 1}\n}\n```","ref":"TypeCheck.Builtin.SizedBitstring.html#t:t/0"},{"type":"module","title":"TypeCheck.Builtin.Tuple","doc":"Checks whether the value is any tuple.\n\nReturns a problem tuple with the reason `:no_match` otherwise.","ref":"TypeCheck.Builtin.Tuple.html"},{"type":"type","title":"TypeCheck.Builtin.Tuple.problem_tuple/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nproblem_tuple :: {t(), :no_match, %{}, any()}\n```","ref":"TypeCheck.Builtin.Tuple.html#t:problem_tuple/0"},{"type":"type","title":"TypeCheck.Builtin.Tuple.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt :: %TypeCheck.Builtin.Tuple{}\n```","ref":"TypeCheck.Builtin.Tuple.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides","doc":"Contains a many common types that can be used as overrides for Elixir's standard library's 'Remote Types'.\n\nThis module complements `TypeCheck.Builtin`, contains all 'built-in' types of Elixir.\n\nImplementing TypeSpecs for all types of Elixir's standard library is a work-in-progress.\nSome TypeCheck-versions of the types are a little more general than the original version,\nto make up for functionality in TypeCheck which does not exist yet.\n\nSimply put, this means that you will never get a 'false positive' (a correct value not being accepted by a function),\nbut in very rare cases you might get a 'false negative' (an improper value passing the type-check.)","ref":"TypeCheck.DefaultOverrides.html"},{"type":"function","title":"TypeCheck.DefaultOverrides.default_overrides/0","doc":"Lists all overridden types in {module, function, arity} format.","ref":"TypeCheck.DefaultOverrides.html#default_overrides/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Access","doc":"","ref":"TypeCheck.DefaultOverrides.Access.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Access.access_fun/2","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\naccess_fun(data, current_value) :: get_fun(data) | get_and_update_fun(data, current_value)\n```","ref":"TypeCheck.DefaultOverrides.Access.html#t:access_fun/2"},{"type":"type","title":"TypeCheck.DefaultOverrides.Access.any_container/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nany_container() :: any()\n```","ref":"TypeCheck.DefaultOverrides.Access.html#t:any_container/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Access.container/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ncontainer() :: keyword() | struct() | map()\n```","ref":"TypeCheck.DefaultOverrides.Access.html#t:container/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Access.get_and_update_fun/2","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nget_and_update_fun(data, current_value) :: (:get_and_update, data, (term() -> term()) -> {current_value, new_data :: container()} | :pop)\n```","ref":"TypeCheck.DefaultOverrides.Access.html#t:get_and_update_fun/2"},{"type":"type","title":"TypeCheck.DefaultOverrides.Access.get_fun/1","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nget_fun(data) :: (:get, data, (term() -> term()) -> new_data :: container())\n```","ref":"TypeCheck.DefaultOverrides.Access.html#t:get_fun/1"},{"type":"type","title":"TypeCheck.DefaultOverrides.Access.key/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nkey() :: any()\n```","ref":"TypeCheck.DefaultOverrides.Access.html#t:key/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Access.nil_container/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nnil_container() :: nil\n```","ref":"TypeCheck.DefaultOverrides.Access.html#t:nil_container/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Access.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: container() | nil_container() | any_container()\n```","ref":"TypeCheck.DefaultOverrides.Access.html#t:t/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Access.value/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nvalue() :: any()\n```","ref":"TypeCheck.DefaultOverrides.Access.html#t:value/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Agent","doc":"","ref":"TypeCheck.DefaultOverrides.Agent.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Agent.agent/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nagent() :: pid() | {atom(), node()} | name()\n```","ref":"TypeCheck.DefaultOverrides.Agent.html#t:agent/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Agent.name/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nname() :: atom() | {:global, term()} | {:via, module(), term()}\n```","ref":"TypeCheck.DefaultOverrides.Agent.html#t:name/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Agent.on_start/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\non_start() :: {:ok, pid()} | {:error, {:already_started, pid()} | term()}\n```","ref":"TypeCheck.DefaultOverrides.Agent.html#t:on_start/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Agent.state/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nstate() :: term()\n```","ref":"TypeCheck.DefaultOverrides.Agent.html#t:state/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Application","doc":"","ref":"TypeCheck.DefaultOverrides.Application.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Application.app/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\napp() :: atom()\n```","ref":"TypeCheck.DefaultOverrides.Application.html#t:app/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Application.application_key/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\napplication_key() :: :start_phases\n| :mod\n| :applications\n| :optional_applications\n| :included_applications\n| :registered\n| :maxT\n| :maxP\n| :modules\n| :vsn\n| :id\n| :description\n```","ref":"TypeCheck.DefaultOverrides.Application.html#t:application_key/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Application.key/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nkey() :: atom()\n```","ref":"TypeCheck.DefaultOverrides.Application.html#t:key/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Application.restart_type/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nrestart_type() :: :permanent | :transient | :temporary\n```","ref":"TypeCheck.DefaultOverrides.Application.html#t:restart_type/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Application.start_type/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nstart_type() :: :normal | {:takeover, node()} | {:failover, node()}\n```","ref":"TypeCheck.DefaultOverrides.Application.html#t:start_type/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Application.state/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nstate() :: term()\n```","ref":"TypeCheck.DefaultOverrides.Application.html#t:state/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Application.value/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nvalue() :: term()\n```","ref":"TypeCheck.DefaultOverrides.Application.html#t:value/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Base","doc":"","ref":"TypeCheck.DefaultOverrides.Base.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Base.decode_case/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ndecode_case() :: :upper | :lower | :mixed\n```","ref":"TypeCheck.DefaultOverrides.Base.html#t:decode_case/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Base.encode_case/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nencode_case() :: :upper | :lower\n```","ref":"TypeCheck.DefaultOverrides.Base.html#t:encode_case/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Calendar","doc":"","ref":"TypeCheck.DefaultOverrides.Calendar.html"},{"type":"function","title":"TypeCheck.DefaultOverrides.Calendar.calendar_gen/0","doc":"","ref":"TypeCheck.DefaultOverrides.Calendar.html#calendar_gen/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.calendar/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ncalendar() :: wrap_with_gen(module(), &TypeCheck.DefaultOverrides.Calendar.calendar_gen/0)\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:calendar/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.date/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ndate() :: %{optional(any()) => any(), calendar: calendar(), year: year(), month: month(), day: day()}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:date/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.datetime/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ndatetime() :: %{\n  optional(any()) => any(),\n  calendar: calendar(),\n  year: year(),\n  month: month(),\n  day: day(),\n  hour: hour(),\n  minute: minute(),\n  second: second(),\n  microsecond: microsecond(),\n  time_zone: time_zone(),\n  zone_abbr: zone_abbr(),\n  utc_offset: utc_offset(),\n  std_offset: std_offset()\n}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:datetime/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.day/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nday() :: pos_integer()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:day/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.day_fraction/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nday_fraction() :: {parts_in_day :: non_neg_integer(), parts_per_day :: pos_integer()}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:day_fraction/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.day_of_era/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nday_of_era() :: {day :: non_neg_integer(), era()}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:day_of_era/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.day_of_week/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nday_of_week() :: non_neg_integer()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:day_of_week/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.era/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nera() :: non_neg_integer()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:era/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.hour/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nhour() :: non_neg_integer()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:hour/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.iso_days/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\niso_days() :: {days :: integer(), day_fraction()}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:iso_days/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.microsecond/0","doc":"","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:microsecond/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.minute/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nminute() :: non_neg_integer()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:minute/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.month/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nmonth() :: pos_integer()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:month/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.naive_datetime/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nnaive_datetime() :: %{\n  optional(any()) => any(),\n  calendar: calendar(),\n  year: year(),\n  month: month(),\n  day: day(),\n  hour: hour(),\n  minute: minute(),\n  second: second(),\n  microsecond: microsecond()\n}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:naive_datetime/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.second/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nsecond() :: non_neg_integer()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:second/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.std_offset/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nstd_offset() :: integer()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:std_offset/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.time/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ntime() :: %{\n  optional(any()) => any(),\n  hour: hour(),\n  minute: minute(),\n  second: second(),\n  microsecond: microsecond()\n}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:time/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.time_zone/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ntime_zone() :: TypeCheck.DefaultOverrides.String.t()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:time_zone/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.time_zone_database/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ntime_zone_database() :: module()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:time_zone_database/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.utc_offset/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nutc_offset() :: integer()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:utc_offset/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.week/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nweek() :: pos_integer()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:week/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.year/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nyear() :: integer()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:year/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.zone_abbr/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nzone_abbr() :: TypeCheck.DefaultOverrides.String.t()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.html#t:zone_abbr/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Calendar.ISO","doc":"","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.bce/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nbce() :: 0\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:bce/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.ce/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nce() :: 1\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:ce/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.day/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nday() :: %{__struct__: Range, first: 1, last: 31, step: 1}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:day/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.day_of_week/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nday_of_week() :: %{__struct__: Range, first: 1, last: 7, step: 1}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:day_of_week/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.day_of_year/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nday_of_year() :: %{__struct__: Range, first: 1, last: 366, step: 1}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:day_of_year/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.era/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nera() :: bce() | ce()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:era/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.hour/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nhour() :: %{__struct__: Range, first: 0, last: 23, step: 1}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:hour/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.microsecond/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nmicrosecond() :: {%{__struct__: Range, first: 0, last: 999_999, step: 1},\n %{__struct__: Range, first: 0, last: 6, step: 1}}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:microsecond/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.minute/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nminute() :: %{__struct__: Range, first: 0, last: 59, step: 1}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:minute/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.month/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nmonth() :: %{__struct__: Range, first: 1, last: 12, step: 1}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:month/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.quarter_of_year/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nquarter_of_year() :: %{__struct__: Range, first: 1, last: 4, step: 1}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:quarter_of_year/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.second/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nsecond() :: %{__struct__: Range, first: 0, last: 59, step: 1}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:second/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.weekday/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nweekday() :: :monday | :tuesday | :wednesday | :thursday | :friday | :saturday | :sunday\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:weekday/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.year/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nyear() :: %{__struct__: Range, first: -9999, last: 9999, step: 1}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:year/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.ISO.year_of_era/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nyear_of_era() :: {%{__struct__: Range, first: 1, last: 10000, step: 1}, era()}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.ISO.html#t:year_of_era/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Calendar.TimeZoneDatabase","doc":"","ref":"TypeCheck.DefaultOverrides.Calendar.TimeZoneDatabase.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.TimeZoneDatabase.time_zone_period/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ntime_zone_period() :: %{\n  optional(any()) => any(),\n  utc_offset: TypeCheck.DefaultOverrides.Calendar.utc_offset(),\n  std_offset: TypeCheck.DefaultOverrides.Calendar.std_offset(),\n  zone_abbr: TypeCheck.DefaultOverrides.Calendar.zone_abbr()\n}\n```","ref":"TypeCheck.DefaultOverrides.Calendar.TimeZoneDatabase.html#t:time_zone_period/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Calendar.TimeZoneDatabase.time_zone_period_limit/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ntime_zone_period_limit() :: TypeCheck.DefaultOverrides.Calendar.naive_datetime()\n```","ref":"TypeCheck.DefaultOverrides.Calendar.TimeZoneDatabase.html#t:time_zone_period_limit/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Code","doc":"","ref":"TypeCheck.DefaultOverrides.Code.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Code.binding/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nbinding() :: [{atom() | tuple(), any()}]\n```","ref":"TypeCheck.DefaultOverrides.Code.html#t:binding/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Code.Fragment","doc":"","ref":"TypeCheck.DefaultOverrides.Code.Fragment.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Code.Fragment.binding/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nbinding() :: [{atom() | tuple(), any()}]\n```","ref":"TypeCheck.DefaultOverrides.Code.Fragment.html#t:binding/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Collectable","doc":"","ref":"TypeCheck.DefaultOverrides.Collectable.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Collectable.command/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ncommand() :: {:cont, term()} | :done | :halt\n```","ref":"TypeCheck.DefaultOverrides.Collectable.html#t:command/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Collectable.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: impl(Collectable)\n```","ref":"TypeCheck.DefaultOverrides.Collectable.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Config.Provider","doc":"","ref":"TypeCheck.DefaultOverrides.Config.Provider.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Config.Provider.config/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nconfig() :: keyword()\n```","ref":"TypeCheck.DefaultOverrides.Config.Provider.html#t:config/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Config.Provider.config_path/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nconfig_path() :: {:system, binary(), binary()} | binary()\n```","ref":"TypeCheck.DefaultOverrides.Config.Provider.html#t:config_path/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Config.Provider.state/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nstate() :: term()\n```","ref":"TypeCheck.DefaultOverrides.Config.Provider.html#t:state/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Date","doc":"","ref":"TypeCheck.DefaultOverrides.Date.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Date.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %Date{\n  calendar: TypeCheck.DefaultOverrides.Calendar.calendar(),\n  day: TypeCheck.DefaultOverrides.Calendar.day(),\n  month: TypeCheck.DefaultOverrides.Calendar.month(),\n  year: TypeCheck.DefaultOverrides.Calendar.year()\n}\n```","ref":"TypeCheck.DefaultOverrides.Date.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Date.Range","doc":"","ref":"TypeCheck.DefaultOverrides.Date.Range.html"},{"type":"opaque","title":"TypeCheck.DefaultOverrides.Date.Range.iso_days/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n`iso_days()` _(opaque type)_","ref":"TypeCheck.DefaultOverrides.Date.Range.html#t:iso_days/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Date.Range.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %Date.Range{\n  first: TypeCheck.DefaultOverrides.Date.t(),\n  first_in_iso_days: iso_days(),\n  last: TypeCheck.DefaultOverrides.Date.t(),\n  last_in_iso_days: iso_days(),\n  step: pos_integer() | neg_integer()\n}\n```","ref":"TypeCheck.DefaultOverrides.Date.Range.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.DateTime","doc":"","ref":"TypeCheck.DefaultOverrides.DateTime.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.DateTime.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %DateTime{\n  calendar: TypeCheck.DefaultOverrides.Calendar.calendar(),\n  day: TypeCheck.DefaultOverrides.Calendar.day(),\n  hour: TypeCheck.DefaultOverrides.Calendar.hour(),\n  microsecond: TypeCheck.DefaultOverrides.Calendar.microsecond(),\n  minute: TypeCheck.DefaultOverrides.Calendar.minute(),\n  month: TypeCheck.DefaultOverrides.Calendar.month(),\n  second: TypeCheck.DefaultOverrides.Calendar.second(),\n  std_offset: TypeCheck.DefaultOverrides.Calendar.std_offset(),\n  time_zone: TypeCheck.DefaultOverrides.Calendar.time_zone(),\n  utc_offset: TypeCheck.DefaultOverrides.Calendar.utc_offset(),\n  year: TypeCheck.DefaultOverrides.Calendar.year(),\n  zone_abbr: TypeCheck.DefaultOverrides.Calendar.zone_abbr()\n}\n```","ref":"TypeCheck.DefaultOverrides.DateTime.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.DynamicSupervisor","doc":"","ref":"TypeCheck.DefaultOverrides.DynamicSupervisor.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.DynamicSupervisor.init_option/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ninit_option() :: {:strategy, strategy()}\n| {:max_restarts, non_neg_integer()}\n| {:max_seconds, pos_integer()}\n| {:max_children, non_neg_integer() | :infinity}\n| {:extra_arguments, [term()]}\n```","ref":"TypeCheck.DefaultOverrides.DynamicSupervisor.html#t:init_option/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.DynamicSupervisor.on_start_child/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\non_start_child() :: {:ok, pid()}\n| {:ok, pid(), info :: term()}\n| :ignore\n| {:error, {:already_started, pid()} | :max_children | term()}\n```","ref":"TypeCheck.DefaultOverrides.DynamicSupervisor.html#t:on_start_child/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.DynamicSupervisor.option/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\noption() :: TypeCheck.DefaultOverrides.GenServer.option()\n```","ref":"TypeCheck.DefaultOverrides.DynamicSupervisor.html#t:option/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.DynamicSupervisor.strategy/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nstrategy() :: :one_for_one\n```","ref":"TypeCheck.DefaultOverrides.DynamicSupervisor.html#t:strategy/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.DynamicSupervisor.sup_flags/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nsup_flags() :: %{\n  strategy: strategy(),\n  intensity: non_neg_integer(),\n  period: pos_integer(),\n  max_children: non_neg_integer() | :infinity,\n  extra_arguments: [term()]\n}\n```","ref":"TypeCheck.DefaultOverrides.DynamicSupervisor.html#t:sup_flags/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Enum","doc":"","ref":"TypeCheck.DefaultOverrides.Enum.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Enum.acc/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nacc() :: any()\n```","ref":"TypeCheck.DefaultOverrides.Enum.html#t:acc/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Enum.default/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ndefault() :: any()\n```","ref":"TypeCheck.DefaultOverrides.Enum.html#t:default/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Enum.element/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nelement() :: any()\n```","ref":"TypeCheck.DefaultOverrides.Enum.html#t:element/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Enum.index/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nindex() :: integer()\n```","ref":"TypeCheck.DefaultOverrides.Enum.html#t:index/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Enum.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: TypeCheck.DefaultOverrides.Enumerable.t()\n```","ref":"TypeCheck.DefaultOverrides.Enum.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Enumerable","doc":"","ref":"TypeCheck.DefaultOverrides.Enumerable.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Enumerable.acc/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nacc() :: {:cont, term()} | {:halt, term()} | {:suspend, term()}\n```","ref":"TypeCheck.DefaultOverrides.Enumerable.html#t:acc/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Enumerable.continuation/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ncontinuation() :: (acc() -> result())\n```","ref":"TypeCheck.DefaultOverrides.Enumerable.html#t:continuation/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Enumerable.reducer/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nreducer() :: (element :: term(), current_acc :: acc() -> updated_acc :: acc())\n```","ref":"TypeCheck.DefaultOverrides.Enumerable.html#t:reducer/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Enumerable.result/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nresult() :: {:done, term()} | {:halted, term()} | {:suspended, term(), continuation()}\n```","ref":"TypeCheck.DefaultOverrides.Enumerable.html#t:result/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Enumerable.slicing_fun/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nslicing_fun() :: (start :: non_neg_integer(), length :: pos_integer() -> [term()])\n```","ref":"TypeCheck.DefaultOverrides.Enumerable.html#t:slicing_fun/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Enumerable.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: impl(Enumerable)\n```","ref":"TypeCheck.DefaultOverrides.Enumerable.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Erlang.Binary","doc":"","ref":"TypeCheck.DefaultOverrides.Erlang.Binary.html"},{"type":"opaque","title":"TypeCheck.DefaultOverrides.Erlang.Binary.cp/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n`cp()` _(opaque type)_","ref":"TypeCheck.DefaultOverrides.Erlang.Binary.html#t:cp/0"},{"type":"opaque","title":"TypeCheck.DefaultOverrides.Erlang.Binary.part/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n`part()` _(opaque type)_","ref":"TypeCheck.DefaultOverrides.Erlang.Binary.html#t:part/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Erlang.Calendar","doc":"","ref":"TypeCheck.DefaultOverrides.Erlang.Calendar.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Erlang.Calendar.date/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ndate() :: {year(), month(), day()}\n```","ref":"TypeCheck.DefaultOverrides.Erlang.Calendar.html#t:date/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Erlang.Calendar.datetime1970/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ndatetime1970() :: {{year1970(), month(), day()}, time()}\n```","ref":"TypeCheck.DefaultOverrides.Erlang.Calendar.html#t:datetime1970/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Erlang.Calendar.datetime/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ndatetime() :: {date(), time()}\n```","ref":"TypeCheck.DefaultOverrides.Erlang.Calendar.html#t:datetime/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Erlang.Calendar.time/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ntime() :: {hour(), minute(), second()}\n```","ref":"TypeCheck.DefaultOverrides.Erlang.Calendar.html#t:time/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Erlang.Inet","doc":"","ref":"TypeCheck.DefaultOverrides.Erlang.Inet.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Erlang.Inet.port_number/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nport_number() :: %{__struct__: Range, first: 0, last: 65535, step: 1}\n```","ref":"TypeCheck.DefaultOverrides.Erlang.Inet.html#t:port_number/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Exception","doc":"","ref":"TypeCheck.DefaultOverrides.Exception.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Exception.arity_or_args/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\narity_or_args() :: non_neg_integer() | list()\n```","ref":"TypeCheck.DefaultOverrides.Exception.html#t:arity_or_args/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Exception.kind/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nkind() :: :error | non_error_kind()\n```","ref":"TypeCheck.DefaultOverrides.Exception.html#t:kind/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Exception.location/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nlocation() :: keyword()\n```","ref":"TypeCheck.DefaultOverrides.Exception.html#t:location/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Exception.non_error_kind/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nnon_error_kind() :: :exit | :throw | {:EXIT, pid()}\n```","ref":"TypeCheck.DefaultOverrides.Exception.html#t:non_error_kind/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Exception.stacktrace/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nstacktrace() :: [stacktrace_entry()]\n```","ref":"TypeCheck.DefaultOverrides.Exception.html#t:stacktrace/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Exception.stacktrace_entry/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nstacktrace_entry() :: {module(), atom(), arity_or_args(), location()} | {(... -> any()), arity_or_args(), location()}\n```","ref":"TypeCheck.DefaultOverrides.Exception.html#t:stacktrace_entry/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Exception.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %{:__struct__ => module(), :__exception__ => true, optional(atom()) => any()}\n```","ref":"TypeCheck.DefaultOverrides.Exception.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.File","doc":"","ref":"TypeCheck.DefaultOverrides.File.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.File.encoding_mode/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nencoding_mode() :: :utf8\n| {:encoding,\n   :latin1\n   | :unicode\n   | :utf8\n   | :utf16\n   | :utf32\n   | {:utf16, :big | :little}\n   | {:utf32, :big | :little}}\n```","ref":"TypeCheck.DefaultOverrides.File.html#t:encoding_mode/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.File.erlang_time/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nerlang_time() :: {{year :: non_neg_integer(), month :: %{__struct__: Range, first: 1, last: 12, step: 1},\n  day :: %{__struct__: Range, first: 1, last: 31, step: 1}},\n {hour :: %{__struct__: Range, first: 0, last: 23, step: 1},\n  minute :: %{__struct__: Range, first: 0, last: 59, step: 1},\n  second :: %{__struct__: Range, first: 0, last: 59, step: 1}}}\n```","ref":"TypeCheck.DefaultOverrides.File.html#t:erlang_time/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.File.fd/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nfd :: {:file_descriptor, module :: module(), data :: term()}\n```","ref":"TypeCheck.DefaultOverrides.File.html#t:fd/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.File.io_device/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nio_device() :: pid() | fd()\n```","ref":"TypeCheck.DefaultOverrides.File.html#t:io_device/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.File.mode/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nmode() :: :append\n| :binary\n| :charlist\n| :compressed\n| :delayed_write\n| :exclusive\n| :raw\n| :read\n| :read_ahead\n| :sync\n| :write\n| {:read_ahead, pos_integer()}\n| {:delayed_write, non_neg_integer(), non_neg_integer()}\n| encoding_mode()\n```","ref":"TypeCheck.DefaultOverrides.File.html#t:mode/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.File.posix/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nposix :: :eacces\n| :eagain\n| :ebadf\n| :ebadmsg\n| :ebusy\n| :edeadlk\n| :edeadlock\n| :edquot\n| :eexist\n| :efault\n| :efbig\n| :eftype\n| :eintr\n| :einval\n| :eio\n| :eisdir\n| :eloop\n| :emfile\n| :emlink\n| :emultihop\n| :enametoolong\n| :enfile\n| :enobufs\n| :enodev\n| :enolck\n| :enolink\n| :enoent\n| :enomem\n| :enospc\n| :enosr\n| :enostr\n| :enosys\n| :enotblk\n| :enotdir\n| :enotsup\n| :enxio\n| :eopnotsupp\n| :eoverflow\n| :eperm\n| :epipe\n| :erange\n| :erofs\n| :espipe\n| :esrch\n| :estale\n| :etxtbsy\n| :exdev\n```","ref":"TypeCheck.DefaultOverrides.File.html#t:posix/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.File.posix_time/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nposix_time() :: integer()\n```","ref":"TypeCheck.DefaultOverrides.File.html#t:posix_time/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.File.stat_options/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nstat_options() :: [time: :local | :universal | :posix]\n```","ref":"TypeCheck.DefaultOverrides.File.html#t:stat_options/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.File.stream_mode/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nstream_mode() :: encoding_mode()\n| :append\n| :compressed\n| :trim_bom\n| {:read_ahead, pos_integer() | false}\n| {:delayed_write, non_neg_integer(), non_neg_integer()}\n```","ref":"TypeCheck.DefaultOverrides.File.html#t:stream_mode/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.File.Stat","doc":"","ref":"TypeCheck.DefaultOverrides.File.Stat.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.File.Stat.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %File.Stat{\n  access: :read | :write | :read_write | :none,\n  atime: TypeCheck.DefaultOverrides.Erlang.Calendar.datetime() | integer(),\n  ctime: TypeCheck.DefaultOverrides.Erlang.Calendar.datetime() | integer(),\n  gid: non_neg_integer(),\n  inode: non_neg_integer(),\n  links: non_neg_integer(),\n  major_device: non_neg_integer(),\n  minor_device: non_neg_integer(),\n  mode: non_neg_integer(),\n  mtime: TypeCheck.DefaultOverrides.Erlang.Calendar.datetime() | integer(),\n  size: non_neg_integer(),\n  type: :device | :directory | :regular | :other | :symlink,\n  uid: non_neg_integer()\n}\n```","ref":"TypeCheck.DefaultOverrides.File.Stat.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.File.Stream","doc":"","ref":"TypeCheck.DefaultOverrides.File.Stream.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.File.Stream.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %File.Stream{line_or_bytes: term(), modes: term(), path: term(), raw: term()}\n```","ref":"TypeCheck.DefaultOverrides.File.Stream.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Float","doc":"","ref":"TypeCheck.DefaultOverrides.Float.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Float.precision_range/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nprecision_range() :: %{__struct__: Range, first: 0, last: 15, step: 1}\n```","ref":"TypeCheck.DefaultOverrides.Float.html#t:precision_range/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Function","doc":"","ref":"TypeCheck.DefaultOverrides.Function.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Function.information/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ninformation() :: :arity | :env | :index | :module | :name | :new_index | :new_uniq | :pid | :type | :uniq\n```","ref":"TypeCheck.DefaultOverrides.Function.html#t:information/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.GenServer","doc":"","ref":"TypeCheck.DefaultOverrides.GenServer.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.GenServer.debug/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ndebug() :: [:trace | :log | :statistics | {:log_to_file, TypeCheck.DefaultOverrides.Path.t()}]\n```","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:debug/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.GenServer.from/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nfrom() :: {pid(), tag :: term()}\n```","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:from/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.GenServer.name/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nname() :: atom() | {:global, term()} | {:via, module(), term()}\n```","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:name/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.GenServer.on_start/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\non_start() :: {:ok, pid()} | :ignore | {:error, {:already_started, pid()} | term()}\n```","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:on_start/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.GenServer.option/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\noption() :: {:debug, debug()}\n| {:name, name()}\n| {:timeout, timeout()}\n| {:spawn_opt, [TypeCheck.DefaultOverrides.Process.spawn_opt()]}\n| {:hibernate_after, timeout()}\n```","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:option/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.GenServer.options/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\noptions() :: [option()]\n```","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:options/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.GenServer.server/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nserver() :: pid() | name() | {atom(), node()}\n```","ref":"TypeCheck.DefaultOverrides.GenServer.html#t:server/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.IO","doc":"","ref":"TypeCheck.DefaultOverrides.IO.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.IO.chardata/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nchardata() :: TypeCheck.DefaultOverrides.String.t()\n| maybe_improper_list(char() | chardata(), TypeCheck.DefaultOverrides.String.t() | [])\n```","ref":"TypeCheck.DefaultOverrides.IO.html#t:chardata/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.IO.device/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ndevice() :: atom() | pid()\n```","ref":"TypeCheck.DefaultOverrides.IO.html#t:device/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.IO.nodata/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nnodata() :: {:error, term()} | :eof\n```","ref":"TypeCheck.DefaultOverrides.IO.html#t:nodata/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.IO.ANSI","doc":"","ref":"TypeCheck.DefaultOverrides.IO.ANSI.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.IO.ANSI.ansicode/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nansicode() :: atom()\n```","ref":"TypeCheck.DefaultOverrides.IO.ANSI.html#t:ansicode/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.IO.ANSI.ansidata/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nansidata() :: ansilist() | ansicode() | binary()\n```","ref":"TypeCheck.DefaultOverrides.IO.ANSI.html#t:ansidata/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.IO.ANSI.ansilist/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nansilist() :: maybe_improper_list(\n  char() | ansicode() | binary() | ansilist(),\n  binary() | ansicode() | []\n)\n```","ref":"TypeCheck.DefaultOverrides.IO.ANSI.html#t:ansilist/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.IO.Stream","doc":"","ref":"TypeCheck.DefaultOverrides.IO.Stream.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.IO.Stream.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %IO.Stream{device: term(), line_or_bytes: term(), raw: term()}\n```","ref":"TypeCheck.DefaultOverrides.IO.Stream.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Inspect","doc":"","ref":"TypeCheck.DefaultOverrides.Inspect.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Inspect.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: impl(Inspect)\n```","ref":"TypeCheck.DefaultOverrides.Inspect.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Inspect.Algebra","doc":"","ref":"TypeCheck.DefaultOverrides.Inspect.Algebra.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Inspect.Algebra.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: binary()\n| :doc_line\n| :doc_nil\n| doc_break()\n| doc_collapse()\n| doc_color()\n| doc_cons()\n| doc_fits()\n| doc_force()\n| doc_group()\n| doc_nest()\n| doc_string()\n```","ref":"TypeCheck.DefaultOverrides.Inspect.Algebra.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Inspect.Opts","doc":"","ref":"TypeCheck.DefaultOverrides.Inspect.Opts.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Inspect.Opts.color_key/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ncolor_key() :: atom()\n```","ref":"TypeCheck.DefaultOverrides.Inspect.Opts.html#t:color_key/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Inspect.Opts.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %Inspect.Opts{\n  base: :decimal | :binary | :hex | :octal,\n  binaries: :infer | :as_binaries | :as_strings,\n  char_lists: :infer | :as_lists | :as_char_lists,\n  charlists: :infer | :as_lists | :as_charlists,\n  custom_options: keyword(),\n  inspect_fun: (any(), t() -> TypeCheck.DefaultOverrides.Inspect.Algebra.t()),\n  limit: non_neg_integer() | :infinity,\n  pretty: boolean(),\n  printable_limit: non_neg_integer() | :infinity,\n  safe: boolean(),\n  structs: boolean(),\n  syntax_colors: [{color_key(), TypeCheck.DefaultOverrides.IO.ANSI.ansidata()}],\n  syntax_colors: [{color_key(), any()}],\n  width: non_neg_integer() | :infinity\n}\n```","ref":"TypeCheck.DefaultOverrides.Inspect.Opts.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler","doc":"","ref":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.error/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nerror() :: {file :: TypeCheck.DefaultOverrides.Path.t(), line(),\n message :: TypeCheck.DefaultOverrides.String.t()}\n```","ref":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.html#t:error/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.line/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nline() :: non_neg_integer()\n```","ref":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.html#t:line/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.location/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nlocation() :: line() | {line(), column :: non_neg_integer()}\n```","ref":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.html#t:location/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.warning/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nwarning() :: {file :: TypeCheck.DefaultOverrides.Path.t(), location(),\n message :: TypeCheck.DefaultOverrides.String.t()}\n```","ref":"TypeCheck.DefaultOverrides.Kernel.ParallelCompiler.html#t:warning/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Keyword","doc":"","ref":"TypeCheck.DefaultOverrides.Keyword.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Keyword.key/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nkey() :: atom()\n```","ref":"TypeCheck.DefaultOverrides.Keyword.html#t:key/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Keyword.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: [{key(), value()}]\n```","ref":"TypeCheck.DefaultOverrides.Keyword.html#t:t/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Keyword.t/1","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt(value) :: [{key(), value}]\n```","ref":"TypeCheck.DefaultOverrides.Keyword.html#t:t/1"},{"type":"type","title":"TypeCheck.DefaultOverrides.Keyword.value/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nvalue() :: any()\n```","ref":"TypeCheck.DefaultOverrides.Keyword.html#t:value/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.List.Chars","doc":"","ref":"TypeCheck.DefaultOverrides.List.Chars.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.List.Chars.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: impl(List.Chars)\n```","ref":"TypeCheck.DefaultOverrides.List.Chars.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Macro","doc":"","ref":"TypeCheck.DefaultOverrides.Macro.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Macro.captured_remote_function/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ncaptured_remote_function() :: (... -> any())\n```","ref":"TypeCheck.DefaultOverrides.Macro.html#t:captured_remote_function/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Macro.input/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ninput() :: input_expr() | {lazy(input()), lazy(input())} | [lazy(input())] | atom() | number() | binary()\n```","ref":"TypeCheck.DefaultOverrides.Macro.html#t:input/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Macro.metadata/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nmetadata() :: keyword()\n```","ref":"TypeCheck.DefaultOverrides.Macro.html#t:metadata/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Macro.output/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\noutput() :: output_expr()\n| {lazy(output()), lazy(output())}\n| [lazy(output())]\n| atom()\n| number()\n| binary()\n| captured_remote_function()\n| pid()\n```","ref":"TypeCheck.DefaultOverrides.Macro.html#t:output/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Macro.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: input()\n```","ref":"TypeCheck.DefaultOverrides.Macro.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Macro.Env","doc":"","ref":"TypeCheck.DefaultOverrides.Macro.Env.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Macro.Env.context/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ncontext() :: :match | :guard | nil\n```","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:context/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Macro.Env.context_modules/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ncontext_modules() :: [module()]\n```","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:context_modules/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Macro.Env.file/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nfile() :: binary()\n```","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:file/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Macro.Env.line/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nline() :: non_neg_integer()\n```","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:line/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Macro.Env.name_arity/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nname_arity() :: {atom(), arity()}\n```","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:name_arity/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Macro.Env.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %Macro.Env{\n  aliases: aliases(),\n  context: context(),\n  context_modules: context_modules(),\n  file: file(),\n  function: name_arity() | nil,\n  functions: functions(),\n  lexical_tracker: lexical_tracker(),\n  line: line(),\n  macro_aliases: macro_aliases(),\n  macros: macros(),\n  module: module(),\n  requires: requires(),\n  tracers: tracers(),\n  versioned_vars: versioned_vars()\n}\n```","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:t/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Macro.Env.variable/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nvariable() :: {atom(), atom() | term()}\n```","ref":"TypeCheck.DefaultOverrides.Macro.Env.html#t:variable/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Map","doc":"","ref":"TypeCheck.DefaultOverrides.Map.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Map.key/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nkey() :: any()\n```","ref":"TypeCheck.DefaultOverrides.Map.html#t:key/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Map.value/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nvalue() :: any()\n```","ref":"TypeCheck.DefaultOverrides.Map.html#t:value/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.MapSet","doc":"","ref":"TypeCheck.DefaultOverrides.MapSet.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.MapSet.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: t(term())\n```","ref":"TypeCheck.DefaultOverrides.MapSet.html#t:t/0"},{"type":"opaque","title":"TypeCheck.DefaultOverrides.MapSet.t/1","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n`t(value)` _(opaque type)_","ref":"TypeCheck.DefaultOverrides.MapSet.html#t:t/1"},{"type":"type","title":"TypeCheck.DefaultOverrides.MapSet.value/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nvalue() :: term()\n```","ref":"TypeCheck.DefaultOverrides.MapSet.html#t:value/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Module","doc":"","ref":"TypeCheck.DefaultOverrides.Module.html"},{"type":"opaque","title":"TypeCheck.DefaultOverrides.Module.def_kind/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n`def_kind()` _(opaque type)_","ref":"TypeCheck.DefaultOverrides.Module.html#t:def_kind/0"},{"type":"opaque","title":"TypeCheck.DefaultOverrides.Module.definition/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n`definition()` _(opaque type)_","ref":"TypeCheck.DefaultOverrides.Module.html#t:definition/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.NaiveDateTime","doc":"","ref":"TypeCheck.DefaultOverrides.NaiveDateTime.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.NaiveDateTime.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %NaiveDateTime{\n  calendar: TypeCheck.DefaultOverrides.Calendar.calendar(),\n  day: TypeCheck.DefaultOverrides.Calendar.day(),\n  hour: TypeCheck.DefaultOverrides.Calendar.hour(),\n  microsecond: TypeCheck.DefaultOverrides.Calendar.microsecond(),\n  minute: TypeCheck.DefaultOverrides.Calendar.minute(),\n  month: TypeCheck.DefaultOverrides.Calendar.month(),\n  second: TypeCheck.DefaultOverrides.Calendar.second(),\n  year: TypeCheck.DefaultOverrides.Calendar.year()\n}\n```","ref":"TypeCheck.DefaultOverrides.NaiveDateTime.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Node","doc":"","ref":"TypeCheck.DefaultOverrides.Node.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Node.state/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nstate() :: :visible | :hidden | :connected | :this | :known\n```","ref":"TypeCheck.DefaultOverrides.Node.html#t:state/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Node.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: node()\n```","ref":"TypeCheck.DefaultOverrides.Node.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.OptionParser","doc":"","ref":"TypeCheck.DefaultOverrides.OptionParser.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.OptionParser.argv/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nargv() :: list(TypeCheck.DefaultOverrides.String.t())\n```","ref":"TypeCheck.DefaultOverrides.OptionParser.html#t:argv/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.OptionParser.errors/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nerrors() :: list({TypeCheck.DefaultOverrides.String.t(), TypeCheck.DefaultOverrides.String.t() | nil})\n```","ref":"TypeCheck.DefaultOverrides.OptionParser.html#t:errors/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.OptionParser.options/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\noptions() :: list({:switches, keyword()} | {:strict, keyword()} | {:aliases, keyword()})\n```","ref":"TypeCheck.DefaultOverrides.OptionParser.html#t:options/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.OptionParser.parsed/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nparsed() :: keyword()\n```","ref":"TypeCheck.DefaultOverrides.OptionParser.html#t:parsed/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Path","doc":"","ref":"TypeCheck.DefaultOverrides.Path.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Path.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: TypeCheck.DefaultOverrides.IO.chardata()\n```","ref":"TypeCheck.DefaultOverrides.Path.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Port","doc":"","ref":"TypeCheck.DefaultOverrides.Port.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Port.name/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nname() :: {:spawn, charlist() | binary()}\n| {:spawn_driver, charlist() | binary()}\n| {:spawn_executable, charlist() | atom()}\n| {:fd, non_neg_integer(), non_neg_integer()}\n```","ref":"TypeCheck.DefaultOverrides.Port.html#t:name/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Process","doc":"","ref":"TypeCheck.DefaultOverrides.Process.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Process.dest/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ndest() :: pid() | port() | (registered_name :: atom()) | {registered_name :: atom(), node()}\n```","ref":"TypeCheck.DefaultOverrides.Process.html#t:dest/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Process.spawn_opt/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nspawn_opt() :: :link\n| :monitor\n| {:monitor, monitor_option()}\n| {:priority, :low | :normal | :high}\n| {:fullsweep_after, non_neg_integer()}\n| {:min_heap_size, non_neg_integer()}\n| {:min_bin_vheap_size, non_neg_integer()}\n| {:max_heap_size, heap_size()}\n| {:message_queue_data, :off_heap | :on_heap}\n```","ref":"TypeCheck.DefaultOverrides.Process.html#t:spawn_opt/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Process.spawn_opts/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nspawn_opts() :: [spawn_opt()]\n```","ref":"TypeCheck.DefaultOverrides.Process.html#t:spawn_opts/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Range","doc":"","ref":"TypeCheck.DefaultOverrides.Range.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Range.limit/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nlimit() :: integer()\n```","ref":"TypeCheck.DefaultOverrides.Range.html#t:limit/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Range.step/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nstep() :: pos_integer() | neg_integer()\n```","ref":"TypeCheck.DefaultOverrides.Range.html#t:step/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Range.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %Range{first: limit(), last: limit(), step: step()}\n```","ref":"TypeCheck.DefaultOverrides.Range.html#t:t/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Range.t/2","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt(first, last) :: %Range{first: first, last: last, step: step()}\n```","ref":"TypeCheck.DefaultOverrides.Range.html#t:t/2"},{"type":"module","title":"TypeCheck.DefaultOverrides.Regex","doc":"","ref":"TypeCheck.DefaultOverrides.Regex.html"},{"type":"function","title":"TypeCheck.DefaultOverrides.Regex.regex_gen/0","doc":"","ref":"TypeCheck.DefaultOverrides.Regex.html#regex_gen/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Regex.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: wrap_with_gen(\n  %Regex{opts: binary(), re_pattern: term(), re_version: term(), source: binary()},\n  &TypeCheck.DefaultOverrides.Regex.regex_gen/0\n)\n```","ref":"TypeCheck.DefaultOverrides.Regex.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Registry","doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Registry.body/0","doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:body/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Registry.guard/0","doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:guard/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Registry.guards/0","doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:guards/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Registry.key/0","doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:key/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Registry.keys/0","doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:keys/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Registry.match_pattern/0","doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:match_pattern/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Registry.meta_key/0","doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:meta_key/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Registry.meta_value/0","doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:meta_value/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Registry.registry/0","doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:registry/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Registry.spec/0","doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:spec/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Registry.start_option/0","doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:start_option/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Registry.value/0","doc":"","ref":"TypeCheck.DefaultOverrides.Registry.html#t:value/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Stream","doc":"","ref":"TypeCheck.DefaultOverrides.Stream.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Stream.acc/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nacc() :: any()\n```","ref":"TypeCheck.DefaultOverrides.Stream.html#t:acc/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Stream.default/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ndefault() :: any()\n```","ref":"TypeCheck.DefaultOverrides.Stream.html#t:default/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Stream.element/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nelement() :: any()\n```","ref":"TypeCheck.DefaultOverrides.Stream.html#t:element/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Stream.index/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nindex() :: non_neg_integer()\n```","ref":"TypeCheck.DefaultOverrides.Stream.html#t:index/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Stream.timer/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ntimer() :: non_neg_integer() | :infinity\n```","ref":"TypeCheck.DefaultOverrides.Stream.html#t:timer/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.String","doc":"","ref":"TypeCheck.DefaultOverrides.String.html"},{"type":"function","title":"TypeCheck.DefaultOverrides.String.printable_string_gen/0","doc":"","ref":"TypeCheck.DefaultOverrides.String.html#printable_string_gen/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.String.codepoint/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ncodepoint() :: t()\n```","ref":"TypeCheck.DefaultOverrides.String.html#t:codepoint/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.String.grapheme/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ngrapheme() :: t()\n```","ref":"TypeCheck.DefaultOverrides.String.html#t:grapheme/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.String.pattern/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\npattern() :: t() | [t()] | TypeCheck.DefaultOverrides.Erlang.Binary.cp()\n```","ref":"TypeCheck.DefaultOverrides.String.html#t:pattern/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.String.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: wrap_with_gen(binary(), &TypeCheck.DefaultOverrides.String.printable_string_gen/0)\n```","ref":"TypeCheck.DefaultOverrides.String.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.String.Chars","doc":"","ref":"TypeCheck.DefaultOverrides.String.Chars.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.String.Chars.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: impl(String.Chars)\n```","ref":"TypeCheck.DefaultOverrides.String.Chars.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Supervisor","doc":"","ref":"TypeCheck.DefaultOverrides.Supervisor.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Supervisor.child/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nchild() :: pid() | :undefined\n```","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:child/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Supervisor.child_spec/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nchild_spec() :: %{\n  :id => atom() | term(),\n  :start => {module(), atom(), [term()]},\n  optional(:restart) => :permanent | :transient | :temporary,\n  optional(:shutdown) => timeout() | :brutal_kill,\n  optional(:type) => :worker | :supervisor,\n  optional(:modules) => [module()] | :dynamic\n}\n```","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:child_spec/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Supervisor.init_option/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ninit_option() :: {:strategy, strategy()} | {:max_restarts, non_neg_integer()} | {:max_seconds, pos_integer()}\n```","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:init_option/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Supervisor.name/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nname() :: atom() | {:global, term()} | {:via, module(), term()}\n```","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:name/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Supervisor.on_start/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\non_start() :: {:ok, pid()} | :ignore | {:error, {:already_started, pid()} | {:shutdown, term()} | term()}\n```","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:on_start/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Supervisor.on_start_child/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\non_start_child() :: {:ok, child()}\n| {:ok, child(), info :: term()}\n| {:error, {:already_started, child()} | :already_present | term()}\n```","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:on_start_child/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Supervisor.option/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\noption() :: {:name, name()}\n```","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:option/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Supervisor.strategy/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nstrategy() :: :one_for_one | :one_for_all | :rest_for_one\n```","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:strategy/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Supervisor.supervisor/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nsupervisor() :: pid() | name() | {atom(), node()}\n```","ref":"TypeCheck.DefaultOverrides.Supervisor.html#t:supervisor/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.System","doc":"","ref":"TypeCheck.DefaultOverrides.System.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.System.signal/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nsignal() :: :sigabrt\n| :sigalrm\n| :sigchld\n| :sighup\n| :sigquit\n| :sigstop\n| :sigterm\n| :sigtstp\n| :sigusr1\n| :sigusr2\n```","ref":"TypeCheck.DefaultOverrides.System.html#t:signal/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.System.time_unit/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\ntime_unit() :: :second | :millisecond | :microsecond | :nanosecond | pos_integer()\n```","ref":"TypeCheck.DefaultOverrides.System.html#t:time_unit/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Task","doc":"","ref":"TypeCheck.DefaultOverrides.Task.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Task.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %Task{owner: pid(), pid: pid() | nil, ref: reference()}\n```","ref":"TypeCheck.DefaultOverrides.Task.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Task.Supervisor","doc":"","ref":"TypeCheck.DefaultOverrides.Task.Supervisor.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Task.Supervisor.option/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\noption() :: TypeCheck.DefaultOverrides.DynamicSupervisor.option()\n| TypeCheck.DefaultOverrides.DynamicSupervisor.init_option()\n```","ref":"TypeCheck.DefaultOverrides.Task.Supervisor.html#t:option/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Time","doc":"","ref":"TypeCheck.DefaultOverrides.Time.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Time.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %Time{\n  calendar: TypeCheck.DefaultOverrides.Calendar.calendar(),\n  hour: TypeCheck.DefaultOverrides.Calendar.hour(),\n  microsecond: TypeCheck.DefaultOverrides.Calendar.microsecond(),\n  minute: TypeCheck.DefaultOverrides.Calendar.minute(),\n  second: TypeCheck.DefaultOverrides.Calendar.second()\n}\n```","ref":"TypeCheck.DefaultOverrides.Time.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.URI","doc":"","ref":"TypeCheck.DefaultOverrides.URI.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.URI.port_number/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nport_number() :: %{__struct__: Range, first: 0, last: 65535, step: 1}\n```","ref":"TypeCheck.DefaultOverrides.URI.html#t:port_number/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.URI.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %URI{\n  authority: nil | binary(),\n  fragment: nil | binary(),\n  host: nil | binary(),\n  path: nil | binary(),\n  port: nil | port_number(),\n  query: nil | binary(),\n  scheme: nil | binary(),\n  userinfo: nil | binary()\n}\n```","ref":"TypeCheck.DefaultOverrides.URI.html#t:t/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Version","doc":"","ref":"TypeCheck.DefaultOverrides.Version.html"},{"type":"type","title":"TypeCheck.DefaultOverrides.Version.build/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nbuild() :: TypeCheck.DefaultOverrides.String.t() | nil\n```","ref":"TypeCheck.DefaultOverrides.Version.html#t:build/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Version.major/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nmajor() :: non_neg_integer()\n```","ref":"TypeCheck.DefaultOverrides.Version.html#t:major/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Version.minor/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nminor() :: non_neg_integer()\n```","ref":"TypeCheck.DefaultOverrides.Version.html#t:minor/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Version.patch/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\npatch() :: non_neg_integer()\n```","ref":"TypeCheck.DefaultOverrides.Version.html#t:patch/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Version.pre/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\npre() :: [TypeCheck.DefaultOverrides.String.t() | non_neg_integer()]\n```","ref":"TypeCheck.DefaultOverrides.Version.html#t:pre/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Version.requirement/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nrequirement() :: TypeCheck.DefaultOverrides.String.t() | TypeCheck.DefaultOverrides.Version.Requirement.t()\n```","ref":"TypeCheck.DefaultOverrides.Version.html#t:requirement/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Version.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nt() :: %Version{build: build(), major: major(), minor: minor(), patch: patch(), pre: pre()}\n```","ref":"TypeCheck.DefaultOverrides.Version.html#t:t/0"},{"type":"type","title":"TypeCheck.DefaultOverrides.Version.version/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n```elixir\nversion() :: TypeCheck.DefaultOverrides.String.t() | t()\n```","ref":"TypeCheck.DefaultOverrides.Version.html#t:version/0"},{"type":"module","title":"TypeCheck.DefaultOverrides.Version.Requirement","doc":"","ref":"TypeCheck.DefaultOverrides.Version.Requirement.html"},{"type":"opaque","title":"TypeCheck.DefaultOverrides.Version.Requirement.matchable/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n`matchable()` _(opaque type)_","ref":"TypeCheck.DefaultOverrides.Version.Requirement.html#t:matchable/0"},{"type":"opaque","title":"TypeCheck.DefaultOverrides.Version.Requirement.t/0","doc":"_(This type is managed by `TypeCheck`,\nwhich allows checking values against the type at runtime.)_\n\n\n\nFull definition:\n\n`t()` _(opaque type)_","ref":"TypeCheck.DefaultOverrides.Version.Requirement.html#t:t/0"},{"type":"exception","title":"TypeCheck.CompileError","doc":"Raised when during compilation of types or specifications,\nan irrecoverable error occurs.","ref":"TypeCheck.CompileError.html"},{"type":"module","title":"TypeCheck.Credo.Check.Readability.Specs","doc":"A custom Credo check which supports the `@spec!` syntax.\n\nNOTE: This module is only compiled\nif you've added `:credo` as a (dev/test) dependency to your app.\n\nTo use this check in your project, make sure you have a `.credo.exs` config file\n(which you can generate with `mix credo gen.config`)\nand make sure to add it to the `:checks` `:enabled` list:\n\n    %{\n      configs: [\n        %{\n          checks: %{\n            enabled:\n              [\n                {Elixir.TypeCheck.Credo.Check.Readability.Specs, []},\n                # ...\n              ]\n              # ...\n            }\n          # ...\n        }\n      ]\n    }\n\n\nThis check is an alternative to Credo's own experimental `Credo.Check.Readability.Specs`,\nso be sure to turn that check off.\n\n\n---","ref":"TypeCheck.Credo.Check.Readability.Specs.html"},{"type":"module","title":"Basics - TypeCheck.Credo.Check.Readability.Specs","doc":"> #### This check is disabled by default. {: .neutral}\n>\n> [Learn how to enable it](config_file.html#checks) via `.credo.exs`.\n\n> #### This check is tagged `:controversial` {: .warning}\n>\n> This means that this check is more opinionated than others and not for everyone's taste.\n\nThis check has a base priority of `0` and works with any version of Elixir.","ref":"TypeCheck.Credo.Check.Readability.Specs.html#module-basics"},{"type":"module","title":"Explanation - TypeCheck.Credo.Check.Readability.Specs","doc":"Functions, callbacks and macros need typespecs.\n\nAdding typespecs gives tools like Dialyzer and TypeCheck more information when performing\nchecks for type errors in function calls and definitions.\nTypespecs will also be shown in generated documentation,\nand can be a great way to concisely convey how a function should be used.\n\nUsing TypeCheck's `@spec!` syntax which will also enable runtime type-checking:\n(Don't forget to `use TypeCheck` in your module!)\n\n    @spec! sub(integer, integer) :: integer\n    def sub(a, b), do: a + b\n\nIf you do not want runtime checks, write a normal `@spec`:\n\n    @spec add(integer, integer) :: integer\n    def add(a, b), do: a + b\n\nFunctions with multiple arities need to have a spec defined for each arity:\n\n    @spec! foo(integer) :: boolean\n    @spec! foo(integer, integer) :: boolean\n    def foo(a), do: a > 0\n    def foo(a, b), do: a > b\n\nThe check only considers whether the specification is present, it doesn't\nperform any actual type checking while reading your code.\n\nLike all `Readability` issues, this one is not a technical concern.\nBut you can improve the odds of others reading and liking your code by making\nit easier to follow.","ref":"TypeCheck.Credo.Check.Readability.Specs.html#module-explanation"},{"type":"module","title":"Check-Specific Parameters - TypeCheck.Credo.Check.Readability.Specs","doc":"Use the following parameters to configure this check:\n\n### `:include_defp`\n\n  Include private functions.\n\n*This parameter defaults to* `false`.","ref":"TypeCheck.Credo.Check.Readability.Specs.html#module-check-specific-parameters"},{"type":"module","title":"General Parameters - TypeCheck.Credo.Check.Readability.Specs","doc":"Like with all checks, [general params](check_params.html) can be applied.\n\nParameters can be configured via the [`.credo.exs` config file](config_file.html).","ref":"TypeCheck.Credo.Check.Readability.Specs.html#module-general-parameters"},{"type":"extras","title":"Guide/Readme","doc":"![](https://raw.githubusercontent.com/Qqwy/elixir-type_check/master/media/type_check_logo_flat.svg)\n\n# TypeCheck: Fast and flexible runtime type-checking for your Elixir projects.\n\n[![hex.pm version](https://img.shields.io/hexpm/v/type_check.svg)](https://hex.pm/packages/type_check)\n[![Documentation](https://img.shields.io/badge/hexdocs-latest-blue.svg)](https://hexdocs.pm/type_check/index.html)\n[![ci](https://github.com/Qqwy/elixir-type_check/actions/workflows/ci.yml/badge.svg)](https://github.com/Qqwy/elixir-type_check/actions/workflows/ci.yml)\n[![Coverage Status](https://coveralls.io/repos/github/Qqwy/elixir-type_check/badge.svg)](https://coveralls.io/github/Qqwy/elixir-type_check)","ref":"readme.html"},{"type":"extras","title":"Core ideas - Guide/Readme","doc":"- Type- and function specifications are constructed using (essentially) the **same syntax** as built-in Elixir Typespecs.\n- When a value does not match a type check, the user is shown **human-friendly error messages**.\n- Types and type-checks are generated at compiletime.\n  - This means **type-checking code is optimized** rigorously by the compiler.\n- **Property-checking generators** can be extracted from type specifications without extra work.\n  - Automatically create a **spectest** which checks for each function if it adheres to its spec.\n- Flexibility to add **custom checks**: Subparts of a type can be named, and 'type guards' can be specified to restrict what values are allowed to match that refer to these types.\n\n\nPrefer to watch a presentation instead of reading? See [\"TypeCheck: Effortless Runtime Type Checking\" - Marten Wijnja - _ElixirConf EU 2022_](https://www.youtube.com/watch?v=7ykfO2tBwYw).","ref":"readme.html#core-ideas"},{"type":"extras","title":"Usage Example - Guide/Readme","doc":"We add `use TypeCheck` to a module \nand wherever we want to add runtime type-checks \nwe replace the normal calls to `@type` and `@spec` with `@type!` and `@spec!` respectively.\n\n```elixir\ndefmodule User do\n  use TypeCheck\n  defstruct [:name, :age]\n\n  @type! t :: %User{name: binary, age: integer}\nend\n\ndefmodule AgeCheck do\n  use TypeCheck\n\n  @spec! user_older_than?(User.t, integer) :: boolean\n  def user_older_than?(user, age) do\n    user.age >= age\n  end\nend\n```\n\nNow we can try the following:\n\n```elixir\niex> AgeCheck.user_older_than?(%User{name: \"Qqwy\", age: 11}, 10)\ntrue\niex> AgeCheck.user_older_than?(%User{name: \"Qqwy\", age: 9}, 10)\nfalse\n```\n\nSo far so good. Now let's see what happens when we pass values that are incorrect:\n\n```elixir\niex> AgeCheck.user_older_than?(\"foobar\", 42)\n** (TypeCheck.TypeError) At lib/type_check_example.ex:28:\nThe call to `user_older_than?/2` failed,\nbecause parameter no. 1 does not adhere to the spec `%User{age: integer(), name: binary()}`.\nRather, its value is: `\"foobar\"`.\nDetails:\n  The call `user_older_than?(\"foobar\", 42)` \n  does not adhere to spec `user_older_than?(%User{age: integer(), name: binary()},  integer()) :: boolean()`. Reason:\n    parameter no. 1:\n      `\"foobar\"` does not check against `%User{age: integer(), name: binary()}`. Reason:\n        `\"foobar\"` is not a map.\n    (type_check_example 0.1.0) lib/type_check_example.ex:28: AgeCheck.user_older_than?/2\n```\n\n```elixir\niex> AgeCheck.user_older_than?(%User{name: nil, age: 11}, 10)\n** (TypeCheck.TypeError) At lib/type_check_example.ex:28:\nThe call to `user_older_than?/2` failed,\nbecause parameter no. 1 does not adhere to the spec `%User{age: integer(), name: binary()}`.\nRather, its value is: `%User{age: 11, name: nil}`.\nDetails:\n  The call `user_older_than?(%User{age: 11, name: nil}, 10)` \n  does not adhere to spec `user_older_than?(%User{age: integer(), name: binary()},  integer()) :: boolean()`. Reason:\n    parameter no. 1:\n      `%User{age: 11, name: nil}` does not check against `%User{age: integer(), name: binary()}`. Reason:\n        under key `:name`:\n          `nil` is not a binary.\n    (type_check_example 0.1.0) lib/type_check_example.ex:28: AgeCheck.user_older_than?/2\n```\n\n```elixir\niex> AgeCheck.user_older_than?(%User{name: \"Aaron\", age: nil}, 10) \n** (TypeCheck.TypeError) At lib/type_check_example.ex:28:\nThe call to `user_older_than?/2` failed,\nbecause parameter no. 1 does not adhere to the spec `%User{age: integer(), name: binary()}`.\nRather, its value is: `%User{age: nil, name: \"Aaron\"}`.\nDetails:\n  The call `user_older_than?(%User{age: nil, name: \"Aaron\"}, 10)` \n  does not adhere to spec `user_older_than?(%User{age: integer(), name: binary()},  integer()) :: boolean()`. Reason:\n    parameter no. 1:\n      `%User{age: nil, name: \"Aaron\"}` does not check against `%User{age: integer(), name: binary()}`. Reason:\n        under key `:age`:\n          `nil` is not an integer.\n    (type_check_example 0.1.0) lib/type_check_example.ex:28: AgeCheck.user_older_than?/2\n```\n\n```elixir\n    \niex> AgeCheck.user_older_than?(%User{name: \"José\", age: 11}, 10.0) \n** (TypeCheck.TypeError) At lib/type_check_example.ex:28:\nThe call to `user_older_than?/2` failed,\nbecause parameter no. 2 does not adhere to the spec `integer()`.\nRather, its value is: `10.0`.\nDetails:\n  The call `user_older_than?(%User{age: 11, name: \"José\"}, 10.0)` \n  does not adhere to spec `user_older_than?(%User{age: integer(), name: binary()},  integer()) :: boolean()`. Reason:\n    parameter no. 2:\n      `10.0` is not an integer.\n    (type_check_example 0.1.0) lib/type_check_example.ex:28: AgeCheck.user_older_than?/2\n```\n\nAnd if we were to introduce an error in the function definition:\n\n```elixir\ndefmodule AgeCheck do\n  use TypeCheck\n\n  @spec! user_older_than?(User.t, integer) :: boolean\n  def user_older_than?(user, age) do\n    user.age\n  end\nend\n```\n\nThen we get a nice error message explaining that problem as well:\n\n```elixir\n** (TypeCheck.TypeError) The call to `user_older_than?/2` failed,\nbecause the returned result does not adhere to the spec `boolean()`.\nRather, its value is: `26`.\nDetails:\n  The result of calling `user_older_than?(%User{age: 26, name: \"Marten\"}, 10)` \n  does not adhere to spec `user_older_than?(%User{age: integer(), name: binary()},  integer()) :: boolean()`. Reason:\n    Returned result:\n      `26` is not a boolean.\n    (type_check_example 0.1.0) lib/type_check_example.ex:28: AgeCheck.user_older_than?/2\n```","ref":"readme.html#usage-example"},{"type":"extras","title":"Features & Roadmap - Guide/Readme","doc":"","ref":"readme.html#features-roadmap"},{"type":"extras","title":"Implemented - Guide/Readme","doc":"- [x] Proof and implementation of the basic concept\n- [x] Custom type definitions (type, typep, opaque)\n  - [x] Basic\n  - [x] Parameterized\n- [x] Hide implementation of `opaque` from documentation\n- [x] Spec argument types checking\n- [x] Spec return type checking\n- [x] Spec possibly named arguments\n- [x] Implementation of Elixir's builtin types\n  - [x] Primitive types\n  - [x] More primitive types\n  - [x] Compound types\n  - [x] special forms like `|`, `a..b` etc.\n  - [x] Literal lists\n  - [x] Maps with keys => types\n  - [x] Structs with keys => types\n  - [x] More map/list-based structures.\n  - [x] Bitstring type syntax (`<<>>`, `<<_ :: size>>`, `<<_ :: _ * unit>>`, `<<_ :: size, _ :: _ * unit>>`)\n- [x] A `when` to add guards to typedefs for more power.\n- [x] Make errors raised when types do not match humanly readable\n  - [x] Improve readability of spec-errors by repeating spec and which parameter did not match.\n- [x] Creating generators from types\n- [x] Don't warn on zero-arity types used without parentheses.\n- [x] Hide structure of `opaque` and `typep` from documentation\n- [x] Make sure to handle recursive (and mutually recursive) types without hanging.\n  - [x] A compile-error is raised when a type is expanded more than a million times\n  - [x] A macro called `lazy` is introduced to allow to defer type expansion to runtime (to _within_ the check).\n- [x] the Elixir formatter likes the way types+specs are constructed\n- [x] A type `impl(ProtocolName)` to work with 'any type implementing protocol `Protocolname`'.\n  - [x] Type checks.\n  - [x] StreamData generator.\n- [x] High code-coverage to ensure stability of implementation.\n- [x] Make sure we handle most (if not all) of Typespec's primitive types and syntax. (With the exception of functions and binary pattern matching)\n- [x] Option to turn `@type/@opaque/@typep`-injection off for the cases in which it generates improper results.\n- [x] Manually overriding generators for user-specified types if so desired.\n- [x] Creating generators from specs\n  - [x] Wrap spec-generators so you have a single statement to call in the test suite which will prop-test your function against all allowed inputs/outputs.\n- [x] Option to turn the generation of runtime checks off for a given module in a particular environment (`enable_runtime_checks`).\n- [x] Support for function-types (for typechecks as well as property-testing generators):\n  - `(-> result_type)`\n  - `(...-> result_type)`\n  - `(param_type, param2_type -> result_type)`\n- [x] Basic support for maps with a single `required(type)` or `optional(type)`.\n- [x] Overrides for builtin remote types (`String.t`,`Enum.t`, `Range.t`, `MapSet.t` etc.) **(75% done)** [Details](https://hexdocs.pm/type_check/comparing-typecheck-and-elixir-typespecs.html#elixir-standard-library-types)\n- [x] Overrides for more builtin remote types\n- [x] Support for maps with mixed `required(type)` and `optional(type)` syntaxes.\n- [x] Configurable setting to turn checks on/off at compile-time, on a per-OTP-app basis (so you have control over your dependencies) as well as your individual modules.\n- [x] Hide named types from opaque types.\n- [x] A way to define structs and their field types at the same time.\n- [x] Finalize formatter specification and make a generator for this so that people can easily test their own formatters.","ref":"readme.html#implemented"},{"type":"extras","title":"Pre-stable - Guide/Readme","doc":"","ref":"readme.html#pre-stable"},{"type":"extras","title":"Longer-term future ideas - Guide/Readme","doc":"- [ ] Per-module or even per-spec settings to turn on/off, configure formatter, etc.","ref":"readme.html#longer-term-future-ideas"},{"type":"extras","title":"Installation - Guide/Readme","doc":"TypeCheck [is available in Hex](https://hex.pm/docs/publish). The package can be installed\nby adding `type_check` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:type_check, \"~> 0.13.3\"},\n    # To allow spectesting and property-testing data generators (optional):\n    {:stream_data, \"~> 0.5.0\", only: :test}, \n  ]\nend\n```\n\nThe documentation can be found at [https://hexdocs.pm/type_check](https://hexdocs.pm/type_check).","ref":"readme.html#installation"},{"type":"extras","title":"Formatter - Guide/Readme","doc":"TypeCheck exports a couple of macros that you might want to use without parentheses. To make `mix format` respect this setting, add `import_deps: [:type_check]` to your `.formatter.exs` file.","ref":"readme.html#formatter"},{"type":"extras","title":"Changelog - Guide/Readme","doc":"The full changelog can be found [here](https://github.com/Qqwy/elixir-type_check/blob/main/CHANGELOG.md)","ref":"readme.html#changelog"},{"type":"extras","title":"TypeCheck compared to other tools - Guide/Readme","doc":"TypeCheck is by no means the other solution out there to reduce the number of bugs in your code.","ref":"readme.html#typecheck-compared-to-other-tools"},{"type":"extras","title":"Elixir's builtin typespecs and Dialyzer - Guide/Readme","doc":"[Elixir's builtin type-specifications](https://hexdocs.pm/elixir/typespecs.html) use the same syntax as TypeCheck.\nThey are however not used by the compiler or the runtime, and therefore mainly exist to improve your documentation.\n\nBesides documentation, extra external tools like [Dialyzer](http://erlang.org/doc/apps/dialyzer/dialyzer_chapter.html) can be used to perform static analysis of the types used in your application.\n\nDialyzer is an opt-in static analysis tool. This means that it can point out some inconsistencies or bugs, but because of its opt-in nature, there are also many problems it cannot detect, and it requires your dependencies to have written all of their typespecs correctly.\n\nDialyzer is also (unfortunately) infamous for its at times difficult-to-understand error messages.\n\nAn advantage that Dialyzer has over TypeCheck is that its checking is done without having to execute your program code (thus not having any effect on the runtime behaviour or efficiency of your projects).\n\nBecause TypeCheck adds `@type`, `@typep`, `@opaque` and `@spec`-attributes based on the types that are defined, it is possible to use Dialyzer together with TypeCheck.","ref":"readme.html#elixir-s-builtin-typespecs-and-dialyzer"},{"type":"extras","title":"Norm - Guide/Readme","doc":"[Norm](https://github.com/keathley/norm/) is an Elixir library for specifying the structure of data that can be used for both validation and data-generation.\n\nOn a superficial level, Norm and TypeCheck seem similar. However, there are [important differences in their design considerations](https://github.com/Qqwy/elixir-type_check/blob/master/Comparing%20TypeCheck%20and%20Norm.md).","ref":"readme.html#norm"},{"type":"extras","title":"Is it any good? - Guide/Readme","doc":"[yes](https://news.ycombinator.com/item?id=3067434)","ref":"readme.html#is-it-any-good"},{"type":"extras","title":"Changelog","doc":"","ref":"changelog.html"},{"type":"extras","title":"Changelog - Changelog","doc":"- 0.13.7 -\n  - Fixes: (c.f. #195)\n    - Fixing new compiler warnings (stricter checks, deprecation warnings) of Elixir v1.16/v1.17\n    - Ensuring map order shown in most output is sorted, esp. when creating `fixed_map/1`s (including structs). This ensures output of inspect(my_type) and type error formatting should be identical between OTP <= 25 and OTP 26+\n- 0.13.6 -\n  - Support for Elixir v1.17.x:\n    - Fix an oversight in the pre-expander now that there was a slight change to the Elixir AST format. Thank you, @skwerlman! (c.f. #190)\n    - Fixing a bunch of compile-time warnings. Amongst them, a reordering of split function heads (Thank you, @ulfurinn! c.f. #191)\n  - Fixing some documentation typos, Thank you, @kianmeng! (c.f. #179)\n- 0.13.5 -\n  - Support for Elixir v1.15.x. Thank you so much for your guidance and help, @josevalim!  (c.f. #185, #186)\n    - This required turning off part of TypeCheck's self-usage ('dogfooding'). This change should be temporary and will be re-introduced in a future version (c.f. #187).\n- 0.13.4 -\n  - No code changes, but two small improvements to the documentation:\n    - Changelog has been split off from the README file.\n    - A link to Marten's ElixirConf EU 2022 talk about TypeCheck has been added.\n- 0.13.3 -\n  - Fixes:\n    - Ensures that TypeCheck can be used in Erlang-style modules (that do not start with 'Elixir.') (c.f. #174).\n- 0.13.2 -\n  - Fixes:\n    - Fixes issue when unions are used as type arguments in remote types. Than you very much, @dvic! (c.f. #164).\n\n- 0.13.1 -\n  - Fixes:\n    - Fixes issues with aliases to remote types not being followed. (c.f. #160)\n- 0.13.0 - \n  - Additions:\n    - Support for maps with multiple optional atom keys. Thank you very much, @dvic! (c.f. #165)\n  - Fixes:\n    - Fixes 'Enumerable not implemented' crash when passing a struct to a type expecting a fixed map. (c.f. #161)\n- 0.12.4 - \n  - Fixes:\n    - Allow syntax `%{String.t() => any()}` as shorthand for `%{required(String.t()) => any()}`. (c.f. #152)\n    - Situations in which the fully qualified name of a type defined in the current module were not picked up (`Example.t()` not being found inside specs in `Example`. even though `Example` defined a type called `t()`). (c.f. #154)\n    - A problem with using types with type guards in other modules. (c.f. #147)\n- 0.12.3 -\n  - Fixes:\n    - Compatibility problems with Elixir v1.14. (`Code.Identifier.inspect_as_key`, working with the AST of imported functions)  Thank you very much, @marcandre! (c.f. #150)\n- 0.12.2 -\n  - Fixes:\n    - Compilation error in a new Phoenix 1.16 project. Thank you very much, @assimelha! (c.f. #114)\n- 0.12.1 - \n  - Performance:\n    - Significant reduction on generated code size of runtime type checks. This will speed up compilation roughly ~10-50%.\n  - Fixes:\n    - No longer use white ANSI color in exception formatting, ensuring messages of the DefaultFormatter are also visible on white backgrounds (Livebook, anyone? :D)\n    - Skip OTP app lookup (used to fetch per OTP app compile-time configuration) if we're not compiled inside an OTP app (such as inside a Livebook).\n    - Improve generator for `Regex.t()` to generate only correct regular expressions. Thank you very much, @skwerlman!(c.f. #133)\n- 0.12.0 - \n  - Additions:\n    - The default options used are now fetched from the application configuration. This means that you can configure a default for your app as well as for each of your dependencies(!) by adding `config :app_name, :type_check [...]` to your configuration file(s). (c.f. #61)\n    - `TypeCheck.External` module, with functions to work with typespecs in modules outside of your control. Thank you very much, @orsinium! (c.f. #113)\n      - `fetch_spec` to build a TypeCheck type from any function that has a `@spec`.\n      - `fetch_type` to build a TypeCheck type from any `@type`.\n      - `enforce_spec!` to wrap a call to any function that has a `@spec` with a runtime type-check on the input parameters and return value.\n      - `apply` and `apply!` to wrap a call to any function with the function spec type that you give it.\n    - `TypeCheck.Defstruct.defstruct!`, a way to combine `defstruct`, `@enforce_keys` and the creation of the struct's type, reducing boilerplate and the possibility of mistakes. (c.f. #118)\n  - Fixes:\n    - Long-standing issue where Dialyzer would sometimes complain in apps using TypeCheck is resolved. (c.f. #95)\n    - Creation of the new `maybe_nonempty_list` type will no longer get stuck in an infinite loop on creation. (c.f. #120)\n- 0.11.0 - \n  - Additions:\n    - Support for fancier map syntaxes:\n      - `%{required(key_type) => value_type}` Maps with a single kind of required key-value type.\n      - `%{optional(key_type) => value_type}` Maps with a single kind of optional key-value type.\n      - `%{:some => a(), :fixed => b(), :keys => c(), optional(atom()) => any()}` Maps with any number of fixed keys and a single optional key-value type.\n      - TypeCheck now supports nearly all kinds of map types that see use. Archaic combinations of optional and required are not supported, but also not very useful types in practice.\n      - Because of this, the inspection of the builtin type `map(key, value)` has been changed to look the same as an optional map. _This is a minor backwards-incompatible change._\n    - Desugaring `%{}` has changed from 'any map' to 'the empty map' in line with Elixir's Typespecs. _This is a minor backwards-incompatible change._\n    - Support for the builtin types `port()`, `reference()` and (based on these) `identifier()`.\n    - Support for the builtin type `struct()`.\n    - Support for the builtin type `timeout()`.\n    - Support for the builtin type `nonempty_charlist()` and `maybe_improper_list` and (based on these) `iolist()` and `iodata()`.\n    - Adding types depending on these builtins to the default type overrides. **We now support all modules of the full standard library!**\n    - `TypeCheck.Credo.Check.Readability.Specs`: an opt-in alternative Credo check which will check whether all functions have either a `@spec!` or 'normal' `@spec`. (Fixes #102).\n- 0.10.8 - \n  - Fixes:\n    - Ensures that the `Inspect` protocol is properly implemented for sized bitstring types (c.f. #104). Thank you very much, @trarbr!\n- 0.10.7 - \n  - Fixes:\n    - Ensures fixed-maps are checked for superfluous keys (c.f. #96). Thank you very much, @spatrik!\n- 0.10.6 - \n  - Fixes:\n    - Fixes issue where Dialyzer would sometimes complain about the generated return-type checking code if the implementation of the function itself was trivial. (c.f. #85).\n- 0.10.5 - \n  - Additions:\n    - Support for the builtin types of the `System` module. Thank you, @paulswartz!\n- 0.10.4 -\n  - Fixes:\n    - Fixes issue where sometimes results of specs returning a struct would remove some or all struct fields. (c.f. #78)\n    - Related: Ensures that when a `@type!` containing a `%__MODULE__{}` is used before that module's `defstruct`, a clear CompileError is created (c.f. #83).\n- 0.10.3 - \n  - Fixes:\n    - Fixes issue when TypeCheck specs were used inside a defprotocol module or other modules where the normal `def`/`defp` macros are hidden/overridden.\n- 0.10.2 - \n  - Fixes:\n    - Fixes issue where FixedMaps would accept maps any maps (even those missing the required keys) sometimes. (c.f. #74)\n- 0.10.1 - \n  - Fixes:\n    - Swaps `Murmur` out for Erlang's builtin `:erlang.phash2/2` to generate data for function-types, allowing the removal of the optional dependency on the `:murmur` library.\n- 0.10.0 -\n  - Additions\n    - Support for function-types (for typechecks as well as property-testing generators):\n      - `(-> result_type)`\n      - `(...-> result_type)`\n      - `(param_type, param2_type -> result_type)`\n  - Fixes:\n    - Wrapping private functions no longer make the function public. (c.f. #64)\n    - Wrapping macros now works correctly. (also related to #64)\n    - Using `__MODULE__` inside a struct inside a type now expands correctly. (c.f. #66)\n- 0.9.0 - \n  - Support for bitstring type syntax: `<<>>`, `<<_ :: size>>`, `<<_ :: _ * unit>>`, `<<_ :: size, _ :: _ * unit>>` (both as types and as generators)\n- 0.8.2 - \n  - Fixed compiler warnings when optional dependency StreamData is not installed.\n  - Fixed pretty-printing of typedoc of opaque types.\n- 0.8.1 - \n  - Improved documentation with a page comparing TypeCheck with Elixir's plain typespecs. (Thank you very much, @baldwindavid )\n  - Addition of missing override for the type `t Range.t/0`. (Thank you very much, @baldwindavid )\n- 0.8.0 -\n  - Fixes prettyprinting of `TypeCheck.Builtin.Range`.\n  - Addition of `require TypeCheck.Type` to `use TypeCheck` so there no longer is a need to call this manually if you want to e.g. use `TypeCheck.Type.build/1`.\n  - Pretty-printing of types and TypeError output in multiple colors.\n  - Nicer indentation of errors.\n  - named types are now printed in abbreviated fashion if they are repeated multiple times in an error message. This makes a nested error message much easier to read, especially for larger specs.\n  - `[type]` no longer creates a `fixed_list(type)` but instead a `list(type)` (just as Elixir's own typespecs.)\n  - Support for `[...]` and `[type, ...]`as alias for `nonempty_list()` and `nonempty_list(type)` respectively.\n  - Remove support for list literals with multiple elements.\n  - Improved documentation. Thank you, @0ourobor0s!\n- 0.7.0 - Addition of the option `enable_runtime_checks`. When false, all runtime checks in the given module are completely disabled.\n  - Adding `DateTime.t` to the default overrides, as it was still missing.\n- 0.6.0 - Addition of `spectest` & 'default overrides' Elixir's standard library types:\n  - Adding `TypeCheck.ExUnit`, with the function `spectest` to test function-specifications.\n    - Possibility to use options `:except`, `:only`, `:initial_seed`.\n    - Possibility to pass custom options to StreamData.\n  - Adding `TypeCheck.DefaultOverrides` with many sub-modules containing checked typespecs for the types in Elixir's standard library (75% done).\n    - Ensure that these types are correct also on older Elixir versions (1.9, 1.10, 1.11)\n  - By default load these 'DefaultOverrides', but have the option to turn this behaviour off in `TypeCheck.Option`. \n  - Nice generators for `Enum.t`, `Collectable.t`, `String.t`.\n  - Support for the builtin types:\n    - `pid()`\n    - `nonempty_list()`, `nonempty_list(type)`.\n  - Allow `use TypeCheck` in IEx or other non-module contexts, to require `TypeCheck` and import `TypeCheck.Builtin` in the current scope (without importing/using the macros that only work at the module level.)\n  - The introspection function `__type_check__/1` is now added to any module that contains a `use TypeCheck`.\n  - Fixes the `Inspect` implementation of custom structs, by falling back to `Any`, which is more useful than attempting to use a customized implementation that would try to read the values in the struct and failing because the struct-type containing types in the fields.\n  - Fixes conditional compilation warnings when optional dependency `:stream_data` was not included in your project.\n- 0.5.0 - Stability improvements:\n  - Adding `Typecheck.Option` `debug: true`, which will (at compile-time) print the checks that TypeCheck is generating.\n  - Actually autogenerate a `@spec`, which did not yet happen before.\n  - When writing `@autogen_typespec false`, no typespec is exported for the next `@type!`/`@opaque`/`@spec!` encountered in a module.\n  - Code coverage increased to 85%\n  - Bugfixes w.r.t. generating typespecs\n  - Fixes compiler-warnings on unused named types when using a type guard. (c.f. #25)\n  - Fixes any warnings that were triggered during the test suite before.\n- 0.4.0 - Support for `impl(ProtocolName)` to accept any type implementing a particular protocol.\n  - Also adds rudimentary support for overriding remote types.\n  - Bugfix when inspecting `lazy( ...)`-types.\n- 0.3.2 - Support for unquote fragments inside types and specs. (c.f. #39)\n- 0.3.1 - Fixed link in the documentation.\n- 0.3.0 - Improve DefaultFormatter output when used with long function- or type-signatures (c.f. #32). Also, bugfix for `Builtin.tuple/1`.\n- 0.2.3 - Bugfix release: Ensure TypeCheck compiles on Elixir v1.11 (#30), Ensure StreamData truly is an optional dependency (#27).\n- 0.2.2 - Support for literal strings should no longer break in Elixir's builtin typespecs.\n- 0.2.1 - Improved parsing of types that have a type-guard at the root level. (c.f. #24), support for custom generators.\n- 0.2.0 - Improved (and changed) API that works better with the Elixir formatter: Use `@type!`/`@spec!` instead, support named types in specs.\n- 0.1.2 - Added missing `keyword` type to TypeCheck.Builtin (#20)\n- 0.1.1 - Fixing some documentation typos\n- 0.1.0 - Initial Release","ref":"changelog.html#changelog"},{"type":"extras","title":"Introducing TypeCheck","doc":"![](https://raw.githubusercontent.com/Qqwy/elixir-type_check/master/media/type_check_logo_flat.svg)\n\n# Type-checking and spec-testing with TypeCheck\n\n[TypeCheck](https://hex.pm/packages/type_check) is an elixir library to, you guessed it, check the types of the values, variables and functions in your Elixir projects.\n\nElixir is a strong, dynamically typed programming language.\n'Strong' (as opposed to 'weak') means that when we try to perform an unsupported operation on a value (like 'multiplying strings'), we get an error, rather than silent faulty behaviour.\n'Dynamic' (as opposed to 'Static') means that which operations we do with our values is not checked at compile-time, but only once the program itself is running.\n\nHowever, when such a failure happens at runtime, the resulting error (and more importantly: what the _cause_ was of an error) is not always very clear.\nIs it a bug in your code? Or a bug in a library you are using?\nWhere in the deeply nested tree of structs is the problem that results in a `** (BadFunctionError) expected a function, got: nil` somewhere deep inside the codebase?\n\nRelated to this, we would like to catch problems early in the development cycle: if we find an error while writing our code initially,\nthen fixing it takes significantly less effort (and is much less costly) than if the problem happens in an application which is already deployed to production.\n\n\nTypeCheck gives you the handholds to tackle this situation.","ref":"type-checking-and-spec-testing-with-typecheck.html"},{"type":"extras","title":"What about Elixir's built-in typespecs and Dialyzer? - Introducing TypeCheck","doc":"Elixir (and Erlang) come with a nice description for the types of the values passed between functions, called ['Typespecs'](https://hexdocs.pm/elixir/master/typespecs.html).\nHowever, by default these typespecs are just used for documentation.\nThey are not used in any way to restrict, or even warn when your code is not following them correctly.\n\nSomewhat more recently, tools like [Dialyzer / Dialyxir](https://github.com/jeremyjh/dialyxir) have been introduced in the ecosystem.\nThese perform a static check of the source-code (that is, they look at the code as written without executing it) to catch some potential mis-uses.\nDialyzer has however three commonly mentioned drawbacks:\n\n1. On larger codebases it becomes prohibitively slow to run;\n2. It contains both 'false negatives' (improper usage that it missed) as well as 'false positives' (warnings about things which are actually OK);\n3. Some warnings can be outright cryptic to resolve.\n\nDialyzer is still a very useful tool, but it is not a cure-all.\nAs such, there was definite room in this space for improvement.","ref":"type-checking-and-spec-testing-with-typecheck.html#what-about-elixir-s-built-in-typespecs-and-dialyzer"},{"type":"extras","title":"Introducing TypeCheck - Introducing TypeCheck","doc":"Using TypeCheck is as simple as adding a `use TypeCheck` to the module(s) you want to add checks to.\n\nFor each type-specification ('type') and function-specification ('spec') which is defined using TypeCheck, the following four features become available:\n\n1. 'Normal' Elixir Typespecs for usage with pre-existing external tools (like e.g. Dialyzer).\n2. Documentation with extra detail not found in 'Normal' Elixir typespecs.\n3. Run-time type checks for all parameters to a function (if there is a failure, the function is used improperly) and of the returned value (if this fails, the function has a mistake).\n4. Data generators for all types (and specs), for usage in testing, especially property-tests and spectests (explained below).\n\nLet's take a look at how TypeCheck can be used in practice.\nThis will help to see how the run-time type checks work, as well as how you can use TypeCheck's spectests to supercharge your testing.","ref":"type-checking-and-spec-testing-with-typecheck.html#introducing-typecheck"},{"type":"extras","title":"An Example - Introducing TypeCheck","doc":"Let's say we are writing a module to work with five-star ratings.\n\n```elixir\ndefmodule Rating do\n  @type t() :: %Rating{value: 1..5, author: String.t()}\n  defstruct [:value, :author]\n\n  @spec average(list(t())) :: number()\n  def average(ratings) do\n    values = Enum.map(ratings, &(&1.value))\n    Enum.sum(values) / Enum.count(values)\n  end\nend\n```\n\nIn good Elixir style, the functions and the struct have already received type signatures, even though they are currently only used for documentation.\n(Speaking of documentation: it is good practice to add documentation to all public modules and functions. To keep the examples in this article brief, they have been elided.)\n\nFrom the function-specification, you can already see how the function `average` is intended to be used:\nWhen passing a list of ratings, we will return a single number: the average.\n\n\nLet's try calling it with a couple of inputs:\n\n\n```elixir\niex(8)> Rating.average([%Rating{author: \"Joe\", value: 5}, %Rating{author: \"Mike\", value: 4}])\n4.5\niex(8)> Rating.average([%Rating{author: \"Robert\", value: 3}])\n3.0\n```\n\nSo far so good.\nNow, let's try what happens when someone makes a mistake:\n\n```elixir\niex> Rating.average(%Rating{author: \"José\", value: 3})\n** (Protocol.UndefinedError) protocol Enumerable not implemented for %Rating{author: \"José\", value: 3} of type Rating (a struct). This protocol is implemented for the following type(s): Map, Range, List, MapSet, GenEvent.Stream, Stream, Date.Range, HashDict, IO.Stream, HashSet, Function, File.Stream\n    (elixir 1.12.0) lib/enum.ex:1: Enumerable.impl_for!/1\n    (elixir 1.12.0) lib/enum.ex:141: Enumerable.reduce/3\n    (elixir 1.12.0) lib/enum.ex:3923: Enum.map/2\n    iex:19: Rating.average/1\n```\n\nOof!\nClearly something is going wrong here, but if one were to encounter this error somewhere deep in a codebase,\nit would not be immediately obvious that the problem was that we were calling the function incorrectly.\n\nEven worse is if we happen to pass a list of non-ratings to the function:\n\n```elixir\niex> Rating.average([1, 2, 3])\n** (ArgumentError) you attempted to apply :value on 1. If you are using apply/3, make sure the module is an atom. If you are using the dot syntax, such as map.field or module.function(), make sure the left side of the dot is an atom or a map\n    :erlang.apply(1, :value, [])\n    iex:19: anonymous fn/1 in Rating.average/1\n    (elixir 1.12.0) lib/enum.ex:1553: Enum.\"-map/2-lists^map/1-0-\"/2\n    iex:19: Rating.average/1\n```\n\nAnd finally, there is nothing preventing the creation of malformed rating-objects.\nWhile we have specified in our type that the rating's value should only ever be in the range 1..5,\nthis is not constrained anywhere in the code.\n\nAnd if someone passes a `nil` rating, we'd get a `** (ArithmeticError) bad argument in arithmetic expression: nil + 0` error.\nAlso not very clear.\n\nWhile we could sprinkle checks for this everywhere, this would in the best case result in extremely 'defensive' and badly readable code.\nAnd in the worst case, we might forget to add a check at certain places, still not giving us certainty.\n\n\nLet's see how TypeCheck can improve this situation.","ref":"type-checking-and-spec-testing-with-typecheck.html#an-example"},{"type":"extras","title":"Adding TypeCheck - Introducing TypeCheck","doc":"In general, adding TypeCheck to a module only requires the following two steps:\n1. Add `use TypeCheck` at the top of the module.\n2. Replace all usage of `@type` with `@type!` and all usage of `@spec` with `@spec!`. (For the curious: There indeed are similarly overloaded versions of `@typep!` and `@opaque!` available.)\n\nYou're done!\nOur example ratings module now looks like this:\n\n```elixir\ndefmodule Rating do\n  use TypeCheck\n\n  @type! t() :: %Rating{value: 1..5, author: String.t()}\n  defstruct [:value, :author]\n\n  @spec! average(list(t())) :: number()\n  def average(ratings) do\n    values = Enum.map(ratings, &(&1.value))\n    Enum.sum(values) / Enum.count(values)\n  end\nend\n```\n\nWith this change, correct usage of the function still returns the expected results:\n\n```elixir\niex(8)> Rating.average([%Rating{author: \"Joe\", value: 5}, %Rating{author: \"Mike\", value: 4}])\n4.5\niex(8)> Rating.average([%Rating{author: \"Robert\", value: 3}])\n3.0\n```\n\nSo far, so good. Now let's look at what happens when the function is used incorrectly:\n\n\n```elixir\niex(20)> Rating.average(%Rating{author: \"José\", value: 3})\n** (TypeCheck.TypeError) At iex:17:\n    The call to `average/1` failed,\n    because parameter no. 1 does not adhere to the spec `list(%Rating{author: binary(), value: number()})`.\n    Rather, its value is: `%Rating{author: \"José\", value: 3}`.\n    Details:\n      The call `average(%Rating{author: \"José\", value: 3})`\n      does not adhere to spec `average(list(%Rating{author: binary(), value: number()})) :: number()`. Reason:\n        parameter no. 1:\n          `%Rating{author: \"José\", value: 3}` does not check against `list(%Rating{author: binary(), value: number()})`. Reason:\n            `%Rating{author: \"José\", value: 3}` is not a list.\n        lib/type_check/spec.ex:165: Rating.average/1\n\n```\n\nLook at that! Not only do we see that the problem is is caused by the call to the function itself,\nbut the cause of the problem is also very clear: the passed value is not a list.\n\n\nHere's our second mistake:\n\n```elixir\niex> Rating.average([1, 2, 3])\n** (TypeCheck.TypeError) At iex:17:\n    The call to `average/1` failed,\n    because parameter no. 1 does not adhere to the spec `list(%Rating{author: binary(), value: number()})`.\n    Rather, its value is: `[1, 2, 3]`.\n    Details:\n      The call `average([1, 2, 3])`\n      does not adhere to spec `average(list(%Rating{author: binary(), value: number()})) :: number()`. Reason:\n        parameter no. 1:\n          `[1, 2, 3]` does not check against `list(%Rating{author: binary(), value: number()})`. Reason:\n            at index 0:\n              `1` does not check against `%Rating{author: binary(), value: number()}`. Reason:\n                `1` is not a map.\n        lib/type_check/spec.ex:165: Rating.average/1\n\n```\n\nThis time too, the error is much clearer, showing that the innermost reason of the type-checking failure is that the passed number is not a map.\n\nNow, let's also look at what happens when we pass a malformed `Rating` struct:\n\n\n```elixir\niex> Rating.average([%Rating{author: \"root\", value: -100}])\n** (TypeCheck.TypeError) At iex:21:\n    The call to `average/1` failed,\n    because parameter no. 1 does not adhere to the spec `list(%Rating{author: binary(), value: 1..5})`.\n    Rather, its value is: `[%Rating{author: \"root\", value: -100}]`.\n    Details:\n      The call `average([%Rating{author: \"root\", value: -100}])`\n      does not adhere to spec `average(list(%Rating{author: binary(), value: 1..5})) :: number()`. Reason:\n        parameter no. 1:\n          `[%Rating{author: \"root\", value: -100}]` does not check against `list(%Rating{author: binary(), value: 1..5})`. Reason:\n            at index 0:\n              `%Rating{author: \"root\", value: -100}` does not check against `%Rating{author: binary(), value: 1..5}`. Reason:\n                under key `:value`:\n                  `-100` does not check against `1..5`. Reason:\n                    `-100` falls outside the range 1..5.\n        lib/type_check/spec.ex:165: Rating.average/1\n\n```\n\nMuch nicer! Now, all functions which have a `@spec!` will have there inputs checked properly,\nand passing them a malformed rating-struct is impossible without cluttering our code.\n\n\nAs a final example, say we alter the implementation of the function to something which is clearly wrong,\nsuch as always returning the string `\"something else\"`:\n\n```elixir\niex> Rating.average([%Rating{author: \"Joe\", value: 5}, %Rating{author: \"Mike\", value: 4}])\n** (TypeCheck.TypeError) The call to `average/1` failed,\n    because the returned result does not adhere to the spec `number()`.\n    Rather, its value is: `\"something else\"`.\n    Details:\n      The result of calling `average([%Rating{author: \"Marten\", value: 1}])`\n      does not adhere to spec `average(\n      list(%Rating{author: binary(), value: 1..5})) :: number()`. Reason:\n        Returned result:\n          `\"something else\"` is not a number.\n        lib/type_check/spec.ex:194: Rating.average/1\n```\n\nIn this case, you can clearly see that the problem is caused by the return value, and why.","ref":"type-checking-and-spec-testing-with-typecheck.html#adding-typecheck"},{"type":"extras","title":"Efficiency of runtime checks - Introducing TypeCheck","doc":"TypeCheck adds its runtime checks by wrapping your functions (using [`defoverridable`](https://hexdocs.pm/elixir/1.9.1/Kernel.html?#defoverridable/1)).\nThis means that the Elixir and Erlang compilers are able to optimize the checks to their liking.\nTherefore, code added by TypeCheck is at the very least not slower than any hand-written parameter-checking code.\nIn many cases, the compiler is even smart enough to combine the type-check with a `case`, `if` or `cond`-expression that your function implementation itself contains,\nso even in those cases no duplicate checks are done.\n\nThat said, there are certain cases in which the type-checks might still be too slow, since by default TypeCheck performs a deep check for\nall parameters to a function. In the case of for instance a large collection or a deeply-nested 'tree of structs' this might still be too slow.\n\nIn those cases, you might want to turn off TypeCheck in certain environments (such as production) while still keeping it available in the development and testing environments.\nBe sure to benchmark before making the decision to turn TypeCheck off, as there is a high probability that the bottlenecks in your code are actually found elsewhere.\n\nAnd finally, regardless of whether the checks are used or not, there is one more way in which TypeCheck's types and specs are useful: Automated testing.","ref":"type-checking-and-spec-testing-with-typecheck.html#efficiency-of-runtime-checks"},{"type":"extras","title":"Spectests - Introducing TypeCheck","doc":"Runtime type-checks give an early notice whether a function is being used properly or improperly.\nIn essence, we check 'Does the caller of the function adhere to the function's specification?'\nHowever, to get more certainty about the correctness¹ of our code, we'd also like to check the opposite: 'Does the function itself adhere to its specification?'\n\nThis is where 'function-specification tests', or _spectests_ for short, come in.","ref":"type-checking-and-spec-testing-with-typecheck.html#spectests"},{"type":"extras","title":"What is a spectest? - Introducing TypeCheck","doc":"A spectest is a property-based test in which\nwe check whether the function adheres to its specification's _invariants_\n(also known as the function's _contract_ or its _preconditions and postconditions_).\n\nThis is done by generating a large amount of possible function inputs,\nand for each of these, check whether the function:\n- Does not raise an exception.\n- Returns a result that type-checks against the spec's return-type.\n\nSpectests are given its own test-category in ExUnit, for easier recognition\n(Just like 'doctests' and 'properties' are different from normal tests, so are 'spectests'.)\n\n\n\n\nIf you're new to property-based testing, then ['Overview of Property-based testing'](https://hexdocs.pm/stream_data/ExUnitProperties.html#module-overview-of-property-based-testing)-section of the ExUnitProperties' documentation\nmight be a good place for a general overview.\n\n \n¹: Because of the nature of property-based testing, we can never know for 100% sure that a function is correct. However, with every new randomly-generated test-case, the level of confidence grows a little. So while we can never by  fully  sure, we are able to get asymptotically close to it.","ref":"type-checking-and-spec-testing-with-typecheck.html#what-is-a-spectest"},{"type":"extras","title":"Spectesting our example - Introducing TypeCheck","doc":"To add a spectest, we need to `use TypeCheck.ExUnit` in our testing file,\nand then call `spectest YourModuleName`. This accepts options similarly to [`doctest`](https://hexdocs.pm/ex_unit/ExUnit.DocTest.html#doctest/2) which you might already be familiar with.\n\nTo use spectests (and data generation in general), TypeCheck requires on the optional dependency [`StreamData`](https://hex.pm/packages/stream_data),\nso be sure to add it to your project's mix deps.\n\nLet's spectest the `Rating` module, to make sure our code does not contain any mistakes.\n\n```elixir\ndefmodule RatingTest do\n  use ExUnit.Case, async: true\n  use TypeCheck.ExUnit\n\n  spectest Rating\nend\n```\n\n```elixir\n$ mix test\n\n\n  1) spectest average(list(%Rating{author: binary(), value: 1..5})) :: number() (RatingTest)\n     test/rating_test.exs:5\n     Spectest failed (after 0 successful runs)\n\n     Input: Rating.average([])\n\n     ** (ArithmeticError) bad argument in arithmetic expression\n\n     code: #TypeCheck.Spec \n     stacktrace:\n       (type_check_guide 0.1.0) lib/rating.ex:10: Rating.\"average (overridable 1)\"/1\n       lib/type_check/ex_unit.ex:5: anonymous fn/1 in RatingTest.\"spectest average(\\n  list(%Rating{author: binary(), value: 1..5})\\n)\\n::\\nnumber()\"/1\n       (stream_data 0.5.0) lib/stream_data.ex:2102: StreamData.check_all/7\n       lib/type_check/ex_unit.ex:5: (test)\n\n..\n\nFinished in 0.1 seconds (0.1s async, 0.00s sync)\n1 doctest, 1 spectest, 1 test, 1 failure\n```\n\n\nOof! Our code contained a mistake!\nIt turns out that when `Rating.average` is given an empty list, we end up dividing by zero!\n\nSome math-savvy readers might have seen this coming from a mile away.\nHowever, I'm sure that _some_ of you will have been surprised by this problem 😇.\nOf course this is only an educational example of the kind of issues one might encounter in a _real_ codebase.\n\nWe might resolve this issue in two ways:\n- Decide that an empty list should never be passed, and therefore restrict the parameter types further.\n  For instance, we could change it from a `list(Rating.t())` to a `nonempty_list(Rating.t())`.\n  This means that when someone tries passing it an empty list, they will immediately be notified that this is not supported,\n  using the clear error messages as seen in the earlier examples.\n  Changing the spec this way will also make the spectest pass, as empty lists will no longer be generated.\n- Decide that empty lists are a correct input, but that the output will be changed from `number()` to `{:ok, number()} | {:error, :empty}`,\n  asking code which uses `average` to handle the possibility of an error-result being returned.\n\nIn either case, after these changes the spectest will pass.","ref":"type-checking-and-spec-testing-with-typecheck.html#spectesting-our-example"},{"type":"extras","title":"More general properties - Introducing TypeCheck","doc":"While spectests are a very simple and code-light way to test your functions,\nit is also possible to generate arbitrary values from any type outside of a `spectest`, for use in other, more specialized property-based tests.\nSee [TypeCheck.Type.StreamData.to_gen/1](https://hexdocs.pm/type_check/TypeCheck.Type.StreamData.html#to_gen/1) for more info.","ref":"type-checking-and-spec-testing-with-typecheck.html#more-general-properties"},{"type":"extras","title":"There is more - Introducing TypeCheck","doc":"We have covered the most important features of TypeCheck,\nbut there is more to discover, such as:\n\n- The error-formatter which [is reconfigurable](https://hexdocs.pm/type_check/TypeCheck.TypeError.Formatter.html) and could be overloaded by custom implementations.\n- TypeCheck itself is extensively property-tested using its own types.\n- Most of TypeCheck's type-construction functions use TypeCheck's checks itself, which mean that you will get the same clear error messages when making a mistake while constructing a type (but this time during _compile-time_. nifty, eh?!)\n- TypeCheck supports ['type guards'](https://hexdocs.pm/type_check/TypeCheck.Builtin.html#guarded_by/2) for if you want to add extra custom constraints to any particular type.\n- TypeCheck supports ['unquote fragments'](https://hexdocs.pm/type_check/TypeCheck.Macros.html#module-typecheck-and-metaprogramming) for when you want to go ham with metaprogramming.","ref":"type-checking-and-spec-testing-with-typecheck.html#there-is-more"},{"type":"extras","title":"Summary - Introducing TypeCheck","doc":"In this guide, you have seen how TypeCheck can be used and what value it can add to your projects.\nWe have seen how TypeCheck can be used in a general project to add runtime checks to your functions,\nas well as how to use the `spectest` macro to get automatic property-tests that check whether your functions\nfollow their specs.\n\nTypeCheck currently is at version 0.7.0 and in active development.\nFeedback, issues and pull requests [are very welcome](https://github.com/Qqwy/elixir-type_check)!\n\nThank you very much for sticking through this long read with me 💚.\nI wish you a wonderful day!\n\n~Marten/Qqwy","ref":"type-checking-and-spec-testing-with-typecheck.html#summary"},{"type":"extras","title":"Comparison to Plain Typespecs","doc":"# Comparing TypeCheck and Elixir Typespecs\n\nTypeCheck is intended to be a drop-in supplement to Elixir typespecs. With very few exceptions, all of Elixir's typespecs are supported by TypeCheck. Below is a breakdown of supported typespec syntax in TypeCheck.\n\nIn the tables below:\n- '✅' indicates that something is supported\n- '❌' indicates that something is not supported.\n- '⌛' indicates that something is not currently supported, but there are plans to add support for it in the near future.","ref":"comparing-typecheck-and-elixir-typespecs.html"},{"type":"extras","title":"Basic Types - Comparison to Plain Typespecs","doc":"| Type                                                         | Supported? | Notes                              |\n|--------------------------------------------------------------|------------|------------------------------------|\n| any()                                                        | ✅         | the top type, the set of all terms |\n| none()                                                       | ✅         | the bottom type, contains no terms |\n| atom()                                                       | ✅         |                                    |\n| map()                                                        | ✅         | any map                            |\n| pid()                                                        | ✅         | process identifier                 |\n| port()                                                       | ✅         | port identifier                    |\n| reference()                                                  | ✅         |                                    |\n| tuple()                                                      | ✅         | tuple of any size                  |\n| float()                                                      | ✅         |                                    |\n| integer()                                                    | ✅         |                                    |\n| neg_integer()                                                | ✅         | ..., -3, -2, -1                    |\n| non_neg_integer()                                            | ✅         | 0, 1, 2, 3, ...                    |\n| pos_integer()                                                | ✅         | 1, 2, 3, ...                       |\n| list(type)                                                   | ✅         | proper list                        |\n| nonempty_list(type)                                          | ✅         | non-empty proper list              |\n| maybe_improper_list(content_type, termination_type)          | ✅         | proper or improper list            |\n| nonempty_improper_list(content_type, termination_type)       | ✅         | improper list                      |\n| nonempty_maybe_improper_list(content_type, termination_type) | ✅         | non-empty proper or improper list  |","ref":"comparing-typecheck-and-elixir-typespecs.html#basic-types"},{"type":"extras","title":"Literals - Comparison to Plain Typespecs","doc":"| Type                                                           | Supported? | Notes                                                                                                  |\n|----------------------------------------------------------------|------------|--------------------------------------------------------------------------------------------------------|\n| `true`                                                         | ✅         |                                                                                                        |\n| `false`                                                        | ✅         |                                                                                                        |\n| `nil`                                                          | ✅         |                                                                                                        |\n| `<<>>`                                                         | ✅         | empty bitstring                                                                                        |\n| `<<_::size>`                                                   | ✅         | size is 0 or a positive integer                                                                        |\n| `<<_::_*unit>>`                                                | ✅         | unit is an integer from 1 to 256                                                                       |\n| `<<_::size, _::_*unit>>`                                       | ✅         |                                                                                                        |\n| `(-> type)`                                                    | ✅¹        | 0-arity, returns type                                                                                  |\n| `(type1, type2 -> type)`                                       | ✅¹        | 2-arity, returns type                                                                                  |\n| `(... -> type)`                                                | ✅¹        | any arity, returns type                                                                                |\n| `1`                                                            | ✅         | integer                                                                                                |\n| `1..10`                                                        | ✅         | range                                                                                                  |\n| `[type]`                                                       | ✅         | list with any number of type elements                                                                  |\n| `[]`                                                           | ✅         | empty list                                                                                             |\n| `[...]`                                                        | ✅         | shorthand for nonempty_list(any())                                                                     |\n| `[type, ...]`                                                  | ✅         | shorthand for nonempty_list(type)                                                                      |\n| `[key: value_type]`                                            | ✅         | keyword list with key :key of value_type                                                               |\n| `%{}`                                                          | ✅         | empty map                                                                                              |\n| `%{key: value_type}`                                           | ✅         | map with required (atom) key :key of value_type                                                        |\n| `%{key_type => value_type}`                                    | ✅         | map with required pairs of key_type and value_type                                                     |\n| `%{required(key_type) => value_type}`                          | ✅²        | map with required pairs of key_type and value_type                                                     |\n| `%{optional(key_type) => value_type}`                          | ✅²        | map with optional pairs of key_type and value_type                                                     |\n| `%SomeStruct{}`                                                | ✅         | struct with all fields of any type                                                                     |\n| `%SomeStruct{key: value_type}`                                 | ✅         | struct with required key :key of value_type                                                            |\n| `%{key: value_type, optional(opt_key_type) => opt_value_type}` | ✅³        | struct with required key :key of value_type, and zero or more pairs of opt_key_type and opt_value_type |\n| `{}`                                                           | ✅         | empty tuple                                                                                            |\n| `{:ok, type}`                                                  | ✅         | two-element tuple with an atom and any type                                                            |\n\n¹: Functions passed as parameters can only be fully checked once they are called. \nTypeCheck wraps them in a 'wrapper function' which performs the correct check on their input/output.\nThis wrapper will only run once the the function actually is called.\n\n²: Only a single 'required' or 'optional' field in a map is currently supported. (Types which need more are fortunately very rare.)\n\n³: Only optional is currently supported, and only a single one. (Types which need more are fortunately very rare.)","ref":"comparing-typecheck-and-elixir-typespecs.html#literals"},{"type":"extras","title":"Built-in types - Comparison to Plain Typespecs","doc":"| Type                           | Supported? | Notes                                                               |\n|--------------------------------|------------|---------------------------------------------------------------------|\n| term()                         | ✅         | `any()`                                                               |\n| arity()                        | ✅         | `0..255`                                                              |\n| as_boolean(t)                  | ✅         | `t`                                                                   |\n| binary()                       | ✅         | `<<_::_*8>>`                                                          |\n| bitstring()                    | ✅         | `<<_::_*1>>`                                                          |\n| boolean()                      | ✅         | `true \\| false`                                                        |\n| byte()                         | ✅         | `0..255`                                                              |\n| char()                         | ✅         | `0..0x10FFFF`                                                         |\n| charlist()                     | ✅         | `[char()]`                                                            |\n| nonempty_charlist()            | ✅         | `[char(), ...]`                                                       |\n| fun()                          | ✅         | `(... -> any)`                                                        |\n| function()                     | ✅         | `fun()`                                                               |\n| identifier()                   | ✅         | `pid() \\| port() \\| reference()`                                      |\n| iodata()                       | ✅         | `iolist() \\| binary()`                                                |\n| iolist()                       | ✅         | `maybe_improper_list(byte() \\| binary() \\| iolist(), binary() \\| [])` |\n| keyword()                      | ✅         | `[{atom(), any()}]`                                                   |\n| keyword(t)                     | ✅         | `[{atom(), t}]`                                                       |\n| list()                         | ✅         | `[any()]`                                                             |\n| nonempty_list()                | ✅         | `nonempty_list(any())`                                                |\n| maybe_improper_list()          | ✅         | `maybe_improper_list(any(), any())`                                   |\n| nonempty_maybe_improper_list() | ✅         | `nonempty_maybe_improper_list(any(), any())`                          |\n| mfa()                          | ✅         | `{module(), atom(), arity()}`                                         |\n| module()                       | ✅         | `atom()`                                                              |\n| no_return()                    | ✅         | `none()`                                                              |\n| node()                         | ✅         | `atom()`                                                              |\n| number()                       | ✅         | `integer() \\| float()`                                                |\n| struct()                       | ✅         | `%{:__struct__ => atom(), optional(atom()) => any()}`                 |\n| timeout()                      | ✅         | `:infinity \\| non_neg_integer()`                                      |","ref":"comparing-typecheck-and-elixir-typespecs.html#built-in-types"},{"type":"extras","title":"🚀 TypeCheck Additions - Comparison to Plain Typespecs","doc":"TypeCheck adds the following extensions on Elixir's builtin typespec syntax:\n\n\n| Type                      | Notes                                                                    |\n|---------------------------|--------------------------------------------------------------------------|\n| impl(protocol_name)       | Checks whether the given value implements the particular protocol        |\n| fixed_list(element_types) | Fixed size where element_types dictate types                             |\n| tuple(size)               | Any types, but which has exactly size elements                           |\n| map(key_type, value_type) | Zero or more keys of key_type and values of value_type                   |\n| type when guard           | A 'type guard', an extra check implemented in arbitrary code (see below) |","ref":"comparing-typecheck-and-elixir-typespecs.html#typecheck-additions"},{"type":"extras","title":"Defining Specifications - Comparison to Plain Typespecs","doc":"| Type    | Supported? | Notes       |\n|---------|------------|-------------|\n| @type   | ✅         | as @type!   |\n| @opaque | ✅         | as @opaque! |\n| @typep  | ✅         | as @typep!  |\n| @spec   | ✅         | as @spec!   |\n\n✅ **Basic Spec Definition**\n\n```elixir\n# typespecs\n@spec function_name(type1, type2) :: return_type\n\n# TypeCheck\n@spec! function_name(type1, type2) :: return_type\n```\n\n❌ **Spec Guards**\n\n```elixir\n# typespecs\n@spec function(arg) :: [arg] when arg: atom\n@spec function(arg1, arg2) :: {arg1, arg2} when arg1: atom, arg2: integer\n@spec function(arg) :: [arg] when arg: var\n\n# TypeCheck - unsupported\n```\n\nTypeCheck currently solely allows the usage of `when` to specify 'type guards' (see below).\nSupport for spec guards could be added. However:\n- Their usage is quite rare.\n- Each place where it is used, it matches 'exactly the same value'. Building this check is relatively tricky.\n\nIf you have a strong desire for this feature, please open an issue for it.\n\n✅ **Named Arguments**\n\n```elixir\n# typespecs\n@spec days_since_epoch(year :: integer, month :: integer, day :: integer) :: integer\n@type color :: {red :: integer, green :: integer, blue :: integer}\n\n# TypeCheck\n@spec! days_since_epoch(year :: integer, month :: integer, day :: integer) :: integer\n@type! color :: {red :: integer, green :: integer, blue :: integer}\n```\n\n❌ **Specification Overloads**\n\n```elixir\n# typespecs\n@spec function(integer) :: atom\n@spec function(atom) :: integer\n\n# TypeCheck - unsupported\n```\n\nThere is no intention to support this.\nThe implementation would be very difficult, and it is arguably good practice to have a single specification anyway.","ref":"comparing-typecheck-and-elixir-typespecs.html#defining-specifications"},{"type":"extras","title":"User Defined Types - Comparison to Plain Typespecs","doc":"✅ **Basic Definition**\n\n```elixir\n# typespecs\n@type type_name :: type\n@typep type_name :: type\n@opaque type_name :: type\n\n# TypeCheck\n@type! type_name :: type\n@typep! type_name :: type\n@opaque! type_name :: type\n```\n\n✅ **Parameterized Types**\n\n```elixir\n# typespecs\n@type dict(key, value) :: [{key, value}]\n\n# TypeCheck\n@type! dict(key, value) :: [{key, value}]\n```\n\n🚀 **Type Guards**\n\nTo add extra custom checks to a type, you can use a so-called 'type guard'. This is arbitrary code that is executed during a type-check once the type itself already matches.\n\nYou can use \"named types\" to refer to (parts of) the value that matched the type, and refer to these from a type-guard:\n\n```elixir\n# typespecs - unsupported\n\n# TypeCheck\n@type! sorted_pair :: {lower :: number(), higher :: number()} when lower <= higher\n```","ref":"comparing-typecheck-and-elixir-typespecs.html#user-defined-types"},{"type":"extras","title":"✅ Remote Types - Comparison to Plain Typespecs","doc":"It is often useful to refer to types defined in other modules. These are called 'Remote types'.\nElixir's typespecs and TypeCheck both support remote types.\n\n```elixir\n# typespecs\ndefmodule User do\n  @type t() :: %User{name: String.t(), age: non_negative_integer()}\nend\n\ndefmodule Greeter do\n  @spec greet(User.t()) :: String.t()\n  def greet(user) do\n   # ... \n  end\nend\n\n# TypeCheck\ndefmodule User do\n  use TypeCheck\n\n  @type! t() :: %User{name: String.t(), age: non_negative_integer()}\nend\n\ndefmodule Greeter do\n  use TypeCheck\n\n  @spec! greet(User.t()) :: String.t()\n  def greet(user) do\n   # ... \n  end\nend\n```","ref":"comparing-typecheck-and-elixir-typespecs.html#remote-types"},{"type":"extras","title":"Remote Type Overrides - Comparison to Plain Typespecs","doc":"From time to time we need to interface with modules written in other libraries (or the Elixir standard library) which do not expose their types through TypeCheck yet.\nWe want to be able to use those types in our checks, but they exist in modules that we cannot change ourselves.\n\nThe solution is to allow a list of ‘type overrides’ to be given as part of the options passed to use TypeCheck, which allow you to use the original type in your types and documentation, but have it be checked (and potentially property-generated) as the given TypeCheck-type.\n\nExample:\n\n```elixir\ndefmodule Original do\n  @type t() :: any()\nend\n\ndefmodule Replacement do\n  use TypeCheck\n  @type! t() :: integer()\nend\n\ndefmodule Example do\n  use TypeCheck, overrides: [{&Original.t/0, &Replacement.t/0}]\n\n  @spec! times_two(Original.t()) :: integer()\n  def times_two(input) do\n    input * 2\n  end\nend\n```","ref":"comparing-typecheck-and-elixir-typespecs.html#remote-type-overrides"},{"type":"extras","title":"Elixir Standard Library Types - Comparison to Plain Typespecs","doc":"TypeCheck helpfully ships with the majority of the types in Elixir's Standard Library already implemented as default overrides. This means that your `@spec!` definitions can reference types like `String.t()`, `Date.t()` and `Range.t()` out of the box.\n\n\n| Type                      | Supported? | Notes      |\n|---------------------------|------------|------------|\n| Access                    | ✅         |            |\n| Agent                     | ✅         |            |\n| Application               | ✅         |            |\n| Calendar                  | ✅         |            |\n| Calendar.ISO              | ✅         |            |\n| Calendar.TimeZoneDatabase | ✅         |            |\n| Code                      | ✅         |            |\n| Collectable               | ✅         |            |\n| Config.Provider           | ✅         |            |\n| Date                      | ✅         |            |\n| Date.Range                | ✅         |            |\n| DateTime                  | ✅         |            |\n| Dict                      | ❌         | deprecated |\n| DynamicSupervisor         | ✅         |            |\n| Enum                      | ✅         |            |\n| Enumerable                | ✅         |            |\n| Exception                 | ✅         |            |\n| File                      | ✅         |            |\n| File.Stat                 | ✅         |            |\n| File.Stream               | ✅         |            |\n| Float                     | ✅         |            |\n| Function                  | ✅         |            |\n| GenEvent                  | ❌         | deprecated |\n| GenServer                 | ✅         |            |\n| HashDict                  | ❌         | deprecated |\n| HashSet                   | ❌         | deprecated |\n| IO                        | ✅         |            |\n| IO.ANSI                   | ✅         |            |\n| IO.Stream                 | ✅         |            |\n| Inspect                   | ✅         |            |\n| Inspect.Algebra           | ✅         |            |\n| Inspect.Opts              | ✅         |            |\n| Keyword                   | ✅         |            |\n| List.Chars                | ✅         |            |\n| Macro                     | ✅         |            |\n| Macro.Env                 | ✅         |            |\n| Map                       | ✅         |            |\n| MapSet                    | ✅         |            |\n| NaiveDateTime             | ✅         |            |\n| Node                      | ✅         |            |\n| OptionParser              | ✅         |            |\n| Path                      | ✅         |            |\n| Port                      | ✅         |            |\n| Process                   | ✅         |            |\n| Range                     | ✅         |            |\n| Regex                     | ✅         |            |\n| Registry                  | ✅         |            |\n| Set                       | ❌         | deprecated |\n| Stream                    | ✅         |            |\n| String                    | ✅         |            |\n| String.Chars              | ✅         |            |\n| Supervisor                | ✅         |            |\n| Supervisor.Spec           | ❌         | deprecated |\n| System                    | ✅         |            |\n| Task                      | ✅         |            |\n| Task.Supervisor           | ✅         |            |\n| Time                      | ✅         |            |\n| URI                       | ✅         |            |\n| Version                   | ✅         |            |\n| Version.Requirement       | ✅         |            |","ref":"comparing-typecheck-and-elixir-typespecs.html#elixir-standard-library-types"},{"type":"extras","title":"Comparing TypeCheck and Norm","doc":"","ref":"comparing-typecheck-and-norm.html"},{"type":"extras","title":"Norm - Comparing TypeCheck and Norm","doc":"[Norm](https://github.com/keathley/norm/) is an Elixir library for specifying the structure of data that can be used for both validation and data-generation.\n\nOn a superficial level, Norm and TypeCheck seem similar. However, there are important differences in their design considerations:\n\n#### Primary Focus \n\nNorm focuses on conforming values to specifications by re-using your existing validations. \nNorm also has a focus on 'open' schemas that are designed to allow systems to grow over time.\n\nTypeCheck focuses on making the types your program is using explicit. \nThis is done by retrofitting Elixir's built-in type syntax to allow you to use a single statement to create \n1. an expanded type-/function-specification for in the documentation, \n2. runtime type-checking code and\n3. data-generation for property-testing.\n\n#### Syntax\n\nNorm uses a new, guard-like function-call syntax.\nNorm is light on the little syntactic sugar: Literal atoms and tuples containing specs as elements are treated as specs themselves. \nAnything else requires you to write a (dedicated or anonymous) validation function.\n\nTypeCheck uses the same syntax that Elixir's built-in typespecs use, which is already familiar to many Elixir developers.\nThis makes a TypeCheck `@type!` or `@spec!` often much shorter than the equivalent Norm `spec` or `@contract`.\n\n##### A couple of syntactical examples:\n\nA simple 'manual' validation. (This is common Norm usage but manual validations are more rare in TypeCheck.)\n\n```elixir\n# Norm:\niex> Norm.conform!(123, spec(is_integer() and &(&1 > 0)))\n123\n```\n\n```elixir\n# TypeCheck\niex> TypeCheck.conforms!(123, non_neg_integer())\n# or:\niex> TypeCheck.conforms!(123, x :: integer() when x >= 0)\n```\n\nDefining custom type-specifications ('specs' in Norm parlance) and function-specifications ('contracts' in Norm parlance):\n\n```elixir\n# Norm:\ndefmodule Color do\n  import Norm\n  def rgb(), do: spec(is_integer() and &(&1 in 0..255))\n  def hex(), do: spec(is_binary() and &String.starts_with?(&1, \"#\"))\n  \n  @contract rgb_to_hex(r :: rgb(), g :: rgb(), b :: rgb()) :: hex()\n  def rgb_to_hex(r, g, b) do\n    # ...\n  end\nend\n```\n\n```elixir\n# TypeCheck:\ndefmodule Color do\n  use TypeCheck\n  @type! rgb :: 0..255\n  @type! hex :: (str :: binary() when String.starts_with?(str, \"#\"))\n  \n  @spec! rgb_to_hex(rgb(), rgb(), rgb()) :: hex()\n  def rgb_to_hex(r, g, b) do\n    # ...\n  end\nend\n```\n\nDefining a more complicated specification of a custom structure with multiple fields:\n\n```elixir\n# Norm:\ndefmodule User do\n  use Norm\n\n  defstruct [:name, :age]\n  def age_spec(), do: spec(is_integer() and &(&1 >= 0))\n  def s() do\n    schema(%{\n      name: spec(is_binary()),\n      age: age_spec(),\n    })\n  end\n  \n  @contract new(name :: spec(is_binary()), age :: age_spec()) :: s()\n  def new(name, age) do\n    %__MODULE__{name: name, age: age}\n  end\n  \n  @contract ensure_old_enough(user :: s(), limit :: age_spec()) :: alt(success: {:ok, s()}, problem: {:error, spec(is_binary())})\n  def ensure_old_enough(user, limit) do\n    if user.age >= limit do\n      {:ok, user}\n    else\n      {:error, \"not old enough\"}\n    end\n  end\nend\n```\n\n```elixir\n# TypeCheck:\ndefmodule User do\n  use TypeCheck\n  defstruct [:name, :age]\n  @type! age :: non_neg_integer()\n  @type! t :: %User{name: binary(), age: age()}\n\n  @spec! new(binary(), age()) :: t()\n  def new(name, age) do\n    %User{name: name, age: age}\n  end\n\n  @spec! ensure_old_enough(t(), age()) :: {:ok, t()} | {:error, binary()}\n  def ensure_old_enough(user, limit) do\n    if user.age >= limit do\n      {:ok, user}\n    else\n      {:error, \"not old enough\"}\n    end\n  end\nend\n```\n\n#### Error messages\n\nNorm does not particularly focus on readable error messages (although that might change in the future).\n\nTypeCheck heavily focuses on creating humanly-readable error-messages when a value does not type-check,\ncreating a deeply nested error message indicating the cause of a top-level error (see the main README for some examples of this). \nTypeCheck's error messages are based on a pluggable formatter for which custom alternatives can be provided.\n\nAs an example, consider the execution using the respective definitions of `User.ensure_old_enough` above:\n\n```elixir\nUser.ensure_old_enough(%User{name: \"Marten\", age: 0.5}, 21)\n** (Norm.MismatchError) Could not conform input:\nval: 0.5 in: :age fails: is_integer()\n    (norm 0.13.0) lib/norm.ex:65: Norm.conform!/2\n    (norm_example 0.1.0) lib/norm_example.ex:38: anonymous fn/2 in User.ensure_old_enough/2\n    (elixir 1.12.0) lib/enum.ex:2356: Enum.\"-reduce/3-lists^foldl/2-0-\"/3\n    (norm_example 0.1.0) lib/norm_example.ex:38: User.ensure_old_enough/2\n```\n\n```elixir\n** (TypeCheck.TypeError) At lib/type_check_example.ex:21:\nThe call to `ensure_old_enough/2` failed,\nbecause parameter no. 1 does not adhere to the spec `%User{age: non_neg_integer(), name: binary()}`.\nRather, its value is: `%User{age: 0.5, name: \"Marten\"}`.\nDetails:\n  The call `ensure_old_enough(%User{age: 0.5, name: \"Marten\"}, 21)`\n  does not adhere to spec `ensure_old_enough(%User{age: non_neg_integer(), name: binary()}, non_neg_integer())\n::\n{:ok, %User{age: non_neg_integer(), name: binary()}} | {:error, binary()}`. Reason:\n    parameter no. 1:\n      `%User{age: 0.5, name: \"Marten\"}` does not check against `%User{age: non_neg_integer(), name: binary()}`. Reason:\n        under key `:age`:\n          `0.5` is not a non-negative integer.\n    (type_check_example 0.1.0) lib/type_check/spec.ex:165: User.ensure_old_enough/2\n\n```\n\nAnother example, displaying the difference between error messages when the problem is in the value returned by the function:\n\n```elixir\n# Given the `rgb_to_hex` function as defined above, but with an implementation not adhering to its spec/contract\n  def rgb_to_hex(r, g, b) do\n    {:ok, 42}\n  end\n```\n\n```elixir\niex> Color.rgb_to_hex(10, 20, 30)\n** (Norm.MismatchError) Could not conform input:\nval: {:ok, 42} fails: is_binary()\n    (norm 0.13.0) lib/norm.ex:65: Norm.conform!/2\n\n```\n\n```elixir\niex> Color.rgb_to_hex(10, 20, 30)\n** (TypeCheck.TypeError) The call to `rgb_to_hex/3` failed,\nbecause the returned result does not adhere to the spec `(str :: binary() when String.starts_with?(str, \"#\"))`.\nRather, its value is: `{:ok, 42}`.\nDetails:\n  The result of calling `rgb_to_hex(10, 20, 30)`\n  does not adhere to spec `rgb_to_hex(0..255,  0..255,  0..255) :: (str :: binary() when String.starts_with?(str, \"#\"))`. Reason:\n    Returned result:\n      `{:ok, 42}` does not check against `(str :: binary() when String.starts_with?(str, \"#\"))`. Reason:\n        `{:ok, 42}` does not check against `str :: binary()`. Reason:\n          `{:ok, 42}` is not a binary.\n    (type_check_example 0.1.0) lib/type_check/spec.ex:194: Color.rgb_to_hex/3\n\n```\n\n\n#### Execution & Efficiency\n\nIn Norm, while wrapping functions with a contract happens at compile-time, \nall contracts and specs are resolved at runtime.\nThis makes Norm's internals less metaprogramming-heavy and easily allows specs to be created and manipulated dynamically at runtime, \nbut it does mean that the compiler is not able to optimize the type-checking code at all, and specs are re-evaluated every time a function is called.\n\nTypeCheck requires¹ types to be defined at compile-time and injects the type-checking code into your functions and modules before they are compiled, \nallowing type-checking to be optimized.\nIf there is overlap between the parts of your parameters being checked by TypeCheck and the logic of your function,\nthe BEAM compiler will in most cases be able to combine these into a single check.\n\n¹: In normal usage. Escape hatches to work with types defined at runtime exist. (c.f. `TypeCheck.dynamic_conforms/2` and variants)\n\n#### Documentation\n\nNorm does not focus on documentation.\nNorm's `spec`s are normal functions which you can document manually using `@doc` if you wish. \nNorm's `@contract`s are not used for documentation purposes.\n\nTypeCheck adds `@type`/`@typep`/`@opaque` attributes for the types you specify, making them show up in your documentation \nand allowing you to use the same type definitions for tools like `Dialyzer`. \nYou can also use the `t` helper to look them up in IEx. \nDocumentation can be added to these types by using `@typedoc` (just like for normal typespecs).\n\nFunction-specifications created with TypeCheck will also add `@spec`-attributes, which will end up in the documentation of your functions and are similarly useful for e.g. `Dialyzer`.\n\n#### Data Generation & Testing\n\nIt is very useful to generate examples of good data to be used for property testing.\nBoth Norm and TypeCheck have this capability, by using `:stream_data` as an optional dependency.\n\nNorm's generators (only) work when the first predicate in a `spec(...)` is one of (a subset of) Elixir's built-in guard-clauses. \nIf your spec is too restrictive, you'll have to manually provide a custom data generator as well.\n\nTypeCheck builds more complicated generators out of simple ones just as it builds complicated types out of simple ones. \nThis means that virtually all Elixir types can be turned into generators without extra effort of the user. \nIt is only when 'type guards' are used to add arbitrary checks to a type that you might up with a generator that is too restrictive. \nTypeCheck supports nearly all of Elixir's builtin types, as well as many of the remote types that are part of Elixir's standard library (`Range.t`, `MapSet.t`, `List.t`, `Enum.t` etc.)\n\nFurthermore, TypeCheck allows overriding the generator for a type (c.f. the `TypeCheck.Type.StreamData.wrap_with_gen/2` macro).\n\nFinally and maybe most importantly, TypeCheck ships with the `spectest` macro. (c.f. `TypeCheck.ExUnit.spectest/2`)\n`spectest` works similarly to `doctest` but rather than testing all code snippets in the module's documentation,\nit will automatically run a property-test to check for each `@spec!`-ced function in a module, whether it correctly follows its spec.\n\n\n\n---\n\nNorm and TypeCheck are but two different dots in the datastructure-validation design space. Norm is definitely worth checking out!","ref":"comparing-typecheck-and-norm.html#norm"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}